;******************************************************************************
;                                                                             *
;                                                         *
;                                                                             *
;******************************************************************************
;                                                                             *
;    Filename:		FZG_Einfach_P18F2550.asm                                  *
;    Date:			18.08.2020                                                *
;    File Version:	1.0			                                              *
;    Author:		Werner Hirsch                                         	  *
;	 Internet:		www.wernerhirsch.de                                       *
;    e-mail:		wernerhirschhh@aol.com                                    *
;                                                                             *
;******************************************************************************
;                                                                             *
;    Notes:                                                                   *
;                                                                             *
;                                                                             *
;******************************************************************************
;                                                                             *
;    Revision History:                                                        *
;                                                                             *
;******************************************************************************

;------------------------------------------------------------------------------
; PROCESSOR DECLARATION
;------------------------------------------------------------------------------

     LIST      P=PIC18F2550          ; list directive to define processor
     #INCLUDE <P18F2550.INC>         ; processor specific variable definitions

	;Setup CONFIG1L
	CONFIG PLLDIV = 1, CPUDIV = OSC1_PLL2, USBDIV = 1
	;Setup CONFIG1H
	CONFIG FOSC = XTPLL_XT, FCMEN = OFF, IESO = OFF
	;Setup CONFIG2L
	CONFIG PWRT = OFF, BOR = OFF, BORV = 2, VREGEN = OFF
	;Setup CONFIG2H
	CONFIG WDT = OFF, WDTPS = 1
	;Setup CONFIG3H
	CONFIG CCP2MX = ON, PBADEN = OFF, LPT1OSC = OFF, MCLRE = OFF
	;Setup CONFIG4L
	CONFIG STVREN = OFF, LVP = OFF, XINST = OFF, DEBUG = OFF
	;Setup CONFIG5L
	CONFIG CP0 = OFF, CP1 = OFF, CP2 = OFF, CP3 = OFF
	;Setup CONFIG5H
	CONFIG CPB = OFF, CPD = OFF
	;Setup CONFIG6L
	CONFIG WRT0 = OFF, WRT1 = OFF, WRT2 = OFF, WRT3 = OFF
	;Setup CONFIG6H
	CONFIG WRTC = OFF, WRTB = OFF, WRTD = OFF
	;Setup CONFIG7L
	CONFIG EBTR0 = OFF, EBTR1 = OFF, EBTR2 = OFF, EBTR3 = OFF
	;Setup CONFIG7H
	CONFIG EBTRB = OFF

;------------------------------------------------------------------------------
;
; VARIABLE DEFINITIONS
;
; Available data memory (also RAM) address space is divided into 16 banks, of 
; which 9 may be addressed.  The Access Bank, Special Function Registers, 
; and and General Purpose Registers are shown below:
; 
; ACCESS LOW         0x000-0x05F   Bank 0
; GPR0               0x060-0x0FF   Bank 0
; GPR1               0x100-0x1FF   Bank 1
; GPR2               0x200-0x2FF   Bank 2
; GPR3               0x300-0x3FF   Bank 3
; GPR4               0x400-0x4FF   Bank 4
; GPR5               0x500-0x5FF   Bank 5
; GPR6               0x600-0x6FF   Bank 6
; GPR7               0x700-0x7FF   Bank 7
; GPR8               0x800-0x8FF   Bank 8
; GPR9               0x900-0x9FF   Bank 9
; GPR10              0xA00-0xAFF   Bank 10
; GPR11              0xB00-0xBFF   Bank 11
; GPR12              0xC00-0xCFF   Bank 12
; GPR13              0xD00-0xDFF   Bank 13
; GPR14              0xE00-0xEFF   Bank 14
; GPR15              0xF00-0xF5F   Bank 15
; ACCESS HIGH (SFR)  0xF60-0xFFF   Bank 15
;
;------------------------------------------------------------------------------
;==============================
;  DEFINITIONEN
;==============================
#define IN_VCO			PORTA, 0		;PIN RA0 (pin 2) Analogeingang fr VCO Spannung
#define CS_VOL			PORTA, 1		;PIN RA1 (pin 3) Chip Select Digitalpoti
#define SI_CLK			PORTA, 2		;PIN RA2 (pin 4) Serieller Takt fr Display, DAC und Poti
#define CS_DAC			PORTA, 3		;PIN RA3 (pin 5) Ausgang CS DAC 
#define IN_COUNTF		PORTA, 4		;PIN RA4 (pin 6) Zhler Eingang Frequenz
#define ME_FILT			PORTA, 5		;PIN RA5 (pin 7) Ausgang Messeingang und Filter
#define OUT_DAC_0		PORTB, 0		;Pin RB0 (pin 21) Ausgang DAC, DB0
#define OUT_DAC_1		PORTB, 1		;Pin RB1 (pin 22) Ausgang DAC, DB1
#define OUT_DAC_2		PORTB, 2		;Pin RB2 (pin 23) Ausgang DAC, DB2
#define OUT_DAC_3		PORTB, 3		;Pin RB3 (pin 24) Ausgang DAC, DB3
#define OUT_DAC_4		PORTB, 4		;Pin RB4 (pin 25) Ausgang DAC, DB4
#define OUT_DAC_5		PORTB, 5		;Pin RB5 (pin 26) Ausgang DAC, DB5
#define OUT_DAC_6		PORTB, 6		;Pin RB6 (pin 27) Ausgang DAC, DB6
#define OUT_DAC_7		PORTB, 7		;Pin RB7 (pin 28) Ausgang DAC, DB7
#define DISP_EN			PORTC, 0		;PIN RC0 (pin 11) Mit der negativen Flanke von DISP_EN werden die parallelen Daten bernommen
#define DISP_RS			PORTC, 1		;PIN RC1 (pin 12) Unterscheidung zwischen Befehl und Daten
#define IN_COUNTP		PORTC, 2		;PIN RC2 (pin 13) Zhler Eingang Periode 
#define TA_DB0			PORTC, 4		;PIN RC4 (pin 15) Input only, Tasten DB0
#define TA_DB1			PORTC, 5		;PIN RC5 (pin 16) Input only, Tasten DB1
#define TA_DB2			PORTC, 6		;PIN RC6 (pin 17) Tasten DB2
#define SI_DAT			PORTC, 7		;PIN RC7 (pin 18) Serielle Daten zum Display, DAC und Poti
#define TA_MENUE		PORTE, 3		;Pin RE3 (pin 1) Input only, Eingang "Men"


;Konstanten
HIGH_BYTE_S		EQU	0xB8		;Konstanten fr Sekundenzhler TMR1: 1 Sek = 12.000.000 Takte = 183 x 2^16 + 27 x 2^8
LOW_BYTE_S		EQU	0xE5		;H-Byte = 183+1 => 0xB8, L-Byte = 256-27 = 229 => 0xE5

;Variablen
W_TEMP			EQU 0x000		; w register for context saving (ACCESS)
STATUS_TEMP		EQU 0x001		; status used for context saving 
BSR_TEMP		EQU 0x002		; bank select used for ISR context saving
PCLATH_TEMP		EQU 0x003		; pclath used for ISR context saving

COUNT_H_S		EQU	0x060		;Zhler fr Loops Timer 1
COUNT_ADJ		EQU	0x061		;Zhler fr Feinabgleich Timer 1
WOB_ZEIT_L		EQU	0x062		;Register fr Feinabgleich Timer 1
WOB_ZEIT_M		EQU	0x063		;Register fr Wobble, Zeit/Schritt Zhler: Low Byte, Loop n
T1_ZEIT_H		EQU	0x064		;Register fr Wobble, Zeit/Schritt Zhler: High Byte
MESS_END		EQU	0x065		;Status Register: 	Bit 0: Sekundentakt
								;					Bit 1:
								;					Bit 2:
								;					Bit 3:
STAT_INT		EQU	0x066		;Statusregister fr Interruptroutinen
LOOP_1			EQU 0x067		;Zhler fr Wartezeit
LOOP_2			EQU	0x068		;Zhler fr Wartezeit
LOOP_3			EQU	0x069		;Zhler fr Wartezeit
SHIFT_REG		EQU	0x06A		;Register fr Daten, die an das Display, das Schiebereg. oder den DAC gesendet werden sollen
SHIFT_CNT		EQU	0x06B		;Zhler fr die serielle Datenausgabe (Display, Schiebereg., DAC)
PRESC_TMR0		EQU 0x06C		;Zhler fr Anzahl Multiplikationen x2 bei Prescaler TMR0
DISP_PRESC		EQU 0x06D		;Display Register fr Prescaler Wert
PROGRAM			EQU 0x06E		;Register Programm: 0=Frequenz,1=Periode,2=Zhler,3=DDS,4=VCO,5=Wobbel
EINGANG			EQU	0x06F		;Register fr Messeingang: 0 = TTL, 1 = Analog
KENNLIN			EQU	0x070		;Register fr Kennlinie VCO / Wobble, 0 = Linear, 1 = Logarithmisch
DEKADE			EQU	0x071		;Register fr Anzahl Dekaden VCO / Wobble, => 1 - 3 Dekaden
DDS_VOL			EQU 0x072		;Register fr DDS Volumen (0-7 entspricht 0dB bis -42dB)
DDS_FILTER		EQU	0x073		;Filter Ein/Aus (1/0)
DDS_WAVEFORM	EQU	0x074		;Register fr Wellenform:  0=Sinus, 1=Dreieck, 2=Sgezahn, 3=Rechteck
ZAHL_A0			EQU	0x075		;Zahl A Bits 0-7
ZAHL_A1			EQU	0x076		;Zahl A Bits 8-15
ZAHL_A2			EQU	0x077		;Zahl A Bits 16-23
ZAHL_A3			EQU	0x078		;Zahl A Bits 24-31
ZAHL_B0			EQU	0x079		;Zahl B Bits 0-7
ZAHL_B1			EQU	0x07A		;Zahl B Bits 8-15
ZAHL_B2			EQU	0x07B		;Zahl B Bits 16-23
ZAHL_B3			EQU	0x07C		;Zahl B Bits 24-31
ZAHL_C0			EQU	0x07D		;Zahl C Bits 0-7
ZAHL_C1			EQU	0x07E		;Zahl C Bits 8-15
ZAHL_C2			EQU	0x07F		;Zahl C Bits 16-23
ZAHL_C3			EQU	0x080		;Zahl C Bits 24-31
ZAHL_C4			EQU	0x081		;Zahl C Bits 32-39
ZAHL_C5			EQU	0x082		;Zahl C Bits 40-47
ZAHL_C6			EQU	0x083		;Zahl C Bits 48-55
ZAHL_C7			EQU	0x084		;Zahl C Bits 56-63
TEMP_0			EQU	0x085		;Allgemeines Zwischenspeicherregister
TEMP_1			EQU	0x086		;Allgemeines Zwischenspeicherregister
TEMP_2			EQU	0x087		;Allgemeines Zwischenspeicherregister
TEMP_3			EQU	0x088		;Allgemeines Zwischenspeicherregister
TEMP_10			EQU	0x089		;Allgemeines Zwischenspeicherregister
TEMP_11			EQU	0x08A		;Allgemeines Zwischenspeicherregister
TEMP_12			EQU	0x08B		;Allgemeines Zwischenspeicherregister
TEMP_13			EQU	0x08C		;Allgemeines Zwischenspeicherregister
TEMP_14			EQU	0x08D		;Allgemeines Zwischenspeicherregister
TEMP_15			EQU	0x08E		;Allgemeines Zwischenspeicherregister
TEMP_16			EQU 0x08F		;Allgemeines Zwischenspeicherregister
DISP_0			EQU	0x090		;Display Nachkommastelle oder Einer
DISP_1			EQU	0x091		;Display Einer oder Zehner
DISP_2			EQU	0x092		;Display Zehner oder Hunderter
DISP_3			EQU	0x093		;Display Hunderter oder Tausender
DISP_4			EQU	0x094		;Display Tausender oder Zehntausender
DISP_5			EQU	0x095		;Display Zehntausender oder Hunderttausender
DISP_6			EQU	0x096		;Display Hunderttausender oder Millionen
DISP_7			EQU	0x097		;Display Millionen oder Zehnmillionen
DISP_8			EQU	0x098		;Display Zehnmillionen oder Hundertmillionen
BEFEHL			EQU 0x099		;Register fr Tastaturbefehle
TEMP_DIG_CNT	EQU 0x09A		;Zhler fr Menposition horizontal
TEMP_CNT_MAX	EQU 0x09B		;Max Zhlerwert fr Menposition horizontal
TEMP_DIG_STAT	EQU 0x09C		;Status Register	bit 0: wenn 1: Update der Anzeige des zu ndernden Wertes
								;					bit 1: wenn 1: Erhhung des zu ndernden Wertes
								;					bit 2: wenn 1: Reduzierung des zu ndernden Wertes
MENU_COUNT		EQU	0x09D		;Zhler fr Menposition
MENU_COUNT_MAX	EQU	0x09E		;Maximale Menposition
MENU_STAT		EQU	0x09F		;Status Register fr Men	bit 0: Men 1.Stufe
								;							bit 1: Men 2.Stufe Messen
								;							bit 2: Men 2.Stufe DDS Generator
								;							bit 3: Men 2.Stufe Einstellungen
								;							bit 5: Display Update Bit fr Eingang und DDS Einstellungen
								;							bit 6: Men 2.Stufe Einstellungen, Update Display
								;							bit 7: Men 1.Stufe, Update Display
UNIT			EQU	0x0A0		;Register fr Einheit k = kilo, m = milli, u = micro, n = nano
KOMMA			EQU	0x0A1		;Register fr Kommaposition der Displayanzeige
AD_DA_L			EQU	0x0A2		;Register fr 10 Bit AD/DA Wert VCO / Wobble, Low Byte
AD_DA_H			EQU	0x0A3		;Register fr 10 Bit AD/DA Wert VCO / Wobble, High Byte
AD_DA_REST		EQU	0x0A4		;Register fr Rest AD/DA Wert VCO / Wobble, log
DDS_FREQ_0		EQU 0x0A5		;Register fr DDS Frequenz (x10) 0 (24Bit, Bits 0-7)
DDS_FREQ_1		EQU	0x0A6		;Register fr DDS Frequenz (x10) 1 (24Bit, Bits 8-15)
DDS_FREQ_2		EQU	0x0A7		;Register fr DDS Frequenz (x10) 2 (24Bit, Bits 16-23)
DDS_PHASE_L		EQU	0x0A8		;Register fr DDS Phase (Winkel x10), Low Byte (16Bit, Bits 0-7)
DDS_PHASE_H		EQU	0x0A9		;Register fr DDS Phase (Winkel x10), High Byte (16Bit, Bits 8-15)
DDS_PERIODE_L	EQU 0x0AA		;Register fr DDS MCLK 0 (24Bit, Bits 0-7)
DDS_PERIODE_M	EQU	0x0AB		;Register fr DDS MCLK 1 (24Bit, Bits 8-15)
DDS_PERIODE_H	EQU	0x0AC		;Register fr DDS MCLK 2 (24Bit, Bits 16-23)
DDS_FREQREG_0	EQU	0x0AD		;Frequenzregister 0 (24Bit, Bits 0-7)
DDS_FREQREG_1	EQU	0x0AE		;Frequenzregister 1 (24Bit, Bits 8-15)
DDS_FREQREG_2	EQU	0x0AF		;Frequenzregister 2 (24Bit, Bits 16-23)
DDS_PHASEREG_L	EQU	0x0B0		;Phasenregister, Low Byte (16Bit, Bits 0-7)
DDS_PHASEREG_H	EQU	0x0B1		;Phasenregister, High Byte (16Bit, Bits 8-15)
DAC_L			EQU	0x0B2		;Register fr DAC Daten Low Byte
DAC_H			EQU	0x0B3		;Register fr DAC Daten High Byte
DAC_GAIN		EQU	0x0B4		;Register fr DAC Verstrkung:  1 = 0-2048mV, 0 = 0-4096mV
DDS_WAVE		EQU	0x0B5		;Register fr Wellenform:  Bit0=Sinus, Bit1=Sinus Low, Bit2=Dreieck, Bit3=Sgezahn, Bit4=Rechteck
WOB_FU_L		EQU	0x0B6		;Frequenzregister fr Wobble, Fu L-Byte
WOB_FU_M		EQU	0x0B7		;Frequenzregister fr Wobble, Fu M-Byte
WOB_FU_H		EQU	0x0B8		;Frequenzregister fr Wobble, Fu H-Byte
WOB_M_A			EQU	0x0B9		;Register fr Wobble, Frequenzerhhung: Bit0: 0=Auto, 1=Man, Bit1: 0=Kont, 1=Burst
WOB_STEP		EQU	0x0BA		;Register fr Wobble, Schritte/Durchlauf: 1=1024, 2=512, 3=256, 4=128, 5=64, 6=32
WOB_STEPW		EQU	0x0BB		;Register fr Wobble, Schrittweite: 1=1024, 2=512, 4=256, 8=128, 16=64, 32=32
WOB_ZEIT		EQU	0x0BC		;Register fr Wobble, Zeit/Schritt: 0=1ms, 1=2ms, 2=5ms, 3=10ms, 4=20ms, 
								;   5=50ms, 6=100ms, 7=200ms, 8=500ms, 9=1000ms, 10=2000ms
T1_ZEIT_L1		EQU	0x0BD		;Register fr Wobble, Zeit/Schritt Zhler: Low Byte, Loop n-1
T1_ZEIT_L		EQU	0x0BE		;Register fr Wobble, Zeit/Schritt Zhler: Low Byte, Loop n
T1_ZEIT_LOOP	EQU	0x0BF		;Register fr Wobble, Zeit/Schritt Zhler: Anzahl Loops
WOB_COUNT_L		EQU	0x0C0		;Register fr Wobble, Schritt Zhler: Low Byte
WOB_COUNT_H		EQU	0x0C1		;Register fr Wobble, Schritt Zhler: High Byte
WOB_BURST		EQU	0x0C2		;Register fr VCO/Wobble, kontinuierlich = 0, Burst = 1
FD1DDS_0		EQU	0x0C3		;Register fr VCO/Wobble, Frequenzregister Dekade1: 24Bit, Bits 0-7
FD1DDS_1		EQU	0x0C4		;Register fr VCO/Wobble, Frequenzregister Dekade1: 24Bit, Bits 8-15
FD1DDS_2		EQU	0x0C5		;Register fr VCO/Wobble, Frequenzregister Dekade1: 24Bit, Bits 16-23
FD2DDS_0		EQU	0x0C6		;Register fr VCO/Wobble, Frequenzregister Dekade2: 24Bit, Bits 0-7
FD2DDS_1		EQU	0x0C7		;Register fr VCO/Wobble, Frequenzregister Dekade2: 24Bit, Bits 8-15
FD2DDS_2		EQU	0x0C8		;Register fr VCO/Wobble, Frequenzregister Dekade2: 24Bit, Bits 16-23
FD3DDS_0		EQU	0x0C9		;Register fr VCO/Wobble, Frequenzregister Dekade3: 24Bit, Bits 0-7
FD3DDS_1		EQU	0x0CA		;Register fr VCO/Wobble, Frequenzregister Dekade3: 24Bit, Bits 8-15
FD3DDS_2		EQU	0x0CB		;Register fr VCO/Wobble, Frequenzregister Dekade3: 24Bit, Bits 16-23
DEK_AKTUELL		EQU	0x0CC		;Register fr VCO/Wobble, Aktuelle Dekade
DELTAP_L		EQU	0x0CD		;Register fr VCO/Wobble, 
DELTAP_M		EQU	0x0CE		;Register fr VCO/Wobble, 
DELTAP_H		EQU	0x0CF		;Register fr VCO/Wobble, 
TBURST_H		EQU	0x0D0		;Register fr VCO/Wobble, Burstlnge H-byte (Timer0 Erweiterung)
DDS_FREQ_W0		EQU	0x0D1		;DDS Frequenz-/Arbeitsregister Byte 0 (Interruptroutine)
DDS_FREQ_W1		EQU	0x0D2		;DDS Frequenz-/Arbeitsregister Byte 1 (Interruptroutine)
DDS_FREQ_W2		EQU	0x0D3		;DDS Frequenz-/Arbeitsregister Byte 2 (Interruptroutine)
INT_TEMP_0		EQU	0x0D4		;Temp Register Interrupt Routinen
INT_TEMP_1		EQU	0x0D5		;Temp Register Interrupt Routinen
POTI_L			EQU	0x0D6		;Serielle Daten Digitalpoti, L-Byte
POTI_H			EQU	0x0D7		;Serielle Daten Digitalpoti, L-Byte
TEMP_18			EQU	0x0D8
CONTRAST		EQU	0x0D9		;Kontrastwert fr Display

;------------------------------------------------------------------------------
; EEPROM INITIALIZATION
;
; The 18F26K20 has 1024 bytes of non-volatile EEPROM starting at 0xF00000
; 
;------------------------------------------------------------------------------

DATAEE    ORG  0xF00000 		; Starting address for EEPROM for 18F2550

;Variable Starteinstellungen
		de		0x05,0x00		;Adresse 0x00,	Programm: 0=Frequenz,1=Periode,2=Zhler,3=DDS,4=VCO,5=Wobbel
								;Adresse 0x01,	Messeingang: 0 = TTL, 1 = Analog
;Variable Starteinstellungen DDS
		de		0x00, 0x00		;Adresse 0x02,	DDS Wellenform: 0=Sinus, 1=Dreieck, 2=Sgezahn, 3=Rechteck
								;Adresse 0x03,	DDS Ausgangspegel: 0-63 entspricht 0dB bis -63dB
		de		0x10, 0x27		;Adresse 0x04,	DDS Frequenz (x10): Byte 0 (LSB)
								;Adresse 0x05,	DDS Frequenz (x10): Byte 1
		de		0x00, 0x00		;Adresse 0x06,	DDS Frequenz (x10): Byte 2 (MSB)
								;Adresse 0x07,	DDS Phasenwinkel: Low Byte
		de		0x00, 0x01		;Adresse 0x08,	DDS Phasenwinkel: High Byte
								;Adresse 0x09,	DDS Filter: 0=Off, 1=On
;Variable Starteinstellungen VCO/Wobbel
		de		0xC8, 0x00		;Adresse 0x0A,	VCO/Wobbel Fu L-Byte
								;Adresse 0x0B,	VCO/Wobbel Fu M-Byte
		de		0x00, 0x01		;Adresse 0x0C,	VCO/Wobbel Fu H-Byte
								;Adresse 0x0D,	VCO/Wobbel Kennlinie: 0 = Linear, 1 = Logarithmisch
		de		0x03, 0x03		;Adresse 0x0E,	VCO/Wobbel Dekaden: 1-3
								;Adresse 0x0F,	Wobbel Steps: 1=1024, 2=512, 3=256, 4=128, 5=64, 6=32
		de		0x02, 0x08		;Adresse 0x10,	Wobbel Man/Aut: 0 = Auto, 1 = Manuell, 2 = Auto+Burst, 3 = Manuell+Burst
								;Adresse 0x11,	Wobbel delta t: 0=1ms, 1=2ms, 2=5ms, 3=10ms, 4=20ms, 5=50ms,
								;								6=100ms, 7=200ms, 8=500ms, 9=1000ms, 10=2000ms
;Variable DAC Verstrkung / Kontrast
		de		0x00, 0x28		;Adresse 0x12,	DAC Verstrkung: 1 = 0-2048mV, 0 = 0-4096mV
								;Adresse 0x13,	Kontrastwert: 24-63 (40)

;------------------------------------------------------------------------------
; RESET VECTOR
;------------------------------------------------------------------------------

RES_VECT  ORG     0x0000            ; processor reset vector
          GOTO    START             ; go to beginning of program

;------------------------------------------------------------------------------
; HIGH PRIORITY INTERRUPT VECTOR
;------------------------------------------------------------------------------

ISRH      ORG     0x0008

          ; Run the High Priority Interrupt Service Routine
          GOTO    HIGH_ISR             

;------------------------------------------------------------------------------
; LOW PRIORITY INTERRUPT VECTOR / INTERRUPT SERVICE ROUTINE
;------------------------------------------------------------------------------

ISRL      ORG     0x0018
          
          MOVFF   WREG,W_TEMP          ; save W register
          MOVFF   STATUS, STATUS_TEMP ; save status register

	movf DDS_FREQ_W0,w			;addiere Arbeitsregister zu Periodenregister
	addwf DDS_PERIODE_L
	movf DDS_FREQ_W1,w
	addwfc DDS_PERIODE_M
	movf DDS_FREQ_W2,w
	addwfc DDS_PERIODE_H

	btfsc DDS_WAVE,0			;0=Sinus, 1=Dreieck, 2=Sgezahn, 3=Rechteck
	goto wf_sinus
	btfsc DDS_WAVE,1
	goto wf_dreieck
	btfsc DDS_WAVE,2
	goto wf_saege

wf_rechteck
	movlw d'0'
	btfss DDS_PERIODE_H,7
	addlw d'255'
	goto int_dac

wf_sinus
	movff DDS_PERIODE_H,INT_TEMP_1
	movff DDS_PERIODE_M,INT_TEMP_0
	rlcf INT_TEMP_0
	rlcf INT_TEMP_1
	rlcf INT_TEMP_0
	rlcf INT_TEMP_1
	movf INT_TEMP_1,w
	btfsc DDS_PERIODE_H,6
	sublw 0xFF
	movwf TBLPTRL
	movlw 0x7F				;High Byte der Tabellenadresse 
	movwf TBLPTRH
	TBLRD*	 				;lese den Tabellenwert ins TABLAT Register
	movf TABLAT,W 			;Hole den Tabellenwert nach W
	btfsc DDS_PERIODE_H,7
	sublw 0xFF
	goto int_dac

wf_dreieck
	movff DDS_PERIODE_H,INT_TEMP_1
	movlw d'64'
	addwf INT_TEMP_1
	rlcf DDS_PERIODE_M,w
	rlcf INT_TEMP_1,w	
	btfsc STATUS,C
	sublw d'255'
	goto int_dac

wf_saege
	movf DDS_PERIODE_H,w
	addlw d'128'
	goto int_dac

int_dac
	movwf PORTB					; -> DAC
	bcf PIR1,TMR2IF

	btfss PIR1,TMR1IF			;
	goto int_low_end
	bcf PIR1, TMR1IF			;Timer 1 Interrupt flag lschen
	decfsz COUNT_H_S			;  reduziere Zeitzhler um 1 und prfe, ob Zhler =0
	goto int_low_end			;Zeitzhler ist noch nicht 0, zurck zur Schleife

	movff T1_ZEIT_L1,TMR1H
	decfsz COUNT_ADJ			;Timer 1 wieder einstellen
	goto int_l_sub1
	movff T1_ZEIT_L,TMR1H
	movff T1_ZEIT_LOOP,COUNT_ADJ
int_l_sub1
	movff T1_ZEIT_H,COUNT_H_S
	bsf MESS_END,7				;Setze Schrittweiten Ende Bit

int_low_end
          MOVFF   STATUS_TEMP, STATUS ; restore status register
          MOVFF    W_TEMP, WREG        ; restore W register 
          RETFIE           

;------------------------------------------------------------------------------
; HIGH PRIORITY INTERRUPT SERVICE ROUTINE
;------------------------------------------------------------------------------

HIGH_ISR  
          MOVFF   WREG,W_TEMP              ; save W register
          MOVFF   STATUS, STATUS_TEMP ; save status register

	btfss STAT_INT,0			;Bit0 = Interruptroutine fr Nullsetzung Timer1
	goto int_capture
	clrf TMR1H					;lsche Timer 1, H-Byte
	movlw d'14'					;L-Byte mit Zeitversatz
	movwf TMR1L
	bcf PIR1,CCP1IF				;Interrupt Flag lschen
	bcf PIE1,CCP1IE				;Capture Interrupt verbieten
	bcf STAT_INT,0				;Bit lschen, da nur einmalig ausgefhrt wird
	bsf MESS_END,1
	goto int_high_end

int_capture
	btfss STAT_INT,1			;Bit1 = Interruptroutine fr Periodenmessung
	goto int_zaehler
	bcf PIR1,CCP1IF				;Interrupt Flag lschen
	bcf PIE1,CCP1IE				;Capture Interrupt verbieten
	bsf MESS_END,1

	movff CCPR1L,ZAHL_C0		;sichern der Timer 1 Werte im ZAHL_C Register
	movff CCPR1H,ZAHL_C1

	incfsz ZAHL_A0
	goto int_high_end
	incfsz ZAHL_A1
	goto int_high_end
	incfsz ZAHL_A2
	goto int_high_end
	incf ZAHL_A3
	goto int_high_end

int_zaehler
	btfss STAT_INT,2			;Bit2 = Interruptroutine fr Zhler
	goto int_wobbel
	bcf PIR1,CCP1IF				;Lsche CCP1 Interrupt Flag	
	bsf MESS_END,3				;Setze Update Bit
	btfss TA_DB2
	goto int_zaehl1
	bcf PIE1,CCP1IE				;CCP1 Interupts verbieten
	bsf MESS_END,2				;Setze Bit wenn TA_DB2 Eingang high ist, d.h. Zhlung gesperrt
	goto int_high_end
int_zaehl1
	incfsz ZAHL_A0
	goto int_high_end
	incfsz ZAHL_A1
	goto int_high_end
	incfsz ZAHL_A2
	goto int_high_end
	incf ZAHL_A3
	goto int_high_end


int_wobbel
	btfss STAT_INT,3			;Bit3 = Interruptroutine fr delta t Timer
	goto int_high_end
	bcf PIR1, TMR1IF			;Timer 1 Interrupt flag lschen
	decfsz COUNT_H_S			;  reduziere Zeitzhler um 1 und prfe, ob Zhler =0
	goto int_high_end			;Zeitzhler ist noch nicht 0, zurck zur Schleife

	movff T1_ZEIT_L1,TMR1H
	decfsz COUNT_ADJ			;Timer 1 wieder einstellen
	goto int_h_sub1
	movff T1_ZEIT_L,TMR1H
	movff T1_ZEIT_LOOP,COUNT_ADJ
int_h_sub1
	movff T1_ZEIT_H,COUNT_H_S
	bsf MESS_END,7				;Setze Schrittweiten Ende Bit
	goto int_high_end

clr_tmr1						;Timer 1 auf Null setzen
	clrf TMR1H
	movlw d'15'
	movwf TMR1L
	bcf PIR1,CCP1IF
	bcf PIE1,CCP1IE
	bsf MESS_END,1


int_high_end
          MOVFF    W_TEMP, WREG           ; restore W register
          MOVFF   STATUS_TEMP, STATUS ; restore status register
          RETFIE 


;------------------------------------------------------------------------------
; MAIN PROGRAM
;------------------------------------------------------------------------------

START

	clrf PORTA 				; Initialize PORTC by clearing output data latches
	clrf PORTB 				; Initialize PORTC by clearing output data latches
	clrf PORTC 				; Initialize PORTC by clearing output data latches
	clrf PORTE 				; Initialize PORTC by clearing output data latches
	clrf ADCON0				; ADC off, AN0
	movlw b'00001110'
	movwf ADCON1
	movlw b'10111110'		;rechtsbndig, 20 TAD, A/D Conversion Clock: FOSC/64
	movwf ADCON2
	clrf UCON				; USB disabled
	bsf UCFG,UTRDIS			; On-Chip Transceiver Disabled
	movlw b'11010001' 		; Value used to initialize data direction
	movwf TRISA 			; Set RA0,RA4,RA<7:6> as inputs
	movlw b'00000000' 		; Value used to initialize data direction
	movwf TRISB 			; Set RB<7:0> as outputs
	movlw b'01111100' 		; Value used to initialize data direction
	movwf TRISC 			; Set RC<6:2> as inputs

	movlw d'128'			;DAC auf Mittelstellung setzen
	movwf PORTB

	movlw b'10000000'		;Bit 7: Enable priority levels on interrupts
	movwf RCON
	movlw b'01000000'		;Bit 6: Enables all low priority peripheral interrupts
	movwf INTCON
	clrf INTCON2
	clrf IPR1				;TMR2: low priority interrupt
	bsf IPR1,TMR1IP			;TMR1: high priority interrupt
	bsf IPR1,CCP1IP			;CCP1: high priority interrupt
	movlw b'00100000'		;CCP1 Timer: Timer1 
	movwf T3CON

	movlw d'99'				;Timer2: Zhlt bis 99, rcksetzen auf 0 bei 100
	movwf PR2

	clrf PIE1
	clrf T0CON				;Timer und CCP1 Register lschen 
	clrf T1CON
	clrf T2CON
	clrf CCP1CON

	bsf CS_DAC				;setze Chip select DAC auf high
	bsf CS_VOL				;setze Chip select Digitalpoti auf high
	clrf MENU_STAT
	movlw 0x01
	movwf MENU_COUNT
	movwf DISP_PRESC

	call readeepr_data		;laden der Konstanten aus dem internen EEPROM in die F-Register

	movlw d'250'			;Wartezeit von 250ms
	call wait

	call init_display		;Initialisierung des Displays

;Testbereich Anfang

;Testbereich Ende

main						;Beginn der Hauptschleife
	movlw d'2'				;prfen, ob Programm 2(Zhler) gewhlt ist
	subwf PROGRAM,w
	btfsc STATUS,Z
	goto Zaehler			;ja, Programm 2(Zhler) ist gewhlt, springe zum Programmteil "Zaehler"

	movlw d'3'				;prfen, ob Programm 3(DDS) gewhlt ist
	subwf PROGRAM,w
	btfsc STATUS,Z
	goto dds				;ja, Programm 3(DDS)ist gewhlt, springe zum Programmteil "DDS"

	movlw d'4'				;prfen, ob Programm >=4 (VCO=4,Wobbel=5) gewhlt ist
	subwf PROGRAM,w
	btfsc STATUS,C
	goto vco_wob			;ja, Programm >=3 ist gewhlt, springe zum Programmteil "vco_wob"


Frequenz
	btfss EINGANG,0			;prfen, ob Eingang Analog gewhlt ist
	bcf ME_FILT				;Eingang TTL ist gewhlt,
	btfsc EINGANG,0			;prfen, ob Eingang TTL gewhlt ist
	bsf ME_FILT				;Eingang Analog ist gewhlt,
	movlw b'01000000'		;Komma nach der 1sten Stelle
	movwf KOMMA				;
	movlw 0x20				;ASCII fr "Blank"
	movwf UNIT				;
	clrf DISP_0
	clrf DISP_1
	clrf DISP_2
	clrf DISP_3
	clrf DISP_4
	clrf DISP_5
	clrf DISP_6
	clrf DISP_7
	call disp_main

frequenz_1
							;Im ersten Schritt wird mit Prescaler 1:16 die Frequenz ber 0,001024 sek grob gemessen 
	movlw b'00100011'		;TMR0: Off, 16Bit, extern,Prescaler 1:16 (fr Test)00100011
	movwf T0CON
	movlw b'00000000'		;16Bit read enable, Prescaler 1:1, TMR1 Off
	movwf T1CON
	clrf INTCON				;Interrupts verbieten

	call clear_ZAHL_C		;Register fr Frequenzzhler lschen
	clrf MESS_END			;Register lschen
	movlw 0xD0				;Timer 1 auf 1024 micro sek einstellen
	movwf TMR1H
	movlw 0x04				;Zeitdifferenz Timer 1 innerhalb der Messschleife (3 Takte)
	movwf TMR1L
	clrf TMR0H
	clrf TMR0L				;lschen des Timer0
	bcf PIR1,TMR1IF			;Timer 1 Interrupt flag lschen
	bcf INTCON,T0IF			;Timer 0 Interrupt flag lschen
	bsf T1CON,TMR1ON		;Timer1 starten
	bsf T0CON,TMR0ON		;Timer0 starten

freq1_loop
	btfss PIR1, TMR1IF
	goto $-2
	bcf T1CON,TMR1ON	
	movff TMR0L,ZAHL_C0
	movff TMR0H,ZAHL_C1

	btfss TA_MENUE			;Abfrage des Tasters zum Wechsel des Programms
	goto menue_1
							;wenn Frequenz <156250Hz => Periodenmessung (156250Hz/16*0,001024s = 10 => 0x000A)
	movlw 0x0A				;Hole Wert "0A" ins W-Register
	subwf ZAHL_C0,w			;Subtrahiere W-Register vom Low Byte
	movlw 0x00				;Hole Wert "0" ins W-Register
	btfss STATUS,C			;prfen, ob ein bertrag bei der Subtraktion aufgetreten ist
	addlw d'1'				;es ist ein bertrag aufgetreten, erhhe W-Register um 1
	subwf ZAHL_C1,w			;Subtrahiere W-Register vom High Byte 
	btfss STATUS,C			;prfen, ob das Ergebnis negativ ist
	goto Periode			;Ergebnis ist negativ, d.h. Frequenz <109375Hz

							;wenn Frequenz <6000000Hz => Prescaler=1 (6000000Hz/16*0,001024s = 384 => 0x0180)
	movlw 0x80				;Hole Wert "80" ins W-Register
	subwf ZAHL_C0,w			;Subtrahiere W-Register vom Low Byte
	movlw 0x01				;Hole Wert "01" ins W-Register
	btfss STATUS,C			;prfen, ob ein bertrag bei der Subtraktion aufgetreten ist
	addlw d'1'				;es ist ein bertrag aufgetreten, erhhe W-Register um 1
	subwf ZAHL_C1,w			;Subtrahiere W-Register vom High Byte 
	btfss STATUS,C			;prfen, ob das Ergebnis negativ ist
	goto presc_1			;Ergebnis ist negativ, d.h. Frequenz <6000000Hz

							;wenn Frequenz <12000000Hz => Prescaler=2 (12000000Hz/16*0,001024s = 768 => 0x0300)
	movlw 0x00				;Hole Wert "00" ins W-Register
	subwf ZAHL_C0,w			;Subtrahiere W-Register vom Low Byte
	movlw 0x03				;Hole Wert "03" ins W-Register
	btfss STATUS,C			;prfen, ob ein bertrag bei der Subtraktion aufgetreten ist
	addlw d'1'				;es ist ein bertrag aufgetreten, erhhe W-Register um 1
	subwf ZAHL_C1,w			;Subtrahiere W-Register vom High Byte 
	btfss STATUS, C			;prfen, ob das Ergebnis negativ ist
	goto presc_2			;Ergebnis ist negativ, d.h. Frequenz <12000000Hz

							;wenn Frequenz <24000000Hz => Prescaler=4 (24000000Hz/16*0,001024s = 1536 => 0x0600)
	movlw 0x00				;Hole Wert "00" ins W-Register
	subwf ZAHL_C0,w			;Subtrahiere W-Register vom Low Byte
	movlw 0x06				;Hole Wert "06" ins W-Register
	btfss STATUS,C			;prfen, ob ein bertrag bei der Subtraktion aufgetreten ist
	addlw d'1'				;es ist ein bertrag aufgetreten, erhhe W-Register um 1
	subwf ZAHL_C1,w			;Subtrahiere W-Register vom High Byte 
	btfss STATUS, C			;prfen, ob das Ergebnis negativ ist
	goto presc_4			;Ergebnis ist negativ, d.h. Frequenz <24000000Hz

							;wenn Frequenz <48000000Hz => Prescaler=8 (48000000Hz/16*0,001024s = 3072 => 0x0C00)
	movlw 0x00				;Hole Wert "00" ins W-Register
	subwf ZAHL_C0,w			;Subtrahiere W-Register vom Low Byte
	movlw 0x0C				;Hole Wert "0C" ins W-Register
	btfss STATUS,C			;prfen, ob ein bertrag bei der Subtraktion aufgetreten ist
	addlw d'1'				;es ist ein bertrag aufgetreten, erhhe W-Register um 1
	subwf ZAHL_C1,w			;Subtrahiere W-Register vom High Byte 
	btfss STATUS, C			;prfen, ob das Ergebnis negativ ist
	goto presc_8			;Ergebnis ist negativ, d.h. Frequenz <48000000Hz

							;wenn Frequenz <96000000Hz => Prescaler=16 (96000000Hz/16*0,001024s = 6144 => 0x1800)
	movlw 0x00				;Hole Wert "00" ins W-Register
	subwf ZAHL_C0,w			;Subtrahiere W-Register vom Low Byte
	movlw 0x18				;Hole Wert "18" ins W-Register
	btfss STATUS,C			;prfen, ob ein bertrag bei der Subtraktion aufgetreten ist
	addlw d'1'				;es ist ein bertrag aufgetreten, erhhe W-Register um 1
	subwf ZAHL_C1,w			;Subtrahiere W-Register vom High Byte 
	btfss STATUS, C			;prfen, ob das Ergebnis negativ ist
	goto presc_16			;Ergebnis ist negativ, d.h. Frequenz <96000000Hz
	goto presc_32			;Ergebnis ist positiv, d.h. Frequenz >=96000000Hz

presc_1
	movlw b'00101000'		;TMR0 extern(Prescaler 1:1)
	movwf T0CON
	movlw d'1'
	movwf PRESC_TMR0
	movwf DISP_PRESC
	goto messen_freq

presc_2
	movlw b'00100000'		;TMR0 extern(Prescaler 1:2)
	movwf T0CON
	movlw d'2'
	movwf PRESC_TMR0
	movwf DISP_PRESC
	goto messen_freq

presc_4
	movlw b'00100001'		;TMR0 extern(Prescaler 1:4)
	movwf T0CON
	movlw d'3'
	movwf PRESC_TMR0
	movlw d'4'
	movwf DISP_PRESC
	goto messen_freq

presc_8
	movlw b'00100010'		;TMR0 extern(Prescaler 1:8)
	movwf T0CON
	movlw d'4'
	movwf PRESC_TMR0
	movlw d'8'
	movwf DISP_PRESC
	goto messen_freq

presc_16
	movlw b'00100011'		;TMR0 extern(Prescaler 1:16)
	movwf T0CON
	movlw d'5'
	movwf PRESC_TMR0
	movlw d'16'
	movwf DISP_PRESC
	goto messen_freq

presc_32
	movlw b'00100100'		;TMR0 extern(Prescaler 1:32)
	movwf T0CON
	movlw d'6'
	movwf PRESC_TMR0
	movlw d'32'
	movwf DISP_PRESC
	goto messen_freq


messen_freq
	call clear_ZAHL_C		;lsche ZAHL_C Register fr die Frequenzmessung
	clrf MESS_END
	movlw HIGH_BYTE_S		;Zhler auf 1 sek einstellen
	movwf COUNT_H_S
	movlw LOW_BYTE_S
	movwf TMR1H	
	movlw d'2'				;Zeitdifferenz (5Takte) Timer 1 innerhalb der Messschleife
	movwf TMR1L
	clrf TMR0H				;lsche Timer0
	clrf TMR0L
	bcf PIR1,TMR1IF			;Timer 1 Interrupt flag lschen
	bcf INTCON,T0IF			;Timer 0 Interrupt flag lschen	
	bsf T1CON,TMR1ON		;Timer1 starten
	bsf T0CON,TMR0ON		;Timer0 starten

freq_loop
	btfss PIR1,TMR1IF		;Abfrage des Timer 1 berlauf Bits (Zeitmessung)
	goto freq_loop1			;kein berlauf, weiter 
	movf TMR0L,w			;berlauf vorhanden, hole sicherheitshalber TMR0 Wert ins W-Register
	bcf PIR1, TMR1IF		;lsche Flag,
	btfss TA_MENUE			;Abfrage des Tasters zum Wechsel des Programms
	goto menue_1
	decfsz COUNT_H_S		;  reduziere Zeitzhler um 1 und prfe, ob Zhler =0
	goto freq_loop1			;Zhler >0, weiter
	goto adj_freq			;Zhler =0, d.h. 1 Sekunde ist um, 
freq_loop1
	btfss INTCON, T0IF		;Abfrage Timer 0 berlauf Bits (Anzahl L/H bergnge)
	goto freq_loop			;kein berlauf, zurck zur Schleife 
	bcf INTCON,T0IF			;berlauf vorhanden, lsche Flag,
	incfsz ZAHL_C2			;Erhhe Zhler um 1 und prfe berlauf
	goto freq_loop			;kein berlauf, zurck zur Schleife 
	incf ZAHL_C3			;Erhhe Zhler um 1
	goto freq_loop			;zurck zur Schleife


adj_freq
	movwf ZAHL_C0			;sichern des TMR0L/H Wert in ZAHL_C0/C1 Register
	movff TMR0H,ZAHL_C1

adj_freq1
	btfss TA_MENUE			;Abfrage des Tasters zum Wechsel des Programms
	goto menue_1
	decf PRESC_TMR0				;reduziere den Zhler um 1 und prfe ob 0
	btfsc STATUS,Z				;
	goto adj_freq_end			;Multiplikation mit 2xZhlerinhalt

	bcf STATUS,C				;lsche Status Bit, damit es beim rotieren nicht strt
	rlcf ZAHL_C0					;x2
	rlcf ZAHL_C1					;x2
	rlcf ZAHL_C2					;x2
	rlcf ZAHL_C3					;x2
	goto adj_freq1	

adj_freq_end
	movf PROGRAM,w			;prfen, ob Programm 0, d.h. Frequenz gewhlt ist
	btfss STATUS,Z
	goto freq_to_peri		;Programm ist >0<, d.h. Periode ist gewhlt, umrechnen der Frequenz in Periode
							;Programm ist 0, d.h. Frequenz ist gewhlt,


freq_mhz
	movlw 0x00				;wenn Frequenz <1.000.000.000 Hz ist (0x3B9ACA00) => MHz, 
	movwf TEMP_10			;  wenn Frequenz >=1.000.000.000 Hz ist, gehe zu GHz
	movlw 0xCA
	movwf TEMP_11
	movlw 0x9A
	movwf TEMP_12
	movlw 0x3B
	call test_kleiner		;Unterprogramm zum prfen, ob ZAHL_C < Wert in TEMP/W-Reg.
	btfss STATUS,C
	goto $+0x06				;Frequenz <1.000.000.000 Hz
	goto freq_ghz			;Frequenz >=1.000.000.000 Hz, gehe zu GHz

	movlw A'M'				;ASCII fr "M" Mega Hz
	movwf UNIT				;
	goto display_main

freq_ghz
	movlw 0xE8				;Frequenz >=1.000.000.000 Hz, lade Wert 1.000 (0x000003E8) ins Nennerregister (ZAHL_A) fr die Division,
	movwf ZAHL_A0			;   die Frequenz steht noch in ZAHL_C
	movlw 0x03
	movwf ZAHL_A1
	movlw 0x00
	movwf ZAHL_A2
	movlw 0x00
	movwf ZAHL_A3
	call div32				;teile Frequenz durch 1.000 (ZAHL_C / ZAHL_A = ZAHL_C)

	movlw A'G'				;ASCII fr "G" Giga Hz
	movwf UNIT				;
	goto display_main


freq_to_peri				;Programmteil zur Umwandlung von Frequenz in Periode
	movff ZAHL_C0,ZAHL_A0	;Kopiere Frequenz aus ZAHL_C ins Nennerregister (ZAHL_A) fr die Division,
	movff ZAHL_C1,ZAHL_A1
	movff ZAHL_C2,ZAHL_A2
	movff ZAHL_C3,ZAHL_A3

	movlw 0xE9				;wenn Frequenz <1001 (0x000003E9) => Millisekunden, wenn Frequenz >=1001 gehe zu Mikrosekunden
	movwf TEMP_10
	movlw 0x03
	movwf TEMP_11
	movlw 0x00
	movwf TEMP_12
	movlw 0x00
	call test_kleiner		;Unterprogramm zum prfen, ob ZAHL_C < Wert in TEMP/W-Reg.
	btfss STATUS,C
	goto $+0x06				;Frequenz <1001: 1.000.000.000/Frequenz (=> 9 Stellen)
	goto peri_mikrosek1		;Frequenz >=1001
							;Frequenz <1001: 1.000.000.000/Frequenz (=> 9 Stellen)
	movlw 0x00				;lade Wert 1.000.000.000 (0x3B9ACA00) ins Zhlerregister (ZAHL_C) fr die Division,
	movwf ZAHL_C0
	movlw 0xCA
	movwf ZAHL_C1
	movlw 0x9A
	movwf ZAHL_C2
	movlw 0x3B
	movwf ZAHL_C3
	clrf ZAHL_C4
	clrf ZAHL_C5
	clrf ZAHL_C6
	clrf ZAHL_C7
	call div32				;teile 1.000.000.000 durch Frequenz (ZAHL_C / ZAHL_A = ZAHL_C)

	movlw A'm'				;ASCII fr "m" milli Sek
	movwf UNIT				;
	goto display_main

peri_mikrosek1				;Frequenz >=1001
	movlw 0x00				;lade Wert 1.000.000.000.000 (0x000000E8D4A51000) ins Zhlerregister (ZAHL_C) fr die Division,
	movwf ZAHL_C0
	movlw 0x10
	movwf ZAHL_C1
	movlw 0xA5
	movwf ZAHL_C2
	movlw 0xD4
	movwf ZAHL_C3
	movlw 0xE8
	movwf ZAHL_C4
	clrf ZAHL_C5
	clrf ZAHL_C6
	clrf ZAHL_C7
	call div32				;teile 1.000.000.000.000 durch Frequenz (ZAHL_C / ZAHL_A = ZAHL_C)

	movlw A'u'				;ASCII fr "u" mikro Sek
	movwf UNIT				;
	goto display_main



Periode
	clrf STAT_INT			;Lsche Register und setze Bit fr Periode Interrup Routine
	bsf STAT_INT,0
	bcf PIE1,TMR1IE			;Timer1 interrupt verbieten

	movlw d'1'
	movwf DISP_PRESC

	clrf ZAHL_A0			;Vorbereiten zur Syncronisierung der positiven Flanke von RC2 mit Timer1
	clrf ZAHL_A1			;
	clrf ZAHL_A2
	clrf ZAHL_A3
	call clear_ZAHL_C		;Lsche Register ZAHL_C fr die Zeitmessung
	clrf MESS_END

	movlw b'00000101'		;Bit 3-0: Capture mode: every rising edge
	movwf CCP1CON
	clrf TMR1H				;Lsche Timer1
	clrf TMR1L
	bcf PIR1,TMR1IF			;Timer 1 Interrupt flag lschen
	
	bsf T1CON,TMR1ON

	bcf PIR1,CCP1IF			;Capture Interrupt Flag lschen
	bsf PIE1,CCP1IE			;Capture Interrupt erlauben
	bsf INTCON,GIE			;Globale Interups erlauben

startloop_peri				;Warten auf die positive Flanke von RC2
	btfss TA_MENUE			;Abfrage des Tasters zum Wechsel des Programms (Abbruch)
	goto menue_1

	btfss PIR1, TMR1IF		;
	goto startloop_per1		;
	bcf PIR1, TMR1IF
	incfsz ZAHL_C2
	goto startloop_per1
	incf ZAHL_C3
	btfss ZAHL_C3,2			;Abfrage ob maximale Wartezeit abgelaufen ist (0x04000000 entspricht 67.108.864 Takte = 5,59 s)
	goto startloop_per1		;Maximale Wartezeit noch nicht abgelaufen, weiter
	bcf INTCON,GIE			;maximale Wartezeit ist abgelaufen, Interups verbieten
	goto Frequenz			;maximale Messzeit ist erreicht, Abbruch und zurck zu Frequenz 
startloop_per1
	btfss MESS_END,1		;Abfrage der positiven Flanke von RC2
	goto startloop_peri		;noch keine positive Flanke, zurck zur Warteschleife
	clrf MESS_END			;positive Flanke liegtan, Beginn der Perioden/Frequenzmessung
	bsf STAT_INT,1			;setze Bit fr Capture Interrupt Routine
	bcf PIR1, TMR1IF		;lsche Timer1 berlaufflag
	clrf ZAHL_C2
	clrf ZAHL_C3
peri_loop
	btfss TA_MENUE			;Abfrage des Tasters zum Wechsel des Programms
	goto menue_1
peri_loop1
	btfsc PIR1, TMR1IF
	goto $+8
	bsf PIE1,CCP1IE			;Capture Interrupt wieder erlauben
	goto peri_loop
	bcf MESS_END,1
	bcf PIR1, TMR1IF
	incf ZAHL_C2
	movlw 0xB7				;Abfrage ob Messzeit abgelaufen ist (0x00B70000 entspricht 11.993.088 Takte = 0,999424 s)
	subwf ZAHL_C2,w
	btfss STATUS,Z
	goto peri_loop			;Messzeit ist noch nicht abgelaufen, zurck zur Schleife
peri_loop2					;Messzeit von 0,999424 s abgelaufen, warten auf die nchste positive Flanke
	bsf PIE1,CCP1IE			;Capture Interrupt wieder erlauben
	btfss TA_MENUE			;Abfrage des Tasters zum Wechsel des Programms
	goto menue_1
	btfss MESS_END,1		;Abfrage auf positive Flanke
	goto peri_loop3			;Flag ist nicht gesetzt, weiter
	bcf INTCON,GIE			;Flag ist gesetzt, Interrups disable
	clrf CCP1CON			;Capture disable
	goto auswert_peri
peri_loop3
	btfss PIR1, TMR1IF
	goto peri_loop2
	bcf PIR1, TMR1IF
	incfsz ZAHL_C2
	goto $+0x06
	incf ZAHL_C3
	btfss ZAHL_C3,2			;Abfrage ob maximale Wartezeit abgelaufen ist (0x04000000 entspricht 67.108.864 Takte = 5,59 s)
	goto peri_loop2			;maximale Messzeit ist noch nicht abgelaufen, zurck zur Schleife
	goto Frequenz			;maximale Messzeit ist erreicht, Abbruch und zurck zu Frequenz 

auswert_peri
	movff ZAHL_C0,DISP_0	;temporres sichern der Mezeit (ZAHL_C) in Displayregistern
	movff ZAHL_C1,DISP_1
	movff ZAHL_C2,DISP_2
	movff ZAHL_C3,DISP_3

	movlw d'12'				;Anzahl der L=>H bergnge von RC2 (ZAHL_A) mit 12 multiplizieren (12 MHz)
	movwf ZAHL_B0			;lade Multiplikationsregister ZAHL_B mit Wert 12
	clrf ZAHL_B1
	clrf ZAHL_B2
	clrf ZAHL_B3
	call mul32				;Multiplikation ZAHL_A x ZAHL_B = ZAHL_C
	movff ZAHL_C0,ZAHL_A0	;rcksichern des Ergebnis in ZAHL_A Registern
	movff ZAHL_C1,ZAHL_A1
	movff ZAHL_C2,ZAHL_A2
	movff ZAHL_C3,ZAHL_A3

	movf PROGRAM,w			;prfen, ob Programm 0, d.h. Frequenz gewhlt ist
	btfss STATUS,Z
	goto auswert_peri_1		;Programm ist >0<, d.h. Periode ist gewhlt
			;Frequenz ist gewhlt, Frequenz = Anzahl der L=>H bergnge von RC2 (ZAHL_A) x 1.000.000 / Mezeit (ZAHL_C)

	movlw 0x40				;lade Wert 1.000.000 (0x000F4240) in Zhlerregister ZAHL_B fr die Multiplikation
	movwf ZAHL_B0	
	movlw 0x42
	movwf ZAHL_B1
	movlw 0x0F
	movwf ZAHL_B2
	movlw 0x00
	movwf ZAHL_B3
	call mul32				;Multipliziern der Anzahl der L=>H bergnge von RB0 (ZAHL_A) mit 1.000.000 (ZAHL_B), das
							;Ergebnis steht in ZAHL_C

	movff DISP_0,ZAHL_A0	;Rcksichern der Mezeit in Nennerregister (ZAHL_A) fr die Division
	movff DISP_1,ZAHL_A1
	movff DISP_2,ZAHL_A2
	movff DISP_3,ZAHL_A3
	call div32				;Division durch die Mezeit (ZAHL_C / ZAHL_A = ZAHL_C, Rest in ZAHL_B)

	movff ZAHL_C0,DISP_4	;Zwischenspeichern des Ganzzahligen Ergebnisses der Frequenz in Displayregistern
	movff ZAHL_C1,DISP_5
	movff ZAHL_C2,DISP_6
	movff ZAHL_C3,DISP_7

	movlw 0x01				;wenn Frequenz <1 Hz (0x00000001) => mHz, wenn Frequenz >=1 gehe zu Hz
	movwf TEMP_10
	movlw 0x00
	movwf TEMP_11
	movlw 0x00
	movwf TEMP_12
	movlw 0x00
	call test_kleiner		;Unterprogramm zum prfen, ob ZAHL_C < Wert in TEMP/W-Reg.
	btfss STATUS,C
	goto $+0x06				;Frequenz <1 => mHz
	goto peri_freq_hz		;Frequenz >=1 gehe zu Hz
							;Frequenz und Rest x 1.000.000.000, Rest durch Mezeit teilen und zu Frequenz addieren
	movlw 0x00				;Lade Multiplikationsregister ZAHL_A mit Wert 1.000.000.000 (0x3B9ACA00), Rest steht noch im 
	movwf ZAHL_A0			; zweiten Multiplikationsregister ZAHL_B
	movlw 0xCA
	movwf ZAHL_A1
	movlw 0x9A
	movwf ZAHL_A2
	movlw 0x3B
	movwf ZAHL_A3	
	call fliesskomma1

	movlw 0x00				;Lade Multiplikationsregister ZAHL_A mit Wert 1.000.000.000 (0x3B9ACA00) 
	movwf ZAHL_A0			;
	movlw 0xCA
	movwf ZAHL_A1
	movlw 0x9A
	movwf ZAHL_A2
	movlw 0x3B
	movwf ZAHL_A3	
	call fliesskomma2

	movlw A'm'				;ASCII fr "m" milli Hz
	movwf UNIT				;
	goto display_main

peri_freq_hz
	movlw 0xE8				;wenn Frequenz <1.000 Hz ist (0x000003E8) => Hz, 3/4 Vor-/Nachkommastellen
	movwf TEMP_10			;  wenn Frequenz >=1.000 Hz ist, gehe zu kHz 1/6 Vor-/Nachkommastellen
	movlw 0x03
	movwf TEMP_11
	movlw 0x00
	movwf TEMP_12
	movlw 0x00
	call test_kleiner		;Unterprogramm zum prfen, ob ZAHL_C < Wert in TEMP/W-Reg.
	btfss STATUS,C
	goto $+0x06				;Frequenz <1.000 Hz => Hz
	goto peri_freq_khz		;Frequenz >=1.000 gehe zu kHz 1/6 Vor-/Nachkommastellen
							;Frequenz und Rest x 1.000.000, Rest durch Mezeit teilen und zu Frequenz addieren
	movlw 0x40				;Lade Multiplikationsregister ZAHL_A mit Wert 1.000.000 (0x000F4240), Rest steht noch im 
	movwf ZAHL_A0			; zweiten Multiplikationsregister ZAHL_B
	movlw 0x42
	movwf ZAHL_A1
	movlw 0x0F
	movwf ZAHL_A2
	movlw 0x00
	movwf ZAHL_A3	
	call fliesskomma1

	movlw 0x40				;Lade Multiplikationsregister ZAHL_A mit Wert 1.000.000 (0x000F4240) 
	movwf ZAHL_A0			;
	movlw 0x42
	movwf ZAHL_A1
	movlw 0x0F
	movwf ZAHL_A2
	movlw 0x00
	movwf ZAHL_A3	
	call fliesskomma2

	movlw 0x20				;ASCII fr "blank" 
	movwf UNIT				;
	goto display_main


peri_freq_khz
	movlw 0x40				;wenn Frequenz <1.000.000 Hz ist (0x000F4240) => kHz, 3/4 Vor-/Nachkommastellen
	movwf TEMP_10			;  wenn Frequenz >=1.000.000 Hz ist, gehe zu MHz 1/6 Vor-/Nachkommastellen
	movlw 0x42
	movwf TEMP_11
	movlw 0x0F
	movwf TEMP_12
	movlw 0x00
	call test_kleiner		;Unterprogramm zum prfen, ob ZAHL_C < Wert in TEMP/W-Reg.
	btfss STATUS,C
	goto $+0x06				;Frequenz <1.000.000 Hz => kHz
	goto peri_freq_mhz		;Frequenz >=1.000.000 gehe zu MHz 1/6 Vor-/Nachkommastellen
							;Frequenz und Rest x 1.000, Rest durch Mezeit teilen und zu Frequenz addieren
	movlw 0xE8				;Lade Multiplikationsregister ZAHL_A mit Wert 1.000 (0x000003E8), Rest steht noch im 
	movwf ZAHL_A0			; zweiten Multiplikationsregister ZAHL_B
	movlw 0x03
	movwf ZAHL_A1
	movlw 0x00
	movwf ZAHL_A2
	movlw 0x00
	movwf ZAHL_A3	
	call fliesskomma1

	movlw 0xE8				;Lade Multiplikationsregister ZAHL_A mit Wert 1.000 (0x000003E8) 
	movwf ZAHL_A0			;
	movlw 0x03
	movwf ZAHL_A1
	movlw 0x00
	movwf ZAHL_A2
	movlw 0x00
	movwf ZAHL_A3	
	call fliesskomma2

	movlw A'k'				;ASCII fr "k" kilo Hz
	movwf UNIT				;
	goto display_main

peri_freq_mhz
	movlw A'M'				;ASCII fr "M" Mega Hz
	movwf UNIT				;
	goto display_main



auswert_peri_1
	movff DISP_0,ZAHL_C0	;rcksichern der Mezeit aus Displayregistern nach ZAHL_C
	movff DISP_1,ZAHL_C1
	movff DISP_2,ZAHL_C2
	movff DISP_3,ZAHL_C3	
	clrf ZAHL_C4
	clrf ZAHL_C5
	clrf ZAHL_C6
	clrf ZAHL_C7
	call div32				;im ersten Schritt wird die Messzeit durch die Anzahl der L=>H bergnge von RC2 geteilt
							; ZAHL_C / ZAHL_A = ZAHL_C, Rest in ZAHL_B
	movff ZAHL_A0,DISP_0	;temporres sichern der L=>H bergnge (ZAHL_A) in Displayregistern
	movff ZAHL_A1,DISP_1
	movff ZAHL_A2,DISP_2
	movff ZAHL_A3,DISP_3
	movff ZAHL_C0,DISP_4	;Zwischenspeichern des Ganzzahligen Ergebnisses in Displayregistern
	movff ZAHL_C1,DISP_5
	movff ZAHL_C2,DISP_6
	movff ZAHL_C3,DISP_7

	movlw 0xE8				;wenn Periode <1.000 (0x000003E8) => Microsekunden, wenn Periode >=1.000 gehe zu Millisekunden
	movwf TEMP_10
	movlw 0x03
	movwf TEMP_11
	movlw 0x00
	movwf TEMP_12
	movlw 0x00
	call test_kleiner		;Unterprogramm zum prfen, ob ZAHL_C < Wert in TEMP/W-Reg.
	btfss STATUS,C
	goto $+0x06				;Periode <1.000 => Microsekunden
	goto peri_millisek		;Periode >=1.000 gehe zu Millisekunden
							;Ganzzahlige Periode und Rest x 1.000.000, Rest durch Anzahl L=>H bergnge teilen und zu Periode addieren
	movlw 0x40				;Lade Multiplikationsregister ZAHL_A mit Wert 1.000.000 (0x000F4240), Rest steht noch im 
	movwf ZAHL_A0			; zweiten Multiplikationsregister ZAHL_B
	movlw 0x42
	movwf ZAHL_A1
	movlw 0x0F
	movwf ZAHL_A2
	movlw 0x00
	movwf ZAHL_A3	
	call fliesskomma1

	movlw 0x40				;Lade Multiplikationsregister ZAHL_A mit Wert 1.000.000 (0x000F4240)
	movwf ZAHL_A0
	movlw 0x42
	movwf ZAHL_A1
	movlw 0x0F
	movwf ZAHL_A2
	movlw 0x00
	movwf ZAHL_A3	
	call fliesskomma2

	movlw A'u'				;ASCII fr "u" micro
	movwf UNIT				;
	goto display_main

peri_millisek
	movlw 0x40				;wenn Periode <1.000.000 (0x000F4240) => Millisekunden, wenn Periode >=1.000.000 gehe zu Sekunden (war 40.000)
	movwf TEMP_10
	movlw 0x42
	movwf TEMP_11
	movlw 0x0F
	movwf TEMP_12
	movlw 0x00
	call test_kleiner		;Unterprogramm zum prfen, ob ZAHL_C < Wert in TEMP/W-Reg.
	btfss STATUS,C
	goto $+0x06				;Periode <1.000.000 => Millisekunden
	goto peri_sek			;Periode >=1.000.000, gehe zu Sekunden
							;Ganzzahlige Periode und Rest x 1000, Rest durch Anzahl L=>H bergnge teilen und zu Periode addieren
	movlw 0xE8				;Lade Multiplikationsregister ZAHL_A mit Wert 1000 (0x000003E8), Rest steht noch im 
	movwf ZAHL_A0			; zweiten Multiplikationsregister ZAHL_B
	movlw 0x03
	movwf ZAHL_A1
	movlw 0x00
	movwf ZAHL_A2
	movlw 0x00
	movwf ZAHL_A3	
	call fliesskomma1

	movlw 0xE8				;Lade Multiplikationsregister ZAHL_A mit Wert 1000 (0x000003E8) 
	movwf ZAHL_A0
	movlw 0x03
	movwf ZAHL_A1
	movlw 0x00
	movwf ZAHL_A2
	movlw 0x00
	movwf ZAHL_A3	
	call fliesskomma2

	movlw A'm'				;ASCII fr "m" milli
	movwf UNIT				;
	goto display_main

peri_sek
	movlw 0x20				;ASCII fr "blank"
	movwf UNIT				;

display_main
	call convert_hex_dig	;Aufruf des Unterprogramms zum Zerlegen der 32-Bit Periode in die einzelnen Digits (99.999.999)zur Anzeige auf dem Display
	movf DISP_8,w
	btfsc STATUS,Z
	goto $+0x0E
	movlw d'3'
	movwf LOOP_1
	movlw b'00010000'		;Komma nach der 3ten Stelle
	movwf KOMMA				;
	goto display_main1
	movf DISP_7,w
	btfsc STATUS,Z
	goto $+0x0E
	movlw d'2'
	movwf LOOP_1
	movlw b'00100000'		;Komma nach der 2ten Stelle
	movwf KOMMA				;
	goto display_main1
	movlw d'1'
	movwf LOOP_1
	movlw b'01000000'		;Komma nach der 1ten Stelle
	movwf KOMMA				;
display_main1
	decfsz LOOP_1
	goto $+0x06
	goto display_main2
	movff DISP_1,DISP_0
	movff DISP_2,DISP_1
	movff DISP_3,DISP_2
	movff DISP_4,DISP_3
	movff DISP_5,DISP_4
	movff DISP_6,DISP_5
	movff DISP_7,DISP_6
	movff DISP_8,DISP_7
	clrf DISP_8
	goto display_main1

display_main2
	call disp_main
	goto frequenz_1

Zaehler
	bcf PIE1, TMR1IE		;Timer1 interrupt verbieten
	bcf PIE1, TMR2IE		;Timer2 interrupt verbieten

	btfss EINGANG,0			;prfen, ob Eingang Analog gewhlt ist
	bcf ME_FILT				;Eingang TTL ist gewhlt,
	btfsc EINGANG,0			;prfen, ob Eingang TTL gewhlt ist
	bsf ME_FILT				;Eingang Analog ist gewhlt,

	clrf DISP_0				;Lsche Anzeigeregister fr Display
	clrf DISP_1
	clrf DISP_2
	clrf DISP_3
	clrf DISP_4
	clrf DISP_5
	clrf DISP_6
	clrf DISP_7
	clrf DISP_8
	call clear_display
	call disp_zaehl
zaehler_1
	clrf STAT_INT
	bsf STAT_INT,2
	clrf ZAHL_A0
	clrf ZAHL_A1
	clrf ZAHL_A2
	clrf ZAHL_A3
	clrf MESS_END
	movlw b'00000101'		;Bit 3-0: Capture mode: every rising edge
	movwf CCP1CON
	bcf PIR1,CCP1IF			;CCP1 Interrupt flag lschen
	bsf PIE1,CCP1IE			;CCP1 Interupts erlauben
	btfss TA_MENUE			;Abfrage des Tasters zum Wechsel des Programms
	goto menue_1
	btfsc TA_DB2
	goto $-0x08
	bsf INTCON,GIE			;Interups erlauben
zaehl_loop
	btfss TA_MENUE			;Abfrage des Tasters zum Wechsel des Programms
	goto menue_1
	btfss TA_DB2
	goto zaehl_loop1
	bsf MESS_END,2
	goto zaehl_loop2
zaehl_loop1
	btfss MESS_END,3		;Abfrage Status flag
	goto zaehl_loop
	clrf MESS_END			;Status flag lschen
zaehl_loop2
	movff ZAHL_A0,ZAHL_C0	;Kopiere Register 
	movff ZAHL_A1,ZAHL_C1
	movff ZAHL_A2,ZAHL_C2
	movff ZAHL_A3,ZAHL_C3
	clrf ZAHL_C4
	clrf ZAHL_C5
	clrf ZAHL_C6
	clrf ZAHL_C7
	call convert_hex_dig	;Aufruf des Unterprogramms zum Zerlegen der 32-Bit Zhlers in die einzelnen Digits (99.999.999)zur Anzeige auf dem Display
	call disp_zaehl
	btfss MESS_END,2
	goto zaehl_loop
	bcf INTCON,GIE			;Interups verbieten	
	movlw d'250'			;Wartezeit von 250ms
	call wait 		
	goto zaehler_1

dds
	clrf STAT_INT			;lsche Statusregister

	call init_dds			;Unterprogramm zum setzen der Bits in DDS_WAVE,
							; Filter Ein/Aus, DDS Pegel, DDS Phase
	call clear_ZAHL_C
	movff DDS_PHASE_L,ZAHL_C0
	movff DDS_PHASE_H,ZAHL_C1
	call convert_hex_dig	;Aufruf des Unterprogramms zum Zerlegen Einer 32-Bit Zahl in einzelne Digits (max.999.999.999)
	movff DISP_0,ZAHL_A0
	movff DISP_1,ZAHL_A1
	movff DISP_2,ZAHL_A2
	movff DDS_FREQ_0,ZAHL_C0		;Kopieren der 4 8-Bit DDS_FREQ-Register in die Rechenregister fr die Zerlegung in einzelne Digits
	movff DDS_FREQ_1,ZAHL_C1		;
	movff DDS_FREQ_2,ZAHL_C2		;
	call convert_hex_dig	;Aufruf des Unterprogramms zum Zerlegen Einer 32-Bit Zahl in einzelne Digits (max.999.999.999)
	call clear_display
	call disp_dds
	call calc_dds_freq		;Aufruf des Unterprogramms zur Berechnung der Frequenzregisterdaten
	clrf DDS_PERIODE_L		;Kopieren des Phasenwinkels in die DDS_PERIODE Register
	movff DDS_PHASEREG_L,DDS_PERIODE_M
	movff DDS_PHASEREG_H,DDS_PERIODE_H
	movff DDS_FREQREG_0,DDS_FREQ_W0		;Kopieren des Frequenzregister in die Arbeitsregister
	movff DDS_FREQREG_1,DDS_FREQ_W1
	movff DDS_FREQREG_2,DDS_FREQ_W2
	call dds_pegel			;Aufruf des Unterprogramms zur Ansteuerung des Digitalpoti fr den DDS Pegel

	clrf TMR2
	bcf PIR1,TMR2IF			;Timer 2 Interrupt flag lschen
	bsf PIE1,TMR2IE			;Timer 2 Interrupt erlauben
	bcf PIE1,TMR1IE			;Timer 1 Interrupt verbieten
	bsf T2CON,TMR2ON		;Timer 2 starten
	bsf INTCON,GIE

	btfsc TA_MENUE
	goto $-2
	goto menue_1

vco_wob
	call init_dds			;Unterprogramm zum setzen der Bits in DDS_WAVE,
							; Filter Ein/Aus, DDS Pegel, DDS Phase
	clrf DDS_FREQ_W0
	clrf DDS_FREQ_W1
	clrf DDS_FREQ_W2
	clrf DDS_PERIODE_L
	clrf DDS_PERIODE_M
	clrf DDS_PERIODE_H
	call prep_vco_wob		;Unterprogramm zur Berechnung von statischen Daten, 

	btfsc MESS_END,3		;prfe Fehler Bit (allgemein)
	goto vco_wob_fehler		;Obere Frequenz >=Maximalfrequenz Hz, gehe zu Fehler

	call clear_display
	call disp_vco
	clrf DDS_PERIODE_L		;Kopieren des Phasenwinkels in die DDS_PERIODE Register
	movff DDS_PHASEREG_L,DDS_PERIODE_M
	movff DDS_PHASEREG_H,DDS_PERIODE_H

	bsf MESS_END,7			;setze dt Ende Bit 
	clrf TMR2
	bcf PIR1,TMR2IF			;Timer 2 Interrupt flag lschen
	bcf PIR1,TMR1IF			;Timer 1 Interrupt flag lschen
	bsf PIE1,TMR2IE			;Timer 2 Interrupt erlauben
	bcf PIE1,TMR1IE			;Timer 1 Interrupt verbieten

	bsf INTCON,GIEL
	bsf INTCON,GIEH

	btfsc PROGRAM,0			;Prfen ob Programm VCO (=4 => 00000100) oder Wobbel (=5 => 00000101) gewhlt ist (Bit 0 prfen) 
	goto vco_wob_dac
	bsf T2CON,TMR2ON		;Timer 2 starten
	goto vco_messen			;Bit 0 ist nicht gesetzt, d.h. VCO ist gewhlt


wob_loop
	btfss WOB_M_A,0			;prfen ob Wobbel Manuell (1) oder Automatisch (0) gewhlt ist
	goto inc_wobfreq		;Wobbel Automatisch ist gewhlt

wob_man
	call menu_befehl		;Aufruf Unterprogramm zum Laden der Tastaturbefehle
	btfss TA_MENUE			;prfen ob Bit fr Befehl Menue nicht gesetzt ist
	goto menue_1				;wenn ja berspringe diesen Befehl, wenn nein Rcksprung in die Hauptschleife	
	btfss BEFEHL,1			;prfen ob Bit fr Befehl OK nicht gesetzt ist
	goto menue_1				;wenn ja berspringe diesen Befehl
	btfss BEFEHL,4			;prfen ob Bit fr Befehl "right" nicht gesetzt ist
	goto inc_wobfreq		;wenn ja berspringe diesen Befehl
	btfss BEFEHL,5			;prfen ob Bit fr Befehl "left" nicht gesetzt ist
	goto dec_wobfreq		;wenn ja berspringe diesen Befehl
	btfss BEFEHL,2			;prfen ob Bit fr Befehl "up" gesetzt ist
	goto inc_wobfreq		;Bit ist nicht gesetzt, weiter bei nchster Prfung
	btfss BEFEHL,3			;prfen ob Bit fr Befehl "down" nicht gesetzt ist
	goto dec_wobfreq		;wenn ja berspringe diesen Befehl
	goto wob_man			;zurck zur Schleife 

inc_wobfreq					;Erhhung des 10bit (0-1024) Registers WOB_COUNT_L/H um die Schrittweite WOB_STEPW. Ist der Endwert 
							;erreicht (1024 = 0x0400) wird das Register bei der nchsten Erhhung auf 0x0000 zurckgesetzt,
	btfsc WOB_COUNT_H,2		;prfen ob Endwert von 1024 erreicht ist
	goto inc_wobfreq_sub1	;ja, Endwert von 1024 ist erreicht => rcksetzen des Registers auf 0
	movf WOB_STEPW,w		;nein, "Endwert Bit" ist nicht gesetzt, erhhe Register um Schrittweite 
	addwf WOB_COUNT_L,f
	btfsc STATUS,C			;prfen ob ein bertrag vorhanden ist
	incf WOB_COUNT_H,f		;bertrag vorhanden, erhhe High Byte um 1
	goto wobfreq_end		;
inc_wobfreq_sub1
	clrf WOB_COUNT_L		;rcksetzen der Register auf 0
	clrf WOB_COUNT_H
	goto wobfreq_end
dec_wobfreq
	movf WOB_COUNT_L,w		;prfen, ob Minimalwert von 0 erreicht ist
	btfss STATUS,Z
	goto dec_wobfreq_sub1	;noch nicht 0
	movf WOB_COUNT_H,w		;
	btfsc STATUS,Z
	goto dec_wobfreq_sub2	;Minimalwert von 0 ist erreicht rcksetzen der Register auf Wert 1024
dec_wobfreq_sub1
	movf WOB_STEPW,w		;nein, "Endwert Bit" ist nicht gesetzt, reduziere Register um Schrittweite 
	subwf WOB_COUNT_L,f
	btfss STATUS,C			;prfe ob Ergebnis negativ
	decf WOB_COUNT_H,f		;bertrag vorhanden, reduziere High Byte um 1
	goto wobfreq_end		;kein bertrag vorhanden, Ergebnis ist positiv
dec_wobfreq_sub2
	movlw 0x04				;Rcksetzen des Zhlers mit Wert 1024
	movwf WOB_COUNT_H
	clrf WOB_COUNT_L
	goto wobfreq_end
wobfreq_end
	movff WOB_COUNT_L,AD_DA_L	;kopieren der WOB_COUNT Register in AD_DA Register
	movff WOB_COUNT_H,AD_DA_H
	btfss WOB_M_A,0			;prfen ob Wobbel Manuell (1) oder Automatisch (0) gewhlt ist
	goto vco_wob_dac		;Automatisch (0) ist gewhlt, weiter bei vco_wob_dac (keine Wartezeit)	
	movlw d'250'			;Manuell ist gewhlt, Wartezeit von 250ms
	call wait 		
	goto vco_wob_dac

vco_messen
	bsf	ADCON0,0			;ADC powered up
vco_loop
	btfss TA_MENUE			;Abfrage des Tasters zum Wechsel des Programms
	goto menue_1
	call clear_ZAHL_C		;Lsche Register ZAHL_C
	clrf AD_DA_L			;Es werden 16 Werte gemessen, aufsummiert und daraus der Mittelwert berechnet (durch 16 geteilt)
	clrf AD_DA_H
	movlw d'16'				;schreibe Wert 16 in Zhler
	movwf LOOP_1
loop_summ
	bsf	ADCON0,1			;ADC starten
	btfsc ADCON0,1			;prfen ob der ADC fertig ist?
	goto $-0x02				;nein, weiter warten
	movf ADRESL,w			;hole Low Byte (8 Bit) nach W und addiere zu AD/DA Register Low byte
	addwf AD_DA_L,f
	btfsc STATUS,C			;prfen ob ein bertrag vorhanden ist
	incf AD_DA_H,f			;bertrag vorhanden, erhhe High Byte um 1
	movf ADRESH,w			;hole High Byte (2 Bit) nach W und addiere zu AD/DA Register High byte
	addwf AD_DA_H,f
	btfss ADRESH,1			;prfen ob AD Wert >511 ist
	goto loop_summ_sub1		;<=511, weiter
	incfsz AD_DA_L			;>511, erhhe Low Byte um 1 und prfe auf berlauf
	goto loop_summ_sub1		;kein berlauf
	incf AD_DA_H			;berlauf vorhanden, erhhe High Byte
loop_summ_sub1
	decfsz LOOP_1			;reduziere Zhler um 1 und prfe ob 0
	goto loop_summ			;Zhler hat noch nicht bis 0 runtergezhlt, zurck zur Schleife

	clrf AD_DA_REST			;Mittelwert berechnen; AD_DA_L/AD_DA_H Register durch 16 teilen (4x nach rechts verschieben)
	bsf AD_DA_REST,3		;setze Bit 3 => wenn 4 mal geschoben, erscheint das Bit im Carry
	bcf STATUS,C			;lsche Carry Bit,
	rrcf AD_DA_H
	rrcf AD_DA_L
	rrcf AD_DA_REST
	btfss STATUS,C			;prfe, ob 4x geschoben wurde, d.h. das Carry Bit = High,
	goto $-0x0A				;Carry Bit ist noch Low, d.h. noch nicht 4 mal geschoben, zurck zur Schleife 

	btfss AD_DA_REST,7		;Rest kann zwischen 0 und 240 liegen (00000000 - 11110000), Rest 128 (10000000) entspricht 0,5
	goto vco_wob_dac		;Rest < 128 (<0,5) (0xxxxxxx)
	incfsz AD_DA_L			;Rest >= 128 (>=0,5) (1xxxxxxx), erhhe Low Byte um 1 und prfe den berlauf
	goto vco_wob_dac		;kein berlauf
	incf AD_DA_H			;berlauf vorhanden, erhhe High Byte um 1

vco_wob_dac
	movff AD_DA_L,DAC_L		;kopieren der AD_DA_L/H Register in die DAC Ausgabe Register
	movff AD_DA_H,DAC_H
	movlw d'2'				;prfen, ob DAC Wert <512 ist (0x0200)
	subwf DAC_H,w			;
	btfss STATUS,C			;
	goto $+0x0C				;ja, <512, weiter
	movlw d'1'				;nein, >=512, DAC Wert muss um 1 reduziert werden,
	subwf DAC_L,f			; da AD Wert 0-1024, DAC Wert aber nur 0-1023
	btfss STATUS,C			;
	decf DAC_H				;	

	bcf DAC_H,5				;Ausgang A (Frequenzausgabe) wird angesprochen (Bit 5 wird spter noch 2x nach links verschoben)
	bcf DAC_H,3				;setze Gain auf 0, d.h. 0-1023 entsprechen 4-4096 mV
	btfsc DAC_GAIN,0		;prfen ob gain auf 1 gesetzt werden soll
	bsf DAC_H,3				;ja, Gain = 1, d.h. 0-1023 entsprechen 2-2048 mV

	movf KENNLIN,w			;prfen, ob Kennlinie Linear oder Logarithmisch gewhlt ist
	btfss STATUS,Z
	goto vco_wob_log		;Kennlinie >0<, d.h. nicht linear, goto vco_wob_log

	call vco_wob_lin		;Kennlinie =0, d.h. linear, Aufruf Unterprogramm vco_wob_lin auf Page 3
	goto vco_wob_ausgabe

vco_wob_log					;DEK_AKTUELL
	btfss AD_DA_H,2			;prfen ob AD_DA Register >1023
	goto dekaden1 			;AD_DA Register <1024, weiter 
	clrf ZAHL_B0			;AD_DA Register >1023, lade ZAHL_B Register mit Wert 65.536 (dez.; hex: 0x00010000)
	clrf ZAHL_B1
	movlw 0x01
	movwf ZAHL_B2
	clrf ZAHL_B3
	movf DEKADE,w			;AD_DA Register >1023 und damit in der letzten Dekade, kopiere Anzahl Dekaden 
	movwf DEK_AKTUELL		; in Register fr aktuelle Dekade
	goto log_mul_dek		;weiter
dekaden1
	btfsc DEKADE,1			;prfen wieviel Dekaden gewhlt sind
	goto dekaden2			;Bit 1 ist gesetzt, d.h. 2 oder 3 Dekaden
	movlw 0x01				;Bit 1 ist nicht gesetzt, d.h. 1 Dekade
	movwf DEK_AKTUELL		;=1
	goto ad_da_neu
dekaden2
	btfsc DEKADE,0			;prfen wieviel Dekaden gewhlt sind
	goto dekaden3			;Bit 0 ist gesetzt, d.h. 3 Dekaden
	bcf STATUS,C			;lsche Carry Bit, damit es beim Rotieren nicht strt
	rlcf AD_DA_L,f			;AD_DA Register mit 2 multiplizieren ( 1x nach links verschieben)
	rlcf AD_DA_H,f
	movlw 0x02				;hole Wert 2 nach W, falls AD_DA Register >1023, d.h. Aktuelle Dekade = 2
	btfss AD_DA_H,2			;prfen ob AD_DA Register >1023
	movlw 0x01				;AD_DA Register <1024, Aktuelle Dekade = 1
	movwf DEK_AKTUELL		;Schreibe aktuelle Dekade ( 1 oder 2 ) ins DEK_AKTUELL Register
	btfsc AD_DA_H,2			;prfen ob AD_DA Register <1024
	bcf AD_DA_H,2			;AD_DA Register >1023, reduziere High Byte um 1024
	goto ad_da_neu
dekaden3
	movf AD_DA_H,w			;AD_DA Register mit 3 multiplizieren ( 1x nach links verschieben = x2 und 1x addieren)
	movwf TEMP_10			;High Byte zwischenspeichern
	movf AD_DA_L,w			;Low Byte nach W holen
	bcf STATUS,C			;lsche Carry Bit, damit es beim Rotieren nicht strt
	rlcf AD_DA_L,f			;AD_DA Register mit 2 multiplizieren ( 1x nach links verschieben)
	rlcf AD_DA_H,f
	addwf AD_DA_L,f			;Low Byte addieren
	btfsc STATUS,C			;prft, ob ein bertrag vorhanden ist
	incf AD_DA_H,f			;es ist ein bertrag vorhanden und das High Byte wird um 1 erhht
	movf TEMP_10,w			;zwischengespeichertes High Byte addieren
	addwf AD_DA_H,f
	movlw 0x03				;hole Wert 3 nach W, falls AD_DA Register >2047, d.h. Aktuelle Dekade = 3
	btfsc AD_DA_H,3			;prfen ob AD_DA Register <2048
	goto $+0x0A
	movlw 0x02				;hole Wert 2 nach W, falls AD_DA Register >1023, d.h. Aktuelle Dekade = 2
	btfss AD_DA_H,2			;prfen ob AD_DA Register >1023
	movlw 0x01				;AD_DA Register <1024, Aktuelle Dekade = 1
	movwf DEK_AKTUELL		;Schreibe aktuelle Dekade ( 1,2 oder 3 ) ins DEK_AKTUELL Register
	btfss AD_DA_H,3			;prfen ob AD_DA Register >2047
	goto $+0x08				;AD_DA Register <2048, weiter bei nchster Prfung
	bcf AD_DA_H,3			;AD_DA Register >2047, reduziere High Byte um 2048
	goto ad_da_neu
	btfsc AD_DA_H,2			;prfen ob AD_DA Register <1024
	bcf AD_DA_H,2			;AD_DA Register >1023, reduziere High Byte um 1024

ad_da_neu
	clrf AD_DA_REST			;Lsche Rest Register
	btfsc AD_DA_L,0
	bsf AD_DA_REST,0
	btfsc AD_DA_L,1
	bsf AD_DA_REST,1
	bcf STATUS,C			;lsche Carry Bit, damit es beim Rotieren nicht strt
	rrcf AD_DA_H				;AD_DA Register durch 4 teilen ( 2x nach rechts verschieben, Rest steht in AD_DA_REST)
	rrcf AD_DA_L
	bcf STATUS,C			;lsche Carry Bit, damit es beim Rotieren nicht strt
	rrcf AD_DA_H				;
	rrcf AD_DA_L

	clrf ZAHL_B2
	clrf ZAHL_B3

	call log_tabelle

	movf ZAHL_B0,w			;L-byte n, Differenz berechnen 
	subwf TEMP_12,f			;L-byte n+1
	movf ZAHL_B1,w			;H-byte n
	btfss STATUS,C
	addlw d'1'
	subwf TEMP_13,f			;H-byte n+1, Differenz steht in TEMP_13/12 Registern

	movf AD_DA_REST,w		;Differenz x Rest /4, Prfen ob Rest =0
	btfsc STATUS,Z
	goto log_mul_dek		;Rest ist 0, weiter bei log_mul_dek
	btfsc AD_DA_REST,1		;prfen ob Rest =1
	goto $+0x06				;Rest ist 2 oder 3, weiter bei nchster Prfung
	goto div_vier			;Rest ist 1, Differenz durch vier teilen

	bcf STATUS,C			;Rest ist 2 oder 3, Differenz mit 2 multiplizieren (1x nach links verschieben)
	rlcf TEMP_12
	rlcf TEMP_13
	btfss AD_DA_REST,0		;prfen ob Rest =3
	goto div_vier			;Rest ist 2, Differenz durch vier teilen
	bcf STATUS,C			;Rest ist 3
	rrcf TEMP_13,w
	rrcf TEMP_12,w	
	addwf TEMP_12,f
	btfsc STATUS,C			;prfen ob ein bertrag vorhanden ist
	incf TEMP_13,f			;es ist ein bertrag vorhanden, erhhe High Byte um 1
	bcf STATUS,C			;
	rrcf TEMP_13,w
	addwf TEMP_13,f	

div_vier
	clrf TEMP_14			;wird fr Rest genutzt
	bcf STATUS,C			;lsche Carry Bit und teile Differenz x Rest durch 4 (2x nach rechts verschieben)
	rrcf TEMP_13
	rrcf TEMP_12
	rrcf TEMP_14
	bcf STATUS,C			;lsche Carry Bit
	rrcf TEMP_13
	rrcf TEMP_12
	rrcf TEMP_14
	btfss TEMP_14,7
	goto div_vier_sub1
	incfsz TEMP_12
	goto div_vier_sub1
	incf TEMP_13
div_vier_sub1
	movf TEMP_12,w
	addwf ZAHL_B0,f
	btfsc STATUS,C			;prfen ob ein bertrag vorhanden ist
	incfsz ZAHL_B1,f		;es ist ein bertrag vorhanden, erhhe High Byte um 1
	goto $+0x06
	incf ZAHL_B2
	movf TEMP_13,w
	addwf ZAHL_B1,f
	btfsc STATUS,C			;prfen ob ein bertrag vorhanden ist
	incf ZAHL_B2,f			;es ist ein bertrag vorhanden, erhhe High Byte um 1
	clrf ZAHL_B3

log_mul_dek
	call log_mul_dek1		;Unterprogramm zum Kopieren des Multiplikators fr die aktuelle Dekade in ZAHL_A Register
	call mul32				;Aufruf des Unterprogramms zur Multiplikation (ZAHL_A x ZAHL_B = ZAHL_C)
	call div_2h16			;Aufruf des Unterprogramms zur Division durch 2^16

vco_wob_ausgabe
	movff ZAHL_C0,DDS_FREQREG_0	;Kopiere Frequenz (DDS Format) in DDS_FREQREG Register
	movff ZAHL_C1,DDS_FREQREG_1
	movff ZAHL_C2,DDS_FREQREG_2

	btfsc MESS_END,1		;prfen, ob delta t <0,2s
	goto $+0x08				;delta t <0,2s, Unterprogramme "calc_frequenz" und "convert_hex_dig" werden nicht bentigt
	call calc_frequenz		;
	call convert_hex_dig	;Aufruf des Unterprogramms zum Zerlegen der 32-Bit Zhlers in die einzelnen Digits (99.999.999)zur Anzeige auf dem Display	

	btfss PROGRAM,0			;Prfen ob Programm VCO (=4 => 00000100) oder Wobbel (=5 => 00000101) gewhlt ist (Bit 0 prfen) 
	goto vco_ausgabe		;Bit 0 ist nicht gesetzt, d.h. VCO ist gewhlt	

	btfss WOB_M_A,1			;prfen, ob Burst gewhlt ist
	goto $+0x06				;Burst ist nicht gewhlt
	call calc_burst_length	;Burst ist gewhlt, Berechnung der Burstlnge

	btfss WOB_M_A,1			;prfen, ob Burst gewhlt ist (Burst = 1, Normal = 0)
	goto wob_normal			;Burst ist nicht gewhlt

	btfsc WOB_M_A,0			;prfen ob Wobbel Manuell (1) oder Automatisch (0) gewhlt ist
	goto wob_man_burst		;Wobbel Manuell ist gewhlt, berspringe den Timer fr delta t

	btfss TA_MENUE			;Abfrage des Tasters zum Wechsel des Programms
	goto menue_1
	btfss MESS_END,7		;Abfrage, ob Timer fr delta t abgelaufen ist (von Interrupt)
	goto $-0x08				;Timer noch nicht abgelaufen, zurck zur Schleife

wob_man_burst	
	bcf INTCON,GIE			;Interrupts verbieten
	bcf MESS_END,7			;Timer flag lschen
	bcf PIE1,TMR1IE			;Timer 1 Interupt verbieten
	call send_dac_data		;Aufruf des Unterprogramms zum senden der Daten an den DAC
	clrf DDS_PERIODE_L		;Rcksetzen der Register
	movff DDS_PHASEREG_L,DDS_PERIODE_M
	movff DDS_PHASEREG_H,DDS_PERIODE_H
	movff DDS_FREQREG_0,DDS_FREQ_W0
	movff DDS_FREQREG_1,DDS_FREQ_W1
	movff DDS_FREQREG_2,DDS_FREQ_W2
	bsf T2CON,TMR2ON		;Timer 2, Timer 0 und Timer 1 einschalten
	bsf T0CON,TMR0ON
	bsf T1CON,TMR1ON
	bsf PIE1,TMR2IE			;Timer 2 Interupt erlauben	
	bsf INTCON,GIE			;Interrupts erlauben
wob_mb_sub1
	btfss TA_MENUE			;Abfrage des Tasters zum Wechsel des Programms
	goto menue_1
	btfss INTCON,TMR0IF		;Abfrage, ob Timer 0 bergelaufen ist
	goto wob_mb_sub1
	bcf INTCON,TMR0IF
	decfsz TBURST_H			;reduziere Zhler und prfe ob 0
	goto wob_mb_sub1		;Zhler noch nicht 0,

	bcf INTCON,GIE			;Burst Timer abgelaufen, globale Interrupts verbieten
	movlw d'128' 			;DAC auf Mittelstellung setzen
	movwf PORTB
	bcf PIE1,TMR2IE			;Timer 2 Interupt verbieten
	bsf PIE1,TMR1IE			;Timer 1 Interupt erlauben		
	bsf INTCON,GIE			;globale Interrupts erlauben
	
	goto wob_ausgabe

wob_normal
	btfsc WOB_M_A,0			;prfen ob Wobbel Manuell (1) oder Automatisch (0) gewhlt ist
	goto wob_man_norm		;Wobbel Manuell ist gewhlt, berspringe den Timer fr delta t
	
	btfss TA_MENUE			;Abfrage des Tasters zum Wechsel des Programms
	goto menue_1
	btfss MESS_END,7		;Abfrage, ob Timer fr Schrittweite abgelaufen ist (von Interrupt)
	goto $-0x08
	bcf MESS_END,7			;Timer flag lschen

wob_man_norm				;kopieren der DDS Frequenzregister in die Arbeitsregister
	movff DDS_FREQREG_0,DDS_FREQ_W0
	movff DDS_FREQREG_1,DDS_FREQ_W1
	movff DDS_FREQREG_2,DDS_FREQ_W2
	bsf T2CON,TMR2ON		;Timer 2 starten
	bsf T1CON,TMR1ON		;Timer 1 starten
	call send_dac_data		;Aufruf des Unterprogramms zum senden der Daten an den DAC

wob_ausgabe
	btfsc MESS_END,2
	goto wob_loop			;Bit fr keine Anzeige ist gesetzt, zurck
	btfsc MESS_END,1
	goto wob_ausgabe1		;Bit fr grafische Anzeige ist gesetzt, goto wob_ausgabe1
	call disp_vco_freq		;Normale Frequenzanzeige wird verwendet
	goto wob_loop
wob_ausgabe1
	call disp_wob_graph
	goto wob_loop

vco_ausgabe
	bcf INTCON,GIEL
	movff DDS_FREQREG_0,DDS_FREQ_W0
	movff DDS_FREQREG_1,DDS_FREQ_W1
	movff DDS_FREQREG_2,DDS_FREQ_W2
	bsf INTCON,GIEL
	call send_dac_data		;Aufruf des Unterprogramms zum senden der Daten an den DAC
	call disp_vco_freq
	bcf T1CON,TMR1ON		;Timer 1 stoppen
	movlw d'62'			;Wartezeit von 250ms
	movwf COUNT_H_S
	clrf TMR1H			;Timer 1 lschen
	clrf TMR1L
	bcf PIR1,TMR1IF			;Flag lschen
	bcf MESS_END,7			;Flag lschen
	bsf T1CON,TMR1ON		;Timer 1 starten 
vco_wait	
	btfss TA_MENUE			;Abfrage des Tasters zum Wechsel des Programms
	goto menue_1
	btfss MESS_END,7		;Abfrage Timer 1
	goto vco_wait			;kein berlauf, zurck zur Warteschleife
	bcf MESS_END,7			;Flag lschen
	goto vco_loop


vco_wob_fehler				;Fo > Fmax
	call clear_display
	btfss MESS_END,4		;prfen ob Fehler 1 anliegt
	goto fehler_2			;Fehler 1 liegt nicht an, =>weiter
	call disp_fehler1		;Fehler 1 liegt an, Aufruf des Unterprogramms zur Anzeige der Fehlermeldung "Fu/Dek zu hoch"
	goto fehler_loop
fehler_2
	call disp_fehler2		;Fehler 2 liegt an, Aufruf des Unterprogramms zur Anzeige der Fehlermeldung "Fu/dt zu klein"
fehler_loop
	call menu_befehl		;Aufruf Unterprogramm zum Laden der Tastaturbefehle, mit Men oder OK wird die Meldung quittiert
	btfss TA_MENUE			;prfen ob Bit fr Befehl Menue nicht gesetzt ist
	goto fehler_end			;wenn ja berspringe diesen Befehl, wenn nein Rcksprung in die Hauptschleife	
	btfss BEFEHL,1			;prfen ob Bit fr Befehl OK nicht gesetzt ist
	goto fehler_end			;wenn ja berspringe diesen Befehl
	goto fehler_loop
fehler_end
	movlw d'250'			;Wartezeit von 250ms
	call wait 		
	bsf MENU_STAT,0			;
	goto menue_1


menue_1
	bcf INTCON,GIE			;globale Interrupts verbieten
	clrf PIE1
	clrf T0CON				;Timer und CCP1 Register lschen 
	clrf T1CON
	clrf T2CON
	clrf CCP1CON
	movlw d'128'			;DAC auf Mittelstellung setzen
	movwf PORTB
	movlw d'4'				;laden der maximalen Anzahl Menpunkte
	movwf MENU_COUNT_MAX
	btfsc MENU_STAT,1		;prfen der Bits fr den direkten Sprung in den zuletzt genutzten Menpunkt  
	goto menu_mess_set
	btfsc MENU_STAT,2		;
	goto menu_dds_set
	btfsc MENU_STAT,3		;
	goto menu_vco_set

	call disp_menue_1		;kein Bit gesetzt fr direkten Sprung, Aufruf des Unterprogramms zur Anzeige des Hauptmens
menue1_loop
	btfss MENU_STAT,7		;prfen des Update Bits
	call disp_menue_1		;Bit ist nicht gesetzt, Update der Anzeige fr Hauptmen
	btfss TA_MENUE			;Prfen ob "Men" Taste gedrckt ist (high ist)
	goto menue_end
	call menu_befehl		;Aufruf Unterprogramm zum Laden der Tastaturbefehle
	btfss BEFEHL,3			;Prfen ob Pfeiltaste "down" nicht gedrckt ist (low ist)
	call menue_1dn			;wenn ja berspringe diesen Befehl
	btfss BEFEHL,2			;Prfen ob Pfeiltaste "up" nicht gedrckt ist (low ist)
	call menue_1up			;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,1			;Prfen ob "OK" Taste gedrckt ist (high ist)
	goto menue1_loop		;wenn ja berspringe diesen Befehl

	movlw d'1'				;
	subwf MENU_COUNT,w	;
	btfsc STATUS, Z		
	goto menu_mess_set		; Aufruf des Programmteils menu_mess_set
	movlw d'2'				;
	subwf MENU_COUNT,w	;
	btfsc STATUS, Z		
	goto menu_dds_set		; Aufruf des Programmteils menu_dds_set
	movlw d'3'				;
	subwf MENU_COUNT,w	;
	btfsc STATUS, Z		
	goto menu_vco_set		; Aufruf des Programmteils menu_vco_set
	movlw d'4'				;
	subwf MENU_COUNT,w	;
	btfsc STATUS, Z		
	goto menu_einstell_set	; Aufruf des Programmteils menu_einstell_set
	goto menue_end


menu_mess_set
	bcf INTCON,GIE			;Interrupts verbieten
	btfsc MENU_STAT,1		;prfen ob Bit nicht mehr gesetzt ist
	goto $+0x0A				;Bit ist gesetzt, weiter mit letztem Wert von TEMP_DIG_CNT
	movlw d'1'				;Bit ist nicht gesetzt, Rcksetzen des Zhlers auf 1 und Lschen des Programm Registers
	movwf TEMP_DIG_CNT
	clrf PROGRAM
	movlw d'2'				;Anzeige hat 2 Stellen, die gesetzt werden
	movwf TEMP_CNT_MAX		;
	movf PROGRAM,w			;sichern der aktuellen Messprogramms (F,P,Z) in Temp Register
	movwf TEMP_11
	movf EINGANG,w			;sichern des aktuellen Messeingangs in Temp Register
	movwf TEMP_12
	bsf TEMP_DIG_STAT,0		;setze Statusbit
	call clear_display		;Aufruf des Unterprogramms zum Lschen des Displays
	call cursor_on			;Befehl: Display On/Off Control, Cursor ON (00001110)
	call dig_mess_end		;Unterprogramm: Disp Messen, Set Cursor
loop_mess_set
	call menu_befehl		;Aufruf Unterprogramm zum Laden der Tastaturbefehle
	btfss TA_MENUE			;prfen ob Bit fr Befehl Menue nicht gesetzt ist
	goto copy_mess1			;wenn ja berspringe diesen Befehl, wenn nein Rcksprung in die Hauptschleife	
	btfss BEFEHL,1			;prfen ob Bit fr Befehl OK nicht gesetzt ist
	goto copy_mess			;wenn ja berspringe diesen Befehl
	btfss BEFEHL,4			;prfen ob Bit fr Befehl "right" nicht gesetzt ist
	call inc_dig_temp_cnt	;wenn ja berspringe diesen Befehl
	btfss BEFEHL,5			;prfen ob Bit fr Befehl "left" nicht gesetzt ist
	call dec_dig_temp_cnt	;wenn ja berspringe diesen Befehl
	btfsc TEMP_DIG_STAT,0	;prfen ob bit in register nicht gesetzt ist
	call cursor_mess		;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,2			;prfen ob Bit fr Befehl "up" gesetzt ist
	goto $+0x08				;Bit ist nicht gesetzt, weiter bei nchster Prfung
	bsf TEMP_DIG_STAT,1		;setze Statusbit fr "up"
	call dig_mess1			;wenn ja berspringe diesen Befehl
	btfss BEFEHL,3			;prfen ob Bit fr Befehl "down" nicht gesetzt ist
	call dig_mess1			;wenn ja berspringe diesen Befehl
	goto loop_mess_set		;zurck zur Schleife 

copy_mess
	bcf ME_FILT				;setze den Ausgang ME_FILT fr den gewhlten Messeingang
	btfsc EINGANG,0
	bsf ME_FILT
	call writeepr_mess		;Aufruf des Unterprogramms zum Schreiben von Einstellungswerten in das interne EEPROM
	clrf MENU_STAT			;Lsche Register und setze Bit fr Men "Messen"
	bsf MENU_STAT,1
	goto menue_end1			;weiter bei Standardroutine fr Beendigung des Mens in den Menpunkten 1-4
copy_mess1
	movff TEMP_11,PROGRAM	;rcksichern des aktuellen Programms
	movff TEMP_12,EINGANG	;rcksichern des aktuellen Messeingangs
	clrf MENU_STAT			;Lsche Register
	goto menue_1			;zurck zu Men


menu_dds_set
	btfsc MENU_STAT,2		;prfen ob Bit nicht mehr gesetzt ist
	goto $+0x08				;Bit ist gesetzt, weiter mit letztem Wert von TEMP_DIG_CNT
	movlw d'1'				;Bit ist nicht gesetzt, Rcksetzen des Zhlers auf 1
	movwf TEMP_DIG_CNT
	movlw d'12'				;Anzeige hat 12 Stellen, die gesetzt werden
	movwf TEMP_CNT_MAX		;
	movf DDS_WAVEFORM,w		;sichern der aktuellen Wellenform in Temp Register
	movwf TEMP_10
	movf DDS_VOL,w			;sichern des aktuellen Ausgangspegels in Temp Register
	movwf TEMP_11
	movf DDS_FILTER,w		;sichern des aktuellen Filter Ein/Aus in Temp Register
	movwf TEMP_12

	call clear_ZAHL_C		;lsche ZAHL_C0-ZAHL_C7
	movf DDS_PHASE_L,w		;Kopieren der 2 8-Bit DDS_PHASE-Register in die Rechenregister fr die Zerlegung in einzelne Digits
	movwf ZAHL_C0	
	movf DDS_PHASE_H,w
	movwf ZAHL_C1
	call convert_hex_dig	;Aufruf des Unterprogramms zum Zerlegen Einer 32-Bit Zahl in einzelne Digits (max.999.999.999)
	movff DISP_0,ZAHL_A0	;kopieren der DISP Register nach ZAHL_A Register, diese werden zur Anzeige des Winkels verwendet: 000,0
	movff DISP_1,ZAHL_A1
	movff DISP_2,ZAHL_A2

	call clear_ZAHL_C		;lsche ZAHL_C0-ZAHL_C7
	movff DDS_FREQ_0,ZAHL_C0;Kopieren der 3 8-Bit DDS_FREQ-Register in die Rechenregister fr die Zerlegung in einzelne Digits
	movff DDS_FREQ_1,ZAHL_C1
	movff DDS_FREQ_2,ZAHL_C2
	call convert_hex_dig	;Aufruf des Unterprogramms zum Zerlegen Einer 32-Bit Zahl in einzelne Digits (max.999.999.999)
	bsf TEMP_DIG_STAT,0		;setze Statusbit
	call clear_display		;Aufruf des Unterprogramms zum Lschen des Displays
	call cursor_on			;Befehl: Display On/Off Control, Cursor ON (00001110)
	call dig_dds_end
loop_dds_set
	call menu_befehl		;Aufruf Unterprogramm zum Laden der Tastaturbefehle
	btfss TA_MENUE			;prfen ob Bit fr Befehl Menue nicht gesetzt ist
	goto copy_dds1			;wenn ja berspringe diesen Befehl, wenn nein Rcksprung in die Hauptschleife	
	btfss BEFEHL,1			;prfen ob Bit fr Befehl OK nicht gesetzt ist
	goto copy_dds			;wenn ja berspringe diesen Befehl
	btfss BEFEHL,4			;prfen ob Bit fr Befehl "right" nicht gesetzt ist
	call inc_dig_temp_cnt	;wenn ja berspringe diesen Befehl
	btfss BEFEHL,5			;prfen ob Bit fr Befehl "left" nicht gesetzt ist
	call dec_dig_temp_cnt	;wenn ja berspringe diesen Befehl
	btfsc TEMP_DIG_STAT,0	;prfen ob bit in register nicht gesetzt ist
	call cursor_dds			;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,2			;prfen ob Bit fr Befehl "up" gesetzt ist
	goto $+0x08				;Bit ist nicht gesetzt, weiter bei nchster Prfung
	bsf TEMP_DIG_STAT,1		;setze Statusbit fr "up"
	call dig_dds1			;wenn ja berspringe diesen Befehl
	btfss BEFEHL,3			;prfen ob Bit fr Befehl "down" nicht gesetzt ist
	call dig_dds1			;wenn ja berspringe diesen Befehl
	goto loop_dds_set		;zurck zur Schleife 

copy_dds					;
	movlw d'3'				;Programm DDS
	movwf PROGRAM

	call convert_dig_hex	;Konvertierung der 6 Zahlen des Displays in die 24-Bit Zahl fr die Frequenz
	movff ZAHL_C0,DDS_FREQ_0;Kopieren der 32-Bit Zahl von den Rechenregistern in die Frequenzregister
	movff ZAHL_C1,DDS_FREQ_1
	movff ZAHL_C2,DDS_FREQ_2

	call convert_dds_winkel	;Unterprogramm zur Konvertierung der 4 Digits des Displays in 2 Phasenwinkel Register (hexadezimal)
	call calc_dds_phase		;Unterprogramm zur Berechnung des DDS Phase Register aus dem Phasenwinkel
	call calc_dds_freq		;Aufruf des Unterprogramms zur Berechnung der Frequenzregisterdaten des DDS Chips AD9833
	call writeepr_dds		;Aufruf des Unterprogramms zum Schreiben von Einstellungswerten in das interne EEPROM

	clrf MENU_STAT			;Lsche Register und setze Bit fr DDS Generator
	bsf MENU_STAT,2
	goto menue_end1			;weiter bei Standardroutine fr Beendigung des Mens in den Menpunkten 1-4
copy_dds1
	movff TEMP_10,DDS_WAVEFORM	;rcksichern der aktuellen Wellenform
	movff TEMP_11,DDS_VOL		;rcksichern des aktuellen Ausgangspegels
	movff TEMP_12,DDS_FILTER	;rcksichern des aktuellen Filter Ein/Aus
	clrf MENU_STAT			;Lsche Register
	goto menue_1			;zurck zu Men


menu_vco_set
	bcf INTCON,GIE			;Interrupts verbieten
	btfsc MENU_STAT,3		;prfen ob Bit nicht mehr gesetzt ist
	goto $+0x08				;Bit ist gesetzt, weiter mit letztem Wert von TEMP_DIG_CNT
	movlw d'1'				;Bit ist nicht gesetzt, Rcksetzen des Zhlers auf 1
	movwf TEMP_DIG_CNT

	movff PROGRAM,TEMP_10	;sichern des aktuellen Programms in Temp Register
	movff KENNLIN,TEMP_11	;sichern der aktuellen Kennlinie (Lin/Log) in Temp Register
	movff DEKADE,TEMP_12	;sichern des aktuellen Anzahl Dekaden in Temp Register
	movff WOB_STEP,TEMP_0	;sichern der aktuellen Schritte/Durchlauf in Temp Register
	movff WOB_M_A,TEMP_1	;sichern der aktuellen Einstellung (Manuell=1/Auto=0) in Temp Register
	movff WOB_ZEIT,TEMP_2	;sichern der aktuellen Zeit/Schritt in Temp Register
	movlw d'4'				;wenn Programm <4 ist, setze Programm auf 4 (VCO)	
	btfss PROGRAM,2			;prfen ob VCO (4=100) oder Wobbel (5=101) gewhlt ist 
	movwf PROGRAM			;nein, Programm ist 1,2 oder 3, setze Programm auf 4 (VCO)
	movlw d'7'
	btfss PROGRAM,0			;prfen ob VCO oder Wobbel gewhlt ist
	goto menu_vco_sub1		;VCO ist gewhlt, kopiere Wert 7 ins Register (7 Stellen, die gesetzt werden)
	movlw d'10'
	btfss WOB_M_A,0
	goto menu_vco_sub1		;Auto ist gewhlt, kopiere Wert 10 ins Register (10 Stellen, die gesetzt werden)
	movlw d'9'				;Manuell ist gewhlt, kopiere Wert 9 ins Register (9 Stellen, die gesetzt werden)
menu_vco_sub1
	movwf TEMP_CNT_MAX		;

	call clear_ZAHL_C		;Lsche ZAHL_C Register

	movff WOB_FU_L,ZAHL_C0	;kopiere Startfrequenz (x10) L/M/H-Bytes in ZAHL_C Register fr die Anzeige auf dem Display
	movff WOB_FU_M,ZAHL_C1
	movff WOB_FU_H,ZAHL_C2
	call convert_hex_dig	;
	clrf DISP_0				;Lsche Wert fr Nachkommastelle

	bsf TEMP_DIG_STAT,0		;setze Statusbit
	call cursor_on			;Befehl: Display On/Off Control, Cursor ON (00001110)
	call dig_vco_end		;Unterprogramm: Clear Display, Disp VCO/WOB, Set Cursor
loop_vco_set
	call menu_befehl		;Aufruf Unterprogramm zum Laden der Tastaturbefehle
	btfss TA_MENUE			;prfen ob Bit fr Befehl Menue nicht gesetzt ist
	goto copy_vco1			;wenn ja berspringe diesen Befehl, wenn nein Rcksprung in die Hauptschleife	
	btfss BEFEHL,1			;prfen ob Bit fr Befehl OK nicht gesetzt ist
	goto copy_vco			;wenn ja berspringe diesen Befehl
	btfss BEFEHL,4			;prfen ob Bit fr Befehl "right" nicht gesetzt ist
	call inc_dig_temp_cnt	;wenn ja berspringe diesen Befehl
	btfss BEFEHL,5			;prfen ob Bit fr Befehl "left" nicht gesetzt ist
	call dec_dig_temp_cnt	;wenn ja berspringe diesen Befehl
	btfsc TEMP_DIG_STAT,0	;prfen ob bit in register nicht gesetzt ist
	call cursor_vco			;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,2			;prfen ob Bit fr Befehl "up" gesetzt ist
	goto $+0x08				;Bit ist nicht gesetzt, weiter bei nchster Prfung
	bsf TEMP_DIG_STAT,1		;setze Statusbit fr "up"
	call dig_vco1			;wenn ja berspringe diesen Befehl
	btfss BEFEHL,3			;prfen ob Bit fr Befehl "down" nicht gesetzt ist
	call dig_vco1			;wenn ja berspringe diesen Befehl
	goto loop_vco_set		;zurck zur Schleife 

copy_vco
	call convert_dig_hex	;Aufruf des Unterprogramms zum Umwandeln der einzelnen Digits (max.999.999.999) in eine 32-Bit Zahl
	call writeepr_vco_wob	;Aufruf des Unterprogramms zum Schreiben von Einstellungswerten in das interne EEPROM
	clrf MENU_STAT			;Lsche Register und setze Bit fr Men "VCO"
	bsf MENU_STAT,3
	goto menue_end1			;weiter bei Standardroutine fr Beendigung des Mens in den Menpunkten 1-4
copy_vco1
	movff TEMP_10,PROGRAM	;rcksichern des aktuellen Programms
	movff TEMP_11,KENNLIN	;rcksichern der aktuellen Kennlinie
	movff TEMP_12,DEKADE	;rcksichern des aktuellen Anzahl Dekaden
	movff TEMP_0,WOB_STEP	;rcksichern der aktuellen Schritte/Durchlauf
	movff TEMP_1,WOB_M_A	;rcksichern der aktuellen Einstellung (Manuell=1/Auto=0)
	movff TEMP_2,WOB_ZEIT	;rcksichern der aktuellen Zeit/Schritt

	clrf MENU_STAT			;Lsche Register
	goto menue_1			;zurck zu Men


menu_einstell_set
	movlw d'4'				;laden der maximalen Anzahl Menpunkte in den Menpunktzhler
	movwf MENU_COUNT_MAX
	movff MENU_COUNT,TEMP_11;zwischenspeichern des Programmpunktes von Men1 in Temp Register
	movlw d'1'				;lade Menpunktzhler mit Wert 1
	movwf MENU_COUNT
	bcf MENU_STAT,6			;lsche Status Bit
menu_einstell_loop
	btfss MENU_STAT,6
	call disp_menu_einstell	;	
	btfss TA_MENUE			;Prfen ob "Men" Taste gedrckt ist (high ist)
	goto menue_end
	call menu_befehl		;Aufruf Unterprogramm zum Laden der Tastaturbefehle
	btfss BEFEHL,3			;Prfen ob Pfeiltaste "down" nicht gedrckt ist (low ist)
	call menue_1dn			;wenn ja berspringe diesen Befehl
	btfss BEFEHL,2			;Prfen ob Pfeiltaste "up" nicht gedrckt ist (low ist)
	call menue_1up			;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,1			;Prfen ob "OK" Taste gedrckt ist (high ist)
	goto menu_einstell_loop	;wenn ja berspringe diesen Befehl, wenn nein zurck zur Schleife

	movlw d'1'				;
	subwf MENU_COUNT,w	;
	btfsc STATUS, Z		
	goto menu_quhr_set		; Aufruf des Programmteils menu_quhr_set
	movlw d'2'				;
	subwf MENU_COUNT,w	;
	btfsc STATUS, Z		
	goto menu_q1hz_set		; Aufruf des Programmteils menu_q1hz_set
	movlw d'3'				;
	subwf MENU_COUNT,w	;
	btfsc STATUS, Z		
	goto menu_dacgain_set	; Aufruf des Programmteils menu_dacgain_set
	movlw d'4'				;
	subwf MENU_COUNT,w	;
	btfsc STATUS, Z		
	goto menu_contrast_set	; Aufruf des Programmteils menu_contrast_set
	goto menue_end

menu_quhr_set				;Menpunkt 4.1 Quarzeinstellung ber Uhr
	movff TEMP_11,MENU_COUNT;rcksichern des Programmpunktes von Men1 von Temp Register in MENU_COUNT Register

	clrf ZAHL_A0			;wird fr Sekunden verwendet
	clrf ZAHL_A1			;wird fr Minuten verwendet
	clrf ZAHL_A2			;wird fr Stunden verwendet

	call cursor_off
	call clear_display
	call disp_uhr
	movlw d'250'			;Wartezeit von 250ms
	call wait 		

	clrf STAT_INT
	bsf STAT_INT,3
	clrf MESS_END
	clrf PIE1
	bsf PIE1,TMR1IE			;Timer 1 Interrupt erlauben
	movlw b'00000000'		;TMR1 Off (Bit0), Prescaler 1:1
	movwf T1CON

	movlw HIGH_BYTE_S		;Zhler auf 1 sek einstellen
	movwf COUNT_H_S
	movwf T1_ZEIT_H
	movlw LOW_BYTE_S
	movwf T1_ZEIT_L
	movwf T1_ZEIT_L1
	movwf TMR1H	
	movlw 0xFF
	movwf COUNT_ADJ
	movwf T1_ZEIT_LOOP
	clrf TMR1L
quhr_set1
	btfss TA_MENUE			;prfen ob Bit fr Befehl Menue nicht gesetzt ist
	goto copy_quhr			;wenn ja berspringe diesen Befehl, wenn nein Rcksprung in die Hauptschleife
	call menu_befehl		;Aufruf Unterprogramm zum Laden der Tastaturbefehle
	
	btfsc BEFEHL,1			;prfen ob Bit fr Befehl OK nicht gesetzt ist
	goto quhr_set1			;wenn ja berspringe diesen Befehl

	bsf T1CON,TMR1ON		;Mit dem OK Befehl wird die Uhr gestartet, Timer 1 starten
	bcf PIR1, TMR1IF		;Timer 1 Interrupt flag lschen
	bsf INTCON,GIE			;Globale Interrupts erlauben

	movlw d'250'			;Wartezeit von 250ms
	call wait 		
inc_uz_loop
	btfss TA_MENUE			;prfen ob Bit fr Befehl Menue nicht gesetzt ist
	goto copy_quhr			;wenn ja berspringe diesen Befehl, wenn nein Rcksprung in die Hauptschleife
	call menu_befehl		;Aufruf Unterprogramm zum Laden der Tastaturbefehle	
	btfss BEFEHL,1			;prfen ob Bit fr Befehl OK nicht gesetzt ist
	goto copy_quhr			;wenn ja berspringe diesen Befehl, wenn nein Rcksprung in die Hauptschleife

	btfss MESS_END,7
	goto inc_uz_loop	
	bcf MESS_END,7

	incf	ZAHL_A0,f		; 1 Sekunden erhhen
	movlw	D'60'
	subwf	ZAHL_A0,w
	btfss	STATUS,Z	
	goto	inc_uz_end		; 1 Sekunden <> 10	

	clrf	ZAHL_A0
	incf	ZAHL_A1,f		; 1 Minuten erhhen
	movlw	D'60'
	subwf	ZAHL_A1,w
	btfss	STATUS,Z	
	goto	inc_uz_end		;  1 Minuten <> 10

	clrf	ZAHL_A1
	incf	ZAHL_A2,f		; 1 Stunden erhhen
	movlw	D'24'
	subwf	ZAHL_A2, w
	btfss	STATUS,Z	
	goto	inc_uz_end		; 1 Stunden <> 10

	clrf	ZAHL_A2

inc_uz_end
	call disp_uhr
	goto inc_uz_loop

copy_quhr
	goto menue_end			;weiter bei Standardroutine fr Beendigung des Mens


menu_q1hz_set				;Menpunkt 4.2 Quarzeinstellung ber DCF 1Hz Signal
	movff TEMP_11,MENU_COUNT;rcksichern des Programmpunktes von Men1 von Temp Register in MENU_COUNT Register	

	clrf STAT_INT
	bsf STAT_INT,3			;setze Bit fr Timer1 Sekunden Interrupt Routine
	call clear_ZAHL_C		;lsche Register ZAHL_C
	clrf ZAHL_A0			;lsche Register ZAHL_A
	clrf ZAHL_A1
	clrf ZAHL_A2
	clrf ZAHL_A3

	clrf DISP_0				;lsche Register DISP
	clrf DISP_1
	clrf DISP_2
	clrf DISP_3

	call clear_display		;lsche Display
	call disp_q1hz			;sende an Display -----ppm-----s
	movlw 0x81				;Displayadresse
	call disp_q1hz_dat		;sende an Display -0000, Abweichung in ppm
	movlw 0x89				;Displayadresse
	call disp_q1hz_dat		;sende an Display ---------0000, abgelaufene Sekunden

	clrf MESS_END
	clrf T1CON

	movlw HIGH_BYTE_S		;Zhler auf 1 sek einstellen
	movwf COUNT_H_S
	movwf T1_ZEIT_H
	movlw LOW_BYTE_S
	movwf T1_ZEIT_L
	movwf T1_ZEIT_L1
	movwf TMR1H	
	movlw 0xFF
	movwf COUNT_ADJ
	movwf T1_ZEIT_LOOP
	clrf TMR1L
	movlw b'00000101'		;Bit 3-0: Capture mode: every rising edge
	movwf CCP1CON
	bcf PIR1,TMR1IF			;Timer 1 Interrupt flag lschen
	bcf PIR1,CCP1IF			;Capture Interrupt Flag lschen
	clrf PIE1
	bsf INTCON,GIE			;Globale Interups erlauben
							;Warten auf die positive Flanke von RC2
	btfss TA_MENUE			;Abfrage des Tasters zum Wechsel des Programms (Abbruch)
	goto menue_end
	btfss PIR1,CCP1IF		;Abfrage der positiven Flanke von RC2
	goto $-0x08				;noch keine positive Flanke, zurck zur Warteschleife
	bsf T1CON,TMR1ON		;positive Flanke liegt an, Beginn der Messung, Timer 1 starten
	bcf PIR1,CCP1IF			;lsche CCP1 Flag
	clrf MESS_END			;

	bcf PIR1, TMR1IF		;lsche Timer1 berlaufflag
	bsf PIE1,TMR1IE			;Timer 1 Interrupt erlauben

q1hz_loop
	btfss TA_MENUE			;Abfrage des Men Tasters
	goto menue_end
	btfss MESS_END,7		;Prft ob eine Sekunde abgelaufen ist 
	goto q1hz_loop1
	bcf MESS_END,7
	goto int_first
q1hz_loop1
	btfss PIR1,CCP1IF		;Prft die positive Flanke von RB0 
	goto q1hz_loop
	bcf PIR1,CCP1IF
	goto ext_first

int_first
	btfss TA_MENUE			;Abfrage des Men Tasters
	goto menue_end
	btfsc PIR1,CCP1IF		;Prft die positive Flanke von RB0 
	goto int_first1
	movlw d'146'			; => 0,202sek, Abbruch wenn nach 0,202sek (z.B. DCF 59ste Sek) keine positive Flanke
	subwf COUNT_H_S,w		;
	btfss STATUS,Z
	goto int_first
int_first1
	bcf PIR1,CCP1IF			;lsche CCP1 Flag
	movff COUNT_H_S,ZAHL_C2	;sichern der Timer 1 Werte im ZAHL_C Register
	movff CCPR1L,ZAHL_C0
	movff CCPR1H,ZAHL_C1
	movlw LOW_BYTE_S
	subwf ZAHL_C1,f
	movlw HIGH_BYTE_S
	subfwb ZAHL_C2,f
	movlw A'+'				;Vorzeichen fr Abweichung
	goto auswert_q1hz

ext_first
	movff COUNT_H_S,ZAHL_C2	;sichern der Timer 1 Werte im ZAHL_C Register
	movff CCPR1L,ZAHL_C0
	movff CCPR1H,ZAHL_C1
ext_first1
	btfss TA_MENUE			;Abfrage des Men Tasters
	goto menue_end
	btfss MESS_END,7		;Prft ob eine Sekunde abgelaufen ist 
	goto ext_first1
	bcf MESS_END,7			;Lsche Sekunden berlauf bit
	movlw d'0'
	subfwb ZAHL_C0,f
	movlw d'0'
	subfwb ZAHL_C1,f
	movlw d'0'
	subwfb ZAHL_C2,f
	movlw A'-'				;Vorzeichen fr Abweichung	

auswert_q1hz
	movwf TEMP_18			;Vorzeichen fr Abweichung
	incfsz ZAHL_A0			;erhhe L-Byte des Sekundenzhler um 1 und prfe auf berlauf
	goto $+0x06				;kein berlauf
	incf ZAHL_A1			;berlauf vorhanden, Erhhe H-Byte um 1
	movff ZAHL_A0,TEMP_11	;sichern der Sekunden in Temp register
	movff ZAHL_A1,TEMP_12	;
	call div32				;Teile die Abweichung (us x 12) durch abgelaufene Sekunden => ppm (x 12)
	movlw d'12'				;ppm (x 12) durch 12 teilen => ppm, lade Divisionsregister ZAHL_A mit Wert 12
	movwf ZAHL_A0
	clrf ZAHL_A1
	clrf ZAHL_A2
	clrf ZAHL_A3
	call div32				;Teile ppm (x 12) durch 12 => ppm
	call runden_div32		;ganzzahliges Runden Ergebnis in ZAHL_C
	call convert_hex_dig	;Zerlegen einer 32 Bit Zahl (ZAHL_C) in die einzelnen Digits (max 999.999.999)
	call disp_q1hz_vz		;sende an Display Vorzeichen der Abweichung +/-
	movlw 0x81				;Displayadresse
	call disp_q1hz_dat		;sende an Display -0000, Abweichung in ppm

	movff TEMP_11,ZAHL_C0	;kopiere die abgelaufenen Sekunden nach ZAHL_C
	movff TEMP_12,ZAHL_C1
	call convert_hex_dig	;Zerlegen einer 32 Bit Zahl (ZAHL_C) in die einzelnen Digits (max 999.999.999)
	movlw 0x89				;Displayadresse
	call disp_q1hz_dat		;sende an Display ---------0000, abgelaufene Sekunden

	movff TEMP_11,ZAHL_A0	;Rcksichern der Temp Register
	movff TEMP_12,ZAHL_A1	;Rcksichern der Temp Register
	call clear_ZAHL_C		;lsche Register
	goto q1hz_loop			;Zurck zur Schleife



menu_dacgain_set			;Menpunkt 4.3 DAC Ausgangssignal 0-2V oder 0-4V
	movff TEMP_11,MENU_COUNT;rcksichern des Programmpunktes von Men1 von Temp Register in MENU_COUNT Register
	movff DAC_GAIN,TEMP_11	;Sichern der aktuellen DAC Verstrkung in Temp Register
	call clear_display		;Aufruf des Unterprogramms zum Lschen des Displays
	call disp_set_dacgain	;Aufruf des Unterprogramms zur Anzeige auf dem Display
	movlw d'250'			;Wartezeit von 250ms
	call wait 			
loop_dacgain_set
	call menu_befehl		;Aufruf Unterprogramm zum Laden der Tastaturbefehle
	btfss TA_MENUE			;prfen ob Bit fr Befehl Menue nicht gesetzt ist
	goto copy_dacgain1		;wenn ja berspringe diesen Befehl, wenn nein Rcksprung in die Hauptschleife	
	btfss BEFEHL,1			;prfen ob Bit fr Befehl OK nicht gesetzt ist
	goto copy_dacgain		;wenn ja berspringe diesen Befehl
	btfss BEFEHL,2			;prfen ob Bit fr Befehl "up" nicht gesetzt ist
	call dig_dacgain		;Bit ist gesetzt, Aufruf des Unterprogramms
	btfss BEFEHL,3			;prfen ob Bit fr Befehl "down" nicht gesetzt ist
	call dig_dacgain		;Bit ist gesetzt, Aufruf des Unterprogramms
	goto loop_dacgain_set	;zurck zur Schleife 

copy_dacgain
	movff DAC_GAIN,TEMP_13	;Kopiere den genderten Wert ins Schreibregister fr das EEPROM
	movlw 0x12				;Lade die Adresse des EEPROM in die der Wert geschrieben werden soll
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM
	goto menue_end			;weiter bei Standardroutine fr Beendigung des Mens
copy_dacgain1
	movff TEMP_11,DAC_GAIN	;Rcksichern der aktuellen DAC Verstrkung
	goto menue_end			;weiter bei Standardroutine fr Beendigung des Mens


menu_contrast_set			;Menpunkt 4.4 Kontrast Einstellung
	movff TEMP_11,MENU_COUNT;rcksichern des Programmpunktes von Men1 von Temp Register in MENU_COUNT Register
	movff CONTRAST,TEMP_11	;Kontrastwert in Temp Register sichern
	call clear_display		;Aufruf des Unterprogramms zum Lschen des Displays
	call disp_set_contrast	;Anzeigen des Kontrast Wertes
contr_set_loop
	btfsc TEMP_DIG_STAT,0	;prfen, ob Wert sich nicht gendert hat
	call disp_set_contrast	;Wert hat sich gendert, Anzeigen des neuen Kontrast Wertes
	btfss TA_MENUE			;prfen ob Menue Taste nicht gedrckt ist
	goto contr_set_end1
	call menu_befehl		;Aufruf des Unterprogramms zum laden der Tastenzustnde nach Befehl
	btfss BEFEHL,1			;prfen, ob OK Taste nicht gedrckt ist
	goto contr_set_end
	btfss BEFEHL,2			;prfen, ob Up Taste nicht gedrckt ist
	call inc_contrast
	btfss BEFEHL,3			;prfen, ob Down Taste nicht gedrckt ist
	call dec_contrast
	goto contr_set_loop
contr_set_end				;Einstellungen sollen bernommen werden
	movff CONTRAST,TEMP_13	;Speichern des Kontrast Wertes im internen EEPROM
	movlw 0x13				;EEPROM Adresse
	call writeEE
	goto menue_end
contr_set_end1				;Einstellungen sollen nicht bernommen werden
	movff TEMP_11,CONTRAST	;Rcksichern des in Temp Register gespeicherten Kontrast Wertes
	goto menue_end

menue_end					;Standardroutine fr Beendigung des Mens
	clrf MENU_STAT			;Lsche Register
menue_end1					;Standardroutine fr Beendigung des Mens in den Menpunkten 1-4
	movlw d'250'			;Wartezeit von 250ms
	call wait 			
	call cursor_off
	call clear_display
	goto main

;*******************************************************************************************
;
;	Unterprogramme Frequenz/Periode
;
;*******************************************************************************************

fliesskomma1
	call mul32				; ZAHL_A x ZAHL_B = ZAHL_C

	movff DISP_0,ZAHL_A0	;Rcksichern der Mezeit in Nennerregister (ZAHL_A) fr die Division
	movff DISP_1,ZAHL_A1
	movff DISP_2,ZAHL_A2
	movff DISP_3,ZAHL_A3
	call div32				;Division durch die Mezeit (ZAHL_C / ZAHL_A = ZAHL_C, Rest in ZAHL_B)

	movff ZAHL_C0,DISP_0	;temporres sichern des Zwischenergebnisses (Rest x Faktor / Mezeit) in Displayregistern
	movff ZAHL_C1,DISP_1
	movff ZAHL_C2,DISP_2
	movff ZAHL_C3,DISP_3

	movff DISP_4,ZAHL_B0	;Rcksichern der ganzzahligen Frequenz in Multiplikationsregister (ZAHL_B) fr die Multiplikation
	movff DISP_5,ZAHL_B1
	movff DISP_6,ZAHL_B2
	movff DISP_7,ZAHL_B3
	return

fliesskomma2
	call mul32				;Multiplizieren der ganzzahligen Frequenz mit Faktor: ZAHL_A x ZAHL_B = ZAHL_C
	movff DISP_0,ZAHL_B0	;Rcksichern des Zwischenergebnisses (Rest x Faktor / Mezeit) in Additionsregister ZAHL_B
	movff DISP_1,ZAHL_B1	; der ganzzahlige Teil steht noch im zweiten Additionsregister ZAHL_C
	movff DISP_2,ZAHL_B2
	movff DISP_3,ZAHL_B3
	call add32				;Das Ergebnis der Addition steht in ZAHL_C (ZAHL_B + ZAHL_C = ZAHL_C)
	return

;*******************************************************************************************
;
;	Unterprogramme DDS / VCO / Wobbel
;
;*******************************************************************************************

init_dds
	clrf DDS_WAVE			;
	movf DDS_WAVEFORM,w		;hole Wellenform nach W: 0=Sinus, 1=Dreieck, 2=Sgezahn, 3=Rechteck
	btfsc STATUS,Z			;prfe ob >0<
	bsf DDS_WAVE,0			; =0, setze Bit fr Sinus
	movlw d'1'				;prfe ob Wellenform = 1
	subwf DDS_WAVEFORM,w	;
	btfsc STATUS,Z			;prfe ob >0<
	bsf DDS_WAVE,1			; =0, setze Bit fr Dreieck
	movlw d'2'				;prfe ob Wellenform = 2
	subwf DDS_WAVEFORM,w	;
	btfsc STATUS,Z			;prfe ob >0<
	bsf DDS_WAVE,2			; =0, setze Bit fr Sgezahn
	bsf DDS_WAVE,3			; >0<, setze Bit fr Rechteck
	
	btfss DDS_FILTER,0		;setzen des Ausgangs fr Filter: prfen ob Bit fr Filter Ein gesetzt ist
	bcf ME_FILT				;Bit ist nicht gesetzt, lsche Ausgang
	btfsc DDS_FILTER,0		;prfen ob Bit fr Filter Ein nicht gesetzt ist
	bsf ME_FILT				;Bit ist gesetzt, setze Ausgang
	
	call dds_pegel			;Aufruf des Unterprogramms zur Ansteuerung des Digitalpoti fr den DDS Pegel
	call calc_dds_phase		;Berechnung des Phasenregisters aus dem Phasenwinkel
	return

convert_dds_winkel
	movff ZAHL_A0,DISP_0
	movff ZAHL_A1,DISP_1
	movff ZAHL_A2,DISP_2
	clrf DISP_3
	clrf DISP_4
	clrf DISP_5
	clrf DISP_6
	clrf DISP_7
	clrf DISP_8

	call convert_dig_hex		;Unterprogramm zum Umwandeln der einzelnen Digits (max.999.999.999) in eine 32-Bit Zahl

	movff ZAHL_C0,DDS_PHASE_L	;kopieren der Register
	movff ZAHL_C1,DDS_PHASE_H	;kopieren der Register
	return

calc_dds_phase					;Berechnung des Phasenregisters aus dem Phasenwinkel
	call clear_ZAHL_C			;DDS Phasenregister = Phasenwinkel () x 2^16 / 360
	movff DDS_PHASE_L,ZAHL_C2	;Lade den Phasenwinkel in die ZAHL_C2/C3 Register fr die Division
	movff DDS_PHASE_H,ZAHL_C3	;(ZAHL_C2/C3 Register entsprechen einer Multiplikation mit 2^16)

	movlw 0x68					;kopiere Wert fr 360 (0x0168) in das Nennerregister fr die Division (ZAHL_A)
	movwf ZAHL_A0
	movlw 0x01
	movwf ZAHL_A1
	clrf ZAHL_A2
	clrf ZAHL_A3

	call div32					;Teile ZAHL_C durch ZAHL_A, das Ergebnis steht in ZAHL_C, der Rest in ZAHL_B

	movff ZAHL_C0,DDS_PHASEREG_L
	movff ZAHL_C1,DDS_PHASEREG_H
	return

calc_dds_freq					;Berechnung des Wertes fr das Frequenzregister, FREQREG = 2^24 * Fdds (x10) / MCLK (x10)
	call clear_ZAHL_C			;lade Zahl_C mit Fdds (x10) (Laden in ZAHL_C3/C4/C5 entspricht einer Multiplikation mit 2^24)
	movff DDS_FREQ_0,ZAHL_C3	;
	movff DDS_FREQ_1,ZAHL_C4
	movff DDS_FREQ_2,ZAHL_C5

	movlw 0x80					;Lade DDS_MCLK (x10) in ZAHL_A Register: DDS_MCLK = 48.000.000Hz / 4 / 100Takte = 120.000Hz
	movwf ZAHL_A0				; DDS_MCLK (x10) = 1.200.000 = 0x00124F80
	movlw 0x4F
	movwf ZAHL_A1
	movlw 0x12
	movwf ZAHL_A2
	movlw 0x00
	movwf ZAHL_A3
	call div32					;Teile ZAHL_C durch ZAHL_A, das Ergebnis steht in ZAHL_C

	movff ZAHL_C0,DDS_FREQREG_0	;Kopiere das Ergebnis in die DDS_FREQREG Register
	movff ZAHL_C1,DDS_FREQREG_1
	movff ZAHL_C2,DDS_FREQREG_2

	movlw d'5'					;prfen, ob die Nachkommastelle grer oder gleich 5 ist
	subwf ZAHL_B0,w
	btfsc STATUS,C
	incfsz DDS_FREQREG_0		;Nachkommastelle ist grer oder gleich 5 => erhhe Frequenzregister_0 um 1
	return						;
	incfsz DDS_FREQREG_1		;es ist ein berlauf aufgetreten, erhhe Frequenzregister_1
	return						;
	incf DDS_FREQREG_2			;es ist ein berlauf aufgetreten, erhhe Frequenzregister_2
	return


;*******************************************************************************************
;
;	Unterprogramme VCO / Wobbel
;
;*******************************************************************************************

prep_vco_wob
	bcf INTCON,GIE					;Interups verbieten	
	clrf T1CON	
	clrf MESS_END				;lsche Register, Bit 0: berlauf 10-Bit Zhler inc/dec wob, Bit 1: dt<0,2s, Bit 3: Fehler allgemein
								; Bit 4: 1=Fehler Fu/Dek zu hoch, 0=Fehler Fu/dt zu klein, Bit 5-6 frei, Bit 7: dt abgelaufen

	movff WOB_FU_L,ZAHL_A0		;lade Zahl_A mit Fu (Wobbel/VCO)
	movff WOB_FU_M,ZAHL_A1
	movff WOB_FU_H,ZAHL_A2
	clrf ZAHL_A3

	clrf ZAHL_B1	
	movlw 0x01					;prfen, ob 1 Dekade gewhlt ist
	subwf DEKADE,w
	btfss STATUS,Z
	goto dekade2
	movlw 0x0A					;1 Dekade, lade Zahl_B mit Wert 10
	movwf ZAHL_B0
	goto dekade_end
dekade2
	movlw 0x02					;prfen, ob 2 Dekaden gewhlt sind
	subwf DEKADE,w
	btfss STATUS,Z
	goto dekade3
	movlw 0x64					;2 Dekaden, lade Zahl_B mit Wert 100
	movwf ZAHL_B0	
	goto dekade_end	
dekade3
	movlw 0xE8					;3 Dekaden, lade Zahl_B mit Wert 1000
	movwf ZAHL_B0
	movlw 0x03
	movwf ZAHL_B1
dekade_end
	clrf ZAHL_B2
	clrf ZAHL_B3
	call mul32					;Multipliziere ZAHL_A mit ZAHL_B, Fo (x10) steht in den Registern ZAHL_C0 bis ZAHL_C7


	bsf MESS_END,3				;setze vorsichtshalber Fehler Bits allgemein und fr zu hohe Fu bzw. zu viele Dekaden
	bsf MESS_END,4

	movlw 0xC0					;prfen ob Wert fr obere Frequenz (x10) < Maximalwert von MCLK/2 (x10) ist
	movwf TEMP_10				;MCLK = 48.000.000Hz/4/100 = 120.000Hz => MCLK/2 (x10) = 600.000 (dez) = 0x000927C0
	movlw 0x27
	movwf TEMP_11				;  Wert fr obere Frequenz Fo steht noch in ZAHL_C
	movlw 0x09
	movwf TEMP_12
	movlw 0x00
	call test_kleiner			;Unterprogramm zum prfen, ob ZAHL_C < Wert in TEMP/W-Reg.
	btfsc STATUS,C
	return						;obere Frequenz >= Maximalwert , zurck zur Schleife mit Fehlermeldung

	bcf MESS_END,3				;obere Frequenz < Maximalwert , lsche Fehler Bit fr zu hohe Fu bzw. zu viele Dekaden
	bcf MESS_END,4				;
								
	movff ZAHL_C2,ZAHL_C5		;  fr die oberen Frequenzen jeder Dekade:
	movff ZAHL_C1,ZAHL_C4		;  Freg = Fo (x10) x 2^24 / (10 x MCLK)
	movff ZAHL_C0,ZAHL_C3		;  Fo (x10) fr die hchste Dekade steht noch in ZAHL_C Registern
	clrf ZAHL_C0				;  Multiplikation mit 2^24 heisst, alles 3 Register nach links schieben
	clrf ZAHL_C1
	clrf ZAHL_C2
	clrf ZAHL_C6
	clrf ZAHL_C7

	movlw 0x80					;lade MCLK x10 in ZAHL_A Divisionsregister:
	movwf ZAHL_A0				;MCLK = 48.000.000Hz/4/100 = 120.000Hz => MCLK (x10) = 1.200.000 (dez) = 0x00124F80
	movlw 0x4F
	movwf ZAHL_A1
	movlw 0x12
	movwf ZAHL_A2
	clrf ZAHL_A3

	call div32					;Division Zahl_C / Zahl_A = Zahl_C, Rest steht in Zahl_B
	call runden_div32			;Aufruf des Unterprogramms zum ganzzahligen Runden des Ergebnisses der Division

	movf KENNLIN,w				;prfen, ob Kennlinie Linear oder Logarithmisch gewhlt ist
	btfsc STATUS,Z
	goto calc_delta_lin			;Kennlinie =0, d.h. linear, weiter bei Berechnungen fr lineare Kennlinie
								;Kennlinie >0<, d.h. logarithmisch, Berechnung der Frequenzregisterwerte 
	movlw 0x01					;prfen ob 1, 2 oder 3 Dekaden gewhlt sind
	subwf DEKADE,w
	btfss STATUS,Z
	goto fo_dek2_dds
	call copy_fd1dds			;1 Dekade ist gewhlt, kopiere ZAHL_C Register (Fo Wert DDS) in FD1DDS Register
	goto prep_wobbel			;weiter bei prep_wobbel
fo_dek2_dds
	movlw 0x02					;
	subwf DEKADE,w
	btfss STATUS,Z
	goto fo_dek3_dds
	call copy_fd2dds			;2 Dekaden sind gewhlt, kopiere ZAHL_C Register (Fo Wert Dek2 DDS) in FD2DDS Register
	movlw d'10'					;durch 10 teilen: Lade Wert 10 in ZAHL_A Register
	movwf ZAHL_A0
	clrf ZAHL_A1
	clrf ZAHL_A2
	clrf ZAHL_A3
	call div32					;Teile ZAHL_C durch ZAHL_A, das Ergebnis steht in ZAHL_C, die Nachkommastelle in ZAHL_B0
	call runden_div32			;Aufruf des Unterprogramms zum ganzzahligen Runden des Ergebnisses der Division
	call copy_fd1dds			;kopiere ZAHL_C Register (Fo Wert Dek1 DDS) in FD1DDS Register
	goto prep_wobbel			;weiter bei prep_wobbel
fo_dek3_dds
	call copy_fd3dds			;3 Dekaden sind gewhlt, kopiere ZAHL_C Register (Fo Wert DDS) in FD3DDS Register
	movlw d'10'					;durch 10 teilen: Lade Wert 10 in ZAHL_A Register
	movwf ZAHL_A0
	clrf ZAHL_A1
	clrf ZAHL_A2
	clrf ZAHL_A3
	call div32					;Teile ZAHL_C durch ZAHL_A, das Ergebnis steht in ZAHL_C, die Nachkommastelle in ZAHL_B0
	call runden_div32			;Aufruf des Unterprogramms zum ganzzahligen Runden des Ergebnisses der Division
	call copy_fd2dds			;kopiere ZAHL_C Register (Fo Wert Dek2 DDS) in FD2DDS Register	
	movlw d'10'					;durch 10 teilen: Lade Wert 10 in ZAHL_A Register
	movwf ZAHL_A0
	clrf ZAHL_A1
	clrf ZAHL_A2
	clrf ZAHL_A3
	call div32					;Teile ZAHL_C durch ZAHL_A, das Ergebnis steht in ZAHL_C, die Nachkommastelle in ZAHL_B0
	call runden_div32			;Aufruf des Unterprogramms zum ganzzahligen Runden des Ergebnisses der Division
	call copy_fd1dds			;kopiere ZAHL_C Register (Fo Wert Dek1 DDS) in FD1DDS Register
	goto prep_wobbel			;weiter bei prep_wobbel	
	

calc_delta_lin					;ZAHL_C - ZAHL_B = ZAHL_C
	call copy_fd3dds			;speichern des DDS Wertes fr Fo im FD3DDS Register

	clrf ZAHL_A1				;Berechnung DDS Wert fr Fu: 1 Dek.: Fo / 10,  2 Dek.: Fo / 100,  3 Dek.: Fo / 1000
	clrf ZAHL_A2				;lsche ZAHL_A Register
	clrf ZAHL_A3

	movlw d'1'					;prfen, ob 1 Dekade gewhlt ist
	subwf DEKADE,w
	btfss STATUS,Z
	goto calc_dlin_sub1			;Anzahl Dekaden >1<, weiter bei nchster Prfung
	movlw 0x0A					;Anzahl Dekaden =1, Lade Wert 10 (0x0A) in Nenner Register fr die Division
	movwf ZAHL_A0
	goto calc_fu_value
calc_dlin_sub1
	movlw d'2'					;prfen, ob 2 Dekaden gewhlt sind
	subwf DEKADE,w
	btfss STATUS,Z
	goto calc_dlin_sub2			;Anzahl Dekaden >2<
	movlw 0x64					;Anzahl Dekaden =2, Lade Wert 100 (0x64) in Nenner Register fr die Division
	movwf ZAHL_A0
	goto calc_fu_value
calc_dlin_sub2
	movlw 0xE8					;Anzahl Dekaden =3, Lade Wert 1000 (0x03E8) in Nenner Register fr die Division
	movwf ZAHL_A0
	movlw 0x03
	movwf ZAHL_A1

calc_fu_value					;Teile DDS Wert fr Fo (ZAHL_C) durch 10, 100 oder 1000 (ZAHL_A)
	call div32					;Teile ZAHL_C durch ZAHL_A, das ganzzahlige Ergebnis steht in ZAHL_C, der Rest in ZAHL_B

	call runden_div32			;Aufruf des Unterprogramms zum ganzzahligen Runden des Ergebnisses der Division

	call copy_fd1dds			;speichern des DDS Wertes fr Fu (ZAHL_C) im FD1DDS Register

								;Berechnung der Differenz zwischen den DDS Werten fr Fo und Fu: Fo - Fu = Fd
	movff ZAHL_C0,ZAHL_B0		;ZAHL_C - ZAHL_B = ZAHL_C, Kopiere ZAHL_C nach ZAHL_B
	movff ZAHL_C1,ZAHL_B1
	movff ZAHL_C2,ZAHL_B2
	movff ZAHL_C3,ZAHL_B3

	movff FD3DDS_0,ZAHL_C0		;Kopiere DDS Wert fr Fo von FD3DDS Registern nach ZAHL_C
	movff FD3DDS_1,ZAHL_C1
	movff FD3DDS_2,ZAHL_C2
	clrf ZAHL_C3

	call sub32					;Subtrahiere ZAHL_B von ZAHL_C, das Ergebnis steht in ZAHL_C (Fd)

	bcf STATUS,C				;( Fo - Fu ) x 2^8 / 2^10 = ( Fo - Fu ) / 4 => Register 2x nach rechts schieben
	rrcf ZAHL_C3
	rrcf ZAHL_C2
	rrcf ZAHL_C1
	rrcf ZAHL_C0
	bcf STATUS,C				;
	rrcf ZAHL_C3
	rrcf ZAHL_C2
	rrcf ZAHL_C1
	rrcf ZAHL_C0

	btfss STATUS,C				;prfen, ob das letzte ausgeschobene Bit =1, d.h. Rest >0,5
	goto calc_fu_val1			;Rest <0,5, weiter
	incfsz ZAHL_C0				;Rest >0,5, erhhe Register um 1 und prfe auf berlauf
	goto calc_fu_val1			;kein berlauf
	incfsz ZAHL_C1				;berlauf vorhanden, erhhe Register um 1 und prfe auf berlauf
	goto calc_fu_val1			;kein berlauf
	incfsz ZAHL_C2				;berlauf vorhanden, erhhe Register um 1 und prfe auf berlauf
	goto calc_fu_val1			;kein berlauf
	incf ZAHL_C3				;berlauf vorhanden, erhhe Register um 1

calc_fu_val1
	call copy_fd2dds			;kopiere Ergebnis der Berechnungen ( Fo dds - Fu dds ) x 256 /1024 in FD2DDS Register


prep_wobbel					;Unterprogramm zur Vorbereitung der bentigten Parameter
	btfss PROGRAM,0			;Prfen ob Programm VCO (=4 => 00000100) oder Wobbel (=5 => 00000101) gewhlt ist (Bit 0 prfen) 
	return					;Bit 0 ist nicht gesetzt, d.h. VCO ist gewhlt =>zurck zur Hauptschleife

	clrf AD_DA_L			;lschen der Register
	clrf AD_DA_H
	clrf WOB_COUNT_L		;lschen der Register, diese werden als Zhler fr die Schrittweite benutzt
	clrf WOB_COUNT_H

	movff WOB_STEP,TEMP_0	;kopiere Wert in Temp Register: Schritte/Durchlauf: 1=1024, 2=512, 3=256, 4=128, 5=64, 6=32
	clrf WOB_STEPW			;lsche Schrittweiten Register, setze Carry Bit und schiebe Schrittweiten Register nach links bis Temp Register =0
	bsf STATUS,C			;setze Carry Bit
prep_wob_loop
	rlcf WOB_STEPW			;1x schieben: 00000001 =1, 2x schieben: 00000010 =2, 3x schieben: 00000100 =4,
	decfsz TEMP_0			;4x schieben: 00001000 =8, 5x schieben: 00010000 =16, 6x schieben: 00100000 =32,
	goto prep_wob_loop		;Zhler noch nicht 0, zurck zur Schleife


para_delta_t				;dt = Zeit/Schritt: 0=10ms, 1=20ms, 2=50ms, 3=100ms, 4=200ms, 5=500ms, 6=1000ms, 7=2000ms
							;fr das gewhlte dt werden die Parameter aus der Tabelle im Bereich 0x1E01..0x1E3C geholt.
							;die erforderlichen Takte Ndt (Timer1, Prescaler 1:1) ergeben sich zu:   
							; Ndt = 65536 x (T1_ZEIT_H - 1) + 256 x (256 - T1_ZEIT_L + (256 - T1_ZEIT_L1) x (T1_ZEIT_LOOP - 1)) / T1_ZEIT_LOOP
	movlw 0x44
	addwf WOB_ZEIT,w 
	movwf TBLPTRL
	movlw 0x7E				;High Byte der Tabellenadresse 
	movwf TBLPTRH
	TBLRD*	 				;lese den Tabellenwert ins TABLAT Register
	movf TABLAT,W 			;Hole den Tabellenwert nach W
	addlw 0x44
	movwf TBLPTRL
	TBLRD*+	 				;lese den Tabellenwert ins TABLAT Register und erhhe den Wert
	movf TABLAT,W 			;Hole den Tabellenwert nach W
	movwf T1_ZEIT_H
	TBLRD*+	 				;lese den Tabellenwert ins TABLAT Register und erhhe den Wert
	movf TABLAT,W 			;Hole den Tabellenwert nach W
	movwf T1_ZEIT_L1
	movwf T1_ZEIT_L
	TBLRD*+	 				;lese den Tabellenwert ins TABLAT Register und erhhe den Wert
	movf TABLAT,W 			;Hole den Tabellenwert nach W
	movwf T1_ZEIT_LOOP
	addlw 0xFF				;wenn T1_ZEIT_LOOP >1<, muss T1_ZEIT_L1 um 1 erhht werden.
	btfss STATUS,Z
	incf T1_ZEIT_L1

	TBLRD*+	 				;lese den Tabellenwert ins TABLAT Register und erhhe den Wert
	movf TABLAT,W 			;Hole den Tabellenwert nach W
	movwf DELTAP_H
	movwf TEMP_2
	TBLRD*+	 				;lese den Tabellenwert ins TABLAT Register und erhhe den Wert
	movf TABLAT,W 			;Hole den Tabellenwert nach W
	movwf DELTAP_M
	movwf TEMP_1
	TBLRD*+	 				;lese den Tabellenwert ins TABLAT Register und erhhe den Wert
	movf TABLAT,W 			;Hole den Tabellenwert nach W
	movwf DELTAP_L
	movwf TEMP_0

	movf WOB_STEP,w
	movwf TEMP_3

para_dt_loop
	bcf STATUS,C
	rrcf TEMP_2
	rrcf TEMP_1
	rrcf TEMP_0
	decfsz TEMP_3
	goto para_dt_loop
							;prfen, ob TEMP_0-2 < Wert 782 (0x00030E) (entspricht ca.100ms fr einen Balken in graphischer Anzeige)
	movlw 0x00				;hole 0x00 ins W-Reg.								= DELTAP / 2 hoch WOB_STEP 
	subwf TEMP_2,w			;subtrahiere W-Reg. von TEMP_2
	btfss STATUS,C			;prfen, ob Ergebnis negativ, d.h. <0
	goto set_messend2		;Ergebnis negativ, setze Bit 2 in MESS_END Register
	btfss STATUS,Z			;prfen, ob Ergebnis = 0
	goto test_messend1		;Ergebnis positiv, weiter bei test_messend1
	movlw 0x03				;Ergebnis = 0, nchst kleineres Byte prfen, hole 0x03 ins W-Reg.
	subwf TEMP_1,w			;subtrahiere W-Reg. von TEMP_1
	btfss STATUS,C			;prfen, ob Ergebnis negativ, d.h. <0
	goto set_messend2		;Ergebnis negativ, setze Bit 2 in MESS_END Register
	btfss STATUS,Z			;prfen, ob Ergebnis = 0
	goto test_messend1		;Ergebnis positiv, weiter bei test_messend1
	movlw 0x0E				;Ergebnis = 0, nchst kleineres Byte prfen, hole 0x0E ins W-Reg.
	subwf TEMP_0,w			;subtrahiere W-Reg. von TEMP_0
	btfsc STATUS,C			;prfen, ob Ergebnis 0 oder positiv (STATUS,C=1)
	goto test_messend1		;TEMP >= Wert 782,  weiter bei test_messend1
set_messend2
	bsf MESS_END,1			;TEMP < Wert 782, setze Bit 1+2 in MESS_END Register
	bsf MESS_END,2
	goto set_delta_t

test_messend1				;wenn Wobbel Auto und Schrittweite <200ms gewhlt ist, wird andere Frequenzanzeige genutzt
							;   hierfr wird Bit 1 in Register MESS_END gesetzt
	btfsc WOB_M_A,0			;prfen ob Wobbel Manuell (1) oder Automatisch (0) gewhlt ist
	goto set_delta_t		;Manuell ist gewhlt, weiter im Programm
	movlw d'6'				;Auto ist gewhlt, prfen ob Zeitschrittweite < 6 (<200ms)
	subwf WOB_ZEIT,w
	btfsc STATUS,C
	goto set_delta_t		;Zeitschrittweite >= 6 (>=200ms), weiter im Programm
	bsf MESS_END,1			;setze Statusbit fr Zeitschrittweite <200ms und Wobbel Auto

set_delta_t
	clrf T1CON				;Timer 1 off
	bsf T1CON,T1CKPS0		;Prescaler 1:2
	movff T1_ZEIT_H,COUNT_H_S	;Zhler auf gewhltes delta t einstellen
	movff T1_ZEIT_LOOP,COUNT_ADJ
	clrf TMR1L
	movff T1_ZEIT_L1,TMR1H

	clrf STAT_INT
	bsf STAT_INT,3
	movlw b'00000001'		;Timer 1 Interrupt erlauben
	movwf PIE1 
	bsf INTCON,PEIE			;periphere Interrupts erlauben
	bsf INTCON,GIE			;globale Interrupts erlauben


	btfss WOB_M_A,1			;prfen, ob Burst gewhlt ist
	return					;Burst ist nicht gewhlt, zurck zur Hauptprogrammschleife

					;Programmteil zum Berechnen der Basisdaten fr Wobbel Burst
					;prfen, ob delta t mindestens 2x Periodenlnge von Fu
					;Takte Timer1 (f delta t) > 6.000.000 Takte Timer1 /s x 10 x 2 / Fu (x10)
	movlw b'00000000'		;16 Bit, intern, Prescaler 1:2
	movwf T0CON				;
	movlw 0x00				;lade Wert 6.000.000 x 10 x 2 = 120.000.000 (=> 0x07270E00) in ZAHL_C Register	
	movwf ZAHL_C0			;und teile durch Fu (x10)
	movlw 0x0E				;
	movwf ZAHL_C1
	movlw 0x27
	movwf ZAHL_C2
	movlw 0x07
	movwf ZAHL_C3
	clrf ZAHL_C4
	clrf ZAHL_C5
	clrf ZAHL_C6
	clrf ZAHL_C7

	movff WOB_FU_L,ZAHL_A0		;kopiere Fu (x10) in ZAHL_A Register
	movff WOB_FU_M,ZAHL_A1
	movff WOB_FU_H,ZAHL_A2
	clrf ZAHL_A3

	call div32					;Teile ZAHL_C durch ZAHL_A, das Ergebnis steht in ZAHL_C, der Rest in ZAHL_B

	call runden_div32			;Aufruf des Unterprogramms zum ganzzahligen Runden des Ergebnisses der Division

	bsf MESS_END,3				;setze vorsichtshalber Fehler Bits allgemein und fr zu kleine Fu bzw. zu kleines delta t
	bcf MESS_END,4

								;prfen ob Periode von Fu (Tfu) < delta t /2 ist: subtrahiere delta t /2 von Tfu

	movff DELTAP_L,TEMP_10
	movff DELTAP_M,TEMP_11
	movff DELTAP_H,TEMP_12
	movlw 0x00
	call test_kleiner			;Unterprogramm zum prfen, ob ZAHL_C < Wert in TEMP/W-Reg.
	btfss STATUS,C
	goto $+0x06					;Periode von Fu (Tfu) < delta t /2
	return						;Periode von Fu (Tfu) >= delta t /2, zurck zur Schleife mit Fehlermeldung

	bcf MESS_END,3				;delta t /2 >= Periode von Fu, lsche Fehler Bit allgemein und fr zu kleine Fu bzw. zu kleines delta t
	return

vco_wob_lin					;Berechnung: A/D-Wert * Fd (DDS-Format)/ 256 + Fu (DDS-Format)
	movff AD_DA_L,ZAHL_A0	;kopiere Low Byte des A/D-Wertes in Multiplikationsregister ZAHL_A0
	movff AD_DA_H,ZAHL_A1	;kopiere High Byte des A/D-Wertes in Multiplikationsregister ZAHL_A1
	clrf ZAHL_A2
	clrf ZAHL_A3
							

	movff FD2DDS_0,ZAHL_B0	;kopiere Fd (DDS-Format) von FD2DDS Registern nach Multiplikationsregister ZAHL_B
	movff FD2DDS_1,ZAHL_B1
	movff FD2DDS_2,ZAHL_B2
	clrf ZAHL_B3

	call mul32				;ZAHL_A * ZAHL_B = ZAHL_C, (A/D-Wert) * (Fd) = (A/D-Wert * Fd), Ergebnis steht in ZAHL_C
							;teilen durch 256, d.h. 1 Register nach rechts schieben
	btfss ZAHL_C0,7			;vorher Runden, prfen ob Bit7 =1, d.h. Rest >0,5
	goto vco_wob_lin1		;Rest <0,5, weiter
	incfsz ZAHL_C1			;Rest >0,5, erhhe Register um 1 und prfe auf berlauf
	goto vco_wob_lin1		;kein berlauf
	incfsz ZAHL_C2			;berlauf vorhanden, erhhe Register um 1 und prfe auf berlauf
	goto vco_wob_lin1		;kein berlauf
	incf ZAHL_C3			;berlauf vorhanden, erhhe Register um 1

vco_wob_lin1
	movff ZAHL_C1,ZAHL_C0	;teilen durch 256, d.h. 1 Register nach rechts schieben
	movff ZAHL_C2,ZAHL_C1
	movff ZAHL_C3,ZAHL_C2
	movff ZAHL_C4,ZAHL_C3
	clrf ZAHL_C4
	clrf ZAHL_C5
	clrf ZAHL_C6
	clrf ZAHL_C7

	movff FD1DDS_0,ZAHL_B0		;Berechnung: A/D-Wert * Fd / 256 + Fu
	movff FD1DDS_1,ZAHL_B1		;kopiere DDS Wert fr Fu in Additionsregister ZAHL_B
	movff FD1DDS_2,ZAHL_B2
	clrf ZAHL_B3

	call add32					;ZAHL_C + ZAHL_B = ZAHL_C, (A/D-Wert * Fd / 256) + (Fu)

	return

calc_frequenz					;Berechnung der Frequenz aus dem DDS Frequenzwert: F = DDS Frequenzwert x 10 x MCLK / 2^24
	movlw 0x80					;Lade 10 x MCLK = 120.000 x 10 = 1.200.000 in ZAHL_A Register (hex: 0x00124F80)
	movwf ZAHL_A0
	movlw 0x4F
	movwf ZAHL_A1
	movlw 0x12
	movwf ZAHL_A2
	clrf ZAHL_A3

	movff ZAHL_C0,ZAHL_B0		;Lade DDS Frequenzregisterwert in ZAHL_B Register
	movff ZAHL_C1,ZAHL_B1		;  Frequenzregisterwert steht noch in ZAHL_C
	movff ZAHL_C2,ZAHL_B2
	clrf ZAHL_B3

	call mul32					;Multipliziere ZAHL_A mit ZAHL_B, das Ergebnis steht in den Registern ZAHL_C0 bis ZAHL_C7
	call div_2h24				;aufruf des Unterprogramms zur Division durch 2^24 incl. ganzzahligem Runden
	return

calc_burst_length				;Unterprogramm zur Berechnung der Burst Lnge in Takten von TMR0 (prescaler 2:1)
	bcf T0CON,TMR0ON			;Timer 0 stoppen
	bcf INTCON,TMR0IF			;Timer 0 berlauf Flag lschen
	clrf ZAHL_C0				;kopiere 0x0640000000 in die Zhlerregister fr die Division (ZAHL_C)
	clrf ZAHL_C1				;=> 2^24 x Ftmr0 / MCLK x 32
	clrf ZAHL_C2				;=> 2^24 x 12.000.000Hz / 2 / 120.000Hz x 32 = 0x0640000000
	movlw 0x40
	movwf ZAHL_C3
	movlw 0x06
	movwf ZAHL_C4
	clrf ZAHL_C5
	clrf ZAHL_C6
	clrf ZAHL_C7

	movff DDS_FREQREG_0,ZAHL_A0	;kopiere den DDS Wert fr die Frequenz in das Nennerregister fr die Division (ZAHL_A)
	movff DDS_FREQREG_1,ZAHL_A1
	movff DDS_FREQREG_2,ZAHL_A2
	clrf ZAHL_A3

	call div32					;Teile ZAHL_C durch ZAHL_A, das Ergebnis steht in ZAHL_C, der Rest in ZAHL_B

	call runden_div32			;Aufruf des Unterprogramms zum ganzzahligen Runden des Ergebnisses der Division

	btfsc WOB_M_A,0				;prfen, ob Manuell gewhlt ist
	goto werte_timer0			;Manuell ist gewhlt, es wird der Wert fr 16 Perioden der Frequenz verwendet

t_burst
	movff DELTAP_L,TEMP_10		;wenn ZAHL_C >= delta t /2 ist, d.h. >=0,5 ist, teile ZAHL_C durch 2
	movff DELTAP_M,TEMP_11
	movff DELTAP_H,TEMP_12
	movlw 0x00
	call test_kleiner			;Unterprogramm zum prfen, ob ZAHL_C < Wert in TEMP/W-Reg.
	btfss STATUS,C
	goto werte_timer0			;ZAHL_C < delta t /2, weiter im Unterprogramm
	bcf STATUS,C				;ZAHL_C >= delta t /2, teile ZAHL_C durch 2 (nach rechts verschieben)
	rrcf ZAHL_C3
	rrcf ZAHL_C2
	rrcf ZAHL_C1
	rrcf ZAHL_C0
	goto t_burst

werte_timer0
	movf ZAHL_C0,w				;Berechnete Takte Timer0 um 7 reduzieren (Zeitdifferenz): 7 - Zahl C0
	sublw 0x07
	movwf ZAHL_C0				;kopiere Wert zurck nach Zahl C0
	btfsc STATUS,C				;prfe ob Ergebnis negativ
	goto wert_t0_sub1			;positiv, weiter
	incfsz ZAHL_C1,f			;negativ, erhhe Zahl C1 um 1 und prfe auf berlauf
	goto wert_t0_sub1			;kein berlauf, weiter
	incf ZAHL_C2,f				;Zahl C1 bergelaufen, erhhe Zahl C2 um 1
wert_t0_sub1
	movf ZAHL_C1,w				; 0 - Zahl C1
	sublw 0x00
	movwf TMR0H					;kopiere Wert in Timer 0 H-Byte Register
	btfsc STATUS,C				;prfe ob Ergebnis negativ
	goto wert_t0_sub2			;positiv, weiter			
	incf ZAHL_C2,f				;negativ, erhhe Zahl C2 um 1
wert_t0_sub2
	movff ZAHL_C0,TMR0L			;kopiere Zahl C0 in Timer0 L-Byte Register, mit diesem Write wird auch das 
	movff ZAHL_C2,TBURST_H		; H-Byte aktiviert. Kopiere Zahl C2 in Timer0 Erweiterungsregister
	bcf INTCON,TMR0IF
	return	



copy_fd1dds
	movff ZAHL_C0,FD1DDS_0
	movff ZAHL_C1,FD1DDS_1
	movff ZAHL_C2,FD1DDS_2
	return

copy_fd2dds
	movff ZAHL_C0,FD2DDS_0
	movff ZAHL_C1,FD2DDS_1
	movff ZAHL_C2,FD2DDS_2
	return

copy_fd3dds
	movff ZAHL_C0,FD3DDS_0
	movff ZAHL_C1,FD3DDS_1
	movff ZAHL_C2,FD3DDS_2
	return

log_mul_dek1				;Unterprogramm zum Kopieren des Multiplikators fr die aktuelle Dekade bei Logarithmischer Kennlinie 
	movlw d'1'					;prfen ob die aktuelle Dekade =1
	subwf DEK_AKTUELL,w
	btfss STATUS,Z
	goto log_mul_dek2			;aktuelle Dekade >1<, weiter bei nchster Prfung

	movff FD1DDS_0,ZAHL_A0
	movff FD1DDS_1,ZAHL_A1
	movff FD1DDS_2,ZAHL_A2
	clrf ZAHL_A3
	return

log_mul_dek2
	movlw d'2'					;prfen ob die aktuelle Dekade =2
	subwf DEK_AKTUELL,w
	btfss STATUS,Z
	goto log_mul_dek3			;aktuelle Dekade >2<, weiter bei nchster Prfung

	movff FD2DDS_0,ZAHL_A0
	movff FD2DDS_1,ZAHL_A1
	movff FD2DDS_2,ZAHL_A2
	clrf ZAHL_A3
	return

log_mul_dek3
	movff FD3DDS_0,ZAHL_A0
	movff FD3DDS_1,ZAHL_A1
	movff FD3DDS_2,ZAHL_A2
	clrf ZAHL_A3
	return


log_tabelle
	movlw 0x79				;vorbereiten fuer Log-Tabelle im Bereich 0x7900..0x7AFE
	movwf PCLATH			;
	bcf STATUS,C			;Lsche carry bit, damit es beim rotieren nicht strt
	rlcf AD_DA_L,w 			;x2 
	btfsc STATUS,C			;prfen auf berlauf
	incf PCLATH				;berlauf vorhanden, erhhe PCLATH
	call log_tab_L
	movwf ZAHL_B0			;L-byte n
	movlw 0x7C				;vorbereiten fuer Log-Tabelle im Bereich 0x7C00..0x7DFE
	movwf PCLATH			;
	bcf STATUS,C			;Lsche carry bit, damit es beim rotieren nicht strt
	rlcf AD_DA_L,w 			;x2  
	btfsc STATUS,C			;prfen auf berlauf
	incf PCLATH				;berlauf vorhanden, erhhe PCLATH
	call log_tab_H
	movwf ZAHL_B1			;H-byte n
	incfsz AD_DA_L
	goto log_tab_sub1
	movlw 0x00
	movwf TEMP_12			;L-byte n+1
	movlw 0x00
	movwf TEMP_13			;H-byte n+1
	return
log_tab_sub1
	movlw 0x79				;vorbereiten fuer Log-Tabelle im Bereich 0x7900..0x7AFE
	movwf PCLATH			;
	bcf STATUS,C			;Lsche carry bit, damit es beim rotieren nicht strt
	rlcf AD_DA_L,w 			;x2 
	btfsc STATUS,C			;prfen auf berlauf
	incf PCLATH				;berlauf vorhanden, erhhe PCLATH
	call log_tab_L
	movwf TEMP_12			;L-byte n+1
	movlw 0x7C				;vorbereiten fuer Log-Tabelle im Bereich 0x7C00..0x7DFE
	movwf PCLATH			;
	bcf STATUS,C			;Lsche carry bit, damit es beim rotieren nicht strt
	rlcf AD_DA_L,w 			;x2 
	btfsc STATUS,C			;prfen auf berlauf
	incf PCLATH				;berlauf vorhanden, erhhe PCLATH
	call log_tab_H
	movwf TEMP_13			;H-byte n+1
	return



;*******************************************************************************************
;
;	Unterprogramme zur Division, Multiplikation, Addition, Subtraktion, 
;		 Umwandlung (hex=>dig, dig=>hex)
;
;*******************************************************************************************

;*******************************************************************************************
; 32 bit Division: ZAHL_C / ZAHL_A = ZAHL_C, Rest steht in ZAHL_B
; Die Werte fr den Zhler mssen in den Registern ZAHL_C0 (L) bis ZAHL_C7 (H) stehen
; Die Werte fr den Nenner mssen in den Registern ZAHL_A0 (L) bis ZAHL_A3 (H) stehen
; Das ganzzahlige Ergebnis steht dann in den Registern ZAHL_C0 (L) bis ZAHL_C7 (H)
; Der Rest steht in den Registern ZAHL_B0 (L) bis ZAHL_B3 (H)
;*******************************************************************************************

div32
	clrf ZAHL_B3			;Rest von vorherigen Berechnungen lschen, ZAHL_B
	clrf ZAHL_B2
	clrf ZAHL_B1
	clrf ZAHL_B0					
	movlw d'64'				;LOOP_1 mit 64 laden, da 64 bit im Zhler
	movwf LOOP_1
div32_loop
	rlcf ZAHL_C0,F       	;den gesamtem Zhler um 1 nach links schieben, ZAHL_C
	rlcf ZAHL_C1,F
	rlcf ZAHL_C2,F
	rlcf ZAHL_C3,F
	rlcf ZAHL_C4,F
	rlcf ZAHL_C5,F
	rlcf ZAHL_C6,F
	rlcf ZAHL_C7,F

	rlcf ZAHL_B0,F			;das Register fr den Rest um 1 nach links schieben
	rlcf ZAHL_B1,F
	rlcf ZAHL_B2,F
	rlcf ZAHL_B3,F

	rlcf LOOP_1,F			;das Carrybit wird vorbergehend im LOOP_1 gespeichert
									;dadurch kann man sich ein Register sparen	
	bsf LOOP_1,0
	movf ZAHL_A0,W			;Subtrahiere Nenner _0 von Rest _0 (REST_0-NENNER_0=REST_0)
	subwf ZAHL_B0,F			;
	btfsc STATUS, C			;prft den bertrag
	goto div32_sub2			;kein bertrag
	movlw 0x01
	subwf ZAHL_B1,F			;reduziere REST_1 um 1
	btfsc STATUS, C			;prft den bertrag
	goto div32_sub2			;kein bertrag
	movlw 0x01
	subwf ZAHL_B2,F			;reduziere REST_2 um 1
	btfsc STATUS, C			;prft den bertrag
	goto div32_sub2			;kein bertrag
	movlw 0x01
	subwf ZAHL_B3,F			;reduziere REST_3 um 1
	btfss STATUS, C			;prft den bertrag
	bcf LOOP_1,0			;bertrag vorhanden, lsche Bit 0 in LOOP_1

div32_sub2
	movf ZAHL_A1,W			;Subtrahiere Nenner _1 von Rest _1 (REST_1-NENNER_1=REST_1)
	subwf ZAHL_B1,F			;
	btfsc STATUS, C			;prft den bertrag
	goto div32_sub3			;kein bertrag
	movlw 0x01
	subwf ZAHL_B2,F			;reduziere REST_2 um 1
	btfsc STATUS, C			;prft den bertrag
	goto div32_sub3			;kein bertrag
	movlw 0x01
	subwf ZAHL_B3,F			;reduziere REST_3 um 1
	btfss STATUS, C			;prft den bertrag
	bcf LOOP_1,0			;bertrag vorhanden, lsche Bit 0 in LOOP_1

div32_sub3
	movf ZAHL_A2,W			;Subtrahiere Nenner _2 von Rest _2 (REST_2-NENNER_2=REST_2)
	subwf ZAHL_B2,F			;
	btfsc STATUS, C			;prft den bertrag
	goto div32_sub4			;kein bertrag
	movlw 0x01
	subwf ZAHL_B3,F			;reduziere REST_3 um 1
	btfss STATUS, C			;prft den bertrag
	bcf LOOP_1,0			;bertrag vorhanden, lsche Bit 0 in LOOP_1

div32_sub4
	movf ZAHL_A3,W			;Subtrahiere Nenner _3 von Rest _3 (REST_3-NENNER_3=REST_3)
	subwf ZAHL_B3,F			;

	btfss STATUS, C			;prfe, ob das Ergebnis positiv ist
	bcf LOOP_1,0			;wenn nein, lsche gespeichertes Carrybit in LOOP_1
	btfsc LOOP_1,0			;prfe das gespeicherte Carrybit in LOOP_1
	goto div32_jump			;wenn Carry=1: springe nach div16_jump

	movf ZAHL_A0,W			;wenn Carry=0:	Subtraktion wieder rckgngig machen 
	addwf ZAHL_B0,F			; 
	btfsc STATUS,C
	incfsz ZAHL_B1,F
	goto div32_sub5
	incfsz ZAHL_B2,F
	goto div32_sub5
	incf ZAHL_B3,F
div32_sub5
	movf ZAHL_A1,W			;
	addwf ZAHL_B1,F			;
	btfsc STATUS,C
	incfsz ZAHL_B2,F
	goto div32_sub6
	incf ZAHL_B3,F
div32_sub6
	movf ZAHL_A2,W			; 
	addwf ZAHL_B2,F			; 
	btfsc STATUS,C
	incf ZAHL_B3,F

	movf ZAHL_A3,W			;
	addwf ZAHL_B3,F			; 

div32_jump
	bcf STATUS,C			;Carrybit lschen und das gespeicherte Carrybit in Register
	rrcf LOOP_1,F			;LOOP_1 wieder nach rechts rausschieben.
	decfsz LOOP_1,F 		;LOOP_1 um 1 verringern und prfen ob 0
	goto div32_loop         ;Counter ist noch nicht 0, zurck zur Schleife
	 
	rlcf ZAHL_C0,F			;ZAEHLER_L		gesamter Zhler noch einmal nach links schieben, damit
	rlcf ZAHL_C1,F			;ZAEHLER_M		das richtige Ergebnis im Zhler steht
	rlcf ZAHL_C2,F			;ZAEHLER_H
	rlcf ZAHL_C3,F			;ZAEHLER_H
	rlcf ZAHL_C4,F			;ZAEHLER_H
	rlcf ZAHL_C5,F			;ZAEHLER_H
	rlcf ZAHL_C6,F			;ZAEHLER_H
	rlcf ZAHL_C7,F			;ZAEHLER_H

	return



;*******************************************************************************************
; 32 bit Multiplikation: ZAHL_A x ZAHL_B = ZAHL_C
; Die Werte fr die Multiplikation mssen in den Registern ZAHL_A0 (L) bis ZAHL_A3 (H) 
; und ZAHL_B0 (L) bis ZAHL_B3 (H) stehen
; Das Ergebnis steht dann in den Registern ZAHL_C0 (L) bis ZAHL_C7 (H)
;*******************************************************************************************
mul32

	call clear_ZAHL_C		;Lschen des vorherigen Ergebnisses (ZAHL_C0 - ZAHL_C7)
	bsf ZAHL_C3,7			;um zu erkennen, wann 32 mal geschoben wurde
mul32_loop
	rrcf ZAHL_A3,F			;verschiebe gesammtes Register Multiplikant1 um 1 nach rechts
	rrcf ZAHL_A2,F			;
	rrcf ZAHL_A1,F			;
	rrcf ZAHL_A0,F			;wenn das LSB eine 1 ist wird das Carry gesetzt und es muss
	btfss STATUS,C			;eine Addition durchgefhrt werden ist das Carry = 0 entspricht
	goto mul32_rotate		;dies einer Multiplikation mit 0 und es wird nichts addiert

	movf ZAHL_B0,W			;Beginn der 32 bit Addition: Addiere Multiplikant2 _0 Register zu Ergebnis _4 Register
	addwf ZAHL_C4,F			;
	btfsc STATUS,C			;prft, ob ein bertrag vorhanden ist
	incfsz ZAHL_C5,F		;es ist ein bertrag vorhanden und das Ergebnis _5 Register wird um 1 erhht
	goto mul32_sub1
	incfsz ZAHL_C6,F		;wenn Ergebnis _5 Register berluft, erhhe Ergebnis _6 Register um 1
	goto mul32_sub1
	incf ZAHL_C7,F			;wenn Ergebnis _6 Register berluft, erhhe Ergebnis _7 Register um 1
mul32_sub1
	movf ZAHL_B1,W			;Addiere Multiplikant2 _1 Register zu Ergebnis _5 Register
	addwf ZAHL_C5,F			;
	btfsc STATUS,C			;prft, ob ein bertrag vorhanden ist
	incfsz ZAHL_C6,F		;es ist ein bertrag vorhanden und das Ergebnis _6 Register wird um 1 erhht
	goto mul32_sub2
	incf ZAHL_C7,F			;wenn Ergebnis _6 Register berluft, erhhe Ergebnis _7 Register um 1
mul32_sub2
	movf ZAHL_B2,W			;Addiere Multiplikant2 _2 Register zu Ergebnis _6 Register
	addwf ZAHL_C6,F			;
	btfsc STATUS,C			;prft, ob ein bertrag vorhanden ist
	incf ZAHL_C7,F			;wenn Ergebnis _6 Register berluft, erhhe Ergebnis _7 Register um 1

	movf ZAHL_B3,W			;Addiere Multiplikant2 _3 Register zu Ergebnis _7 Register
	addwf ZAHL_C7,F			;

mul32_rotate
	rrcf ZAHL_C7,F			;Das 64 Bit Ergebnisregister wird um 1 nach rechts geschoben
	rrcf ZAHL_C6,F
	rrcf ZAHL_C5,F
	rrcf ZAHL_C4,F
	rrcf ZAHL_C3,F
	rrcf ZAHL_C2,F
	rrcf ZAHL_C1,F
	rrcf ZAHL_C0,F
	btfss STATUS,C			;ist das Bit 7 von ZAHL_C3 im Carry angekommen wurde das Ergebnis
	goto mul32_loop			;insgesamt 32 mal rotiert und die Berechnung ist beendet
	return                  ;

;*******************************************************************************************
; 32 bit Addition: ZAHL_C + ZAHL_B = ZAHL_C
; Die Werte fr die Addition mssen in den Registern ZAHL_C0 (L) bis ZAHL_C3 (H)
; und ZAHL_B0 (L) bis ZAHL_B3 (H) stehen
; Das Ergebnis steht in den Registern ZAHL_C0 (L) bis ZAHL_C4 (H)
;*******************************************************************************************

add32
	movf ZAHL_B0,W			;Hole Low Byte Zahl2 ins W-Register
	addwf ZAHL_C0,F			;addiere Low Byte Zahl1
	btfsc STATUS,C			;prft, ob ein bertrag vorhanden ist
	incfsz ZAHL_C1,F		;es ist ein bertrag vorhanden und das L+1 Byte von Zahl2 wird um 1 erhht
	goto add32_sub1			;kein bertrag beim L+1 Byte von Zahl2, 
	incfsz ZAHL_C2,F		;es ist ein bertrag vorhanden und das L+2 Byte von Zahl2 wird um 1 erhht
	goto add32_sub1			;kein bertrag beim L+2 Byte von Zahl2, 
	incfsz ZAHL_C3,F		;es ist ein bertrag vorhanden und das L+3 Byte von Zahl2 wird um 1 erhht
	goto add32_sub1			;kein bertrag beim L+3 Byte von Zahl2, 
	incf ZAHL_C4,F			;es ist ein bertrag vorhanden und das L+4 Byte von Zahl2 wird um 1 erhht
add32_sub1
	movf ZAHL_B1,W			;Hole L+1 Byte Zahl2 ins W-Register
	addwf ZAHL_C1,F			;addiere L+1 Byte Zahl1
	btfsc STATUS,C			;prft, ob ein bertrag vorhanden ist
	incfsz ZAHL_C2,F		;es ist ein bertrag vorhanden und das L+2 Byte von Zahl2 wird um 1 erhht
	goto add32_sub2			;kein bertrag beim L+2 Byte von Zahl2, 
	incfsz ZAHL_C3,F		;es ist ein bertrag vorhanden und das L+3 Byte von Zahl2 wird um 1 erhht
	goto add32_sub2			;kein bertrag beim L+3 Byte von Zahl2, 
	incf ZAHL_C4,F			;es ist ein bertrag vorhanden und das L+4 Byte von Zahl2 wird um 1 erhht
add32_sub2
	movf ZAHL_B2,W			;Hole L+2 Byte Zahl2 ins W-Register
	addwf ZAHL_C2,F			;addiere L+2 Byte Zahl1
	btfsc STATUS,C			;prft, ob ein bertrag vorhanden ist
	incfsz ZAHL_C3,F		;es ist ein bertrag vorhanden und das L+3 Byte von Zahl2 wird um 1 erhht
	goto add32_sub3			;kein bertrag beim L+3 Byte von Zahl2, 
	incf ZAHL_C4,F			;es ist ein bertrag vorhanden und das L+4 Byte von Zahl2 wird um 1 erhht
add32_sub3
	movf ZAHL_B3,W			;Hole High Byte Zahl2 ins W-Register
	addwf ZAHL_C3,F			;addiere High Byte Zahl1
	btfsc STATUS,C			;prft, ob ein bertrag vorhanden ist
	incf ZAHL_C4,F			;es ist ein bertrag vorhanden und das L+4 Byte von Zahl2 wird um 1 erhht
	return

;*******************************************************************************************
; 32 bit Subtraktion: ZAHL_C - ZAHL_B = ZAHL_C
; Die Werte fr die Subtraktion mssen in den Registern ZAHL_B0 (L) bis ZAHL_B3 (H)
; und ZAHL_C0 (L) bis ZAHL_C4 (H) stehen, wobei ZAHL_B von ZAHL_C subtrahiert wird 
; Das Ergebnis steht in den Registern ZAHL_C0 (L) bis ZAHL_C4 (H)
;*******************************************************************************************

sub32
	movf ZAHL_B0,w			;Hole 0/7-Byte von B nach W
	subwf ZAHL_C0,F			;Subtrahiere B von C, 0/7-Byte
	btfsc STATUS,C			;prfe ob Ergebnis negativ
	goto sub32_2			;Ergebnis ist positiv (Carry Bit = H) weiter bei Subtraktion der 8/15-Bytes
	movlw 0x01				;Ergebnis ist negativ (Carry Bit = L), subtrahiere 1 von C, 8/15-Byte
	subwf ZAHL_C1,F
	btfsc STATUS,C			;prfe ob Ergebnis negativ
	goto sub32_2			;Ergebnis ist positiv (Carry Bit = H) weiter bei Subtraktion der 8/15-Bytes
	movlw 0x01				;Ergebnis ist negativ (Carry Bit = L), subtrahiere 1 von C, 16/23-Byte
	subwf ZAHL_C2,F
	btfsc STATUS,C			;prfe ob Ergebnis negativ
	goto sub32_2			;Ergebnis ist positiv (Carry Bit = H) weiter bei Subtraktion der 8/15-Bytes
	movlw 0x01				;Ergebnis ist negativ (Carry Bit = L), subtrahiere 1 von C, 24/31-Byte
	subwf ZAHL_C3,F
	btfsc STATUS,C			;prfe ob Ergebnis negativ
	goto sub32_2			;Ergebnis ist positiv (Carry Bit = H) weiter bei Subtraktion der 8/15-Bytes
	movlw 0x01				;Ergebnis ist negativ (Carry Bit = L), subtrahiere 1 von C, 32/39-Byte
	subwf ZAHL_C4,F
sub32_2
	movf ZAHL_B1,w			;Hole 8/15-Byte von B nach W
	subwf ZAHL_C1,F			;Subtrahiere B von C, 8/15-Byte
	btfsc STATUS,C			;prfe ob Ergebnis negativ
	goto sub32_3			;Ergebnis ist positiv (Carry Bit = H) weiter bei Subtraktion der 16/23-Bytes
	movlw 0x01				;Ergebnis ist negativ (Carry Bit = L), subtrahiere 1 von C, 16/23-Byte
	subwf ZAHL_C2,F
	btfsc STATUS,C			;prfe ob Ergebnis negativ
	goto sub32_3			;Ergebnis ist positiv (Carry Bit = H) weiter bei Subtraktion der 16/23-Bytes
	movlw 0x01				;Ergebnis ist negativ (Carry Bit = L), subtrahiere 1 von C, 24/31-Byte
	subwf ZAHL_C3,F
	btfsc STATUS,C			;prfe ob Ergebnis negativ
	goto sub32_3			;Ergebnis ist positiv (Carry Bit = H) weiter bei Subtraktion der 16/23-Bytes
	movlw 0x01				;Ergebnis ist negativ (Carry Bit = L), subtrahiere 1 von C, 32/39-Byte
	subwf ZAHL_C4,F
sub32_3
	movf ZAHL_B2,w			;Hole 16/23-Byte von B nach W
	subwf ZAHL_C2,F			;Subtrahiere B von C, 16/23-Byte
	btfsc STATUS,C			;prfe ob Ergebnis negativ
	goto sub32_4			;Ergebnis ist positiv (Carry Bit = H) weiter bei Subtraktion der 24/31-Bytes
	movlw 0x01				;Ergebnis ist negativ (Carry Bit = L), subtrahiere 1 von C, 24/31-Byte
	subwf ZAHL_C3,F
	btfsc STATUS,C			;prfe ob Ergebnis negativ
	goto sub32_4			;Ergebnis ist positiv (Carry Bit = H) weiter bei Subtraktion der 24/31-Bytes
	movlw 0x01				;Ergebnis ist negativ (Carry Bit = L), subtrahiere 1 von C, 32/39-Byte
	subwf ZAHL_C4,F
sub32_4
	movf ZAHL_B3,w			;Hole 24/31-Byte von B nach W
	subwf ZAHL_C3,F			;Subtrahiere B von C, 24/31-Byte
	btfsc STATUS,C			;prfe ob Ergebnis negativ
	goto sub32_end			;Ergebnis ist positiv (Carry Bit = H) weiter zum Ende des Unterprogramms
	movlw 0x01				;Ergebnis ist negativ (Carry Bit = L), subtrahiere 1 von C, 32/39-Byte
	subwf ZAHL_C4,F
sub32_end
	return


runden_div32					;Unterprogramm zum ganzzahligen Runden des Ergebnisses der Division aus div32
	bcf STATUS,C				;lsche Carry Bit, damit es beim rotieren nicht strt
	rrcf ZAHL_A3					;ZAHL_A 1x nach rechts schieben, d.h. durch 2 teilen 
	rrcf ZAHL_A2
	rrcf ZAHL_A1
	rrcf ZAHL_A0

	movf ZAHL_A3,w				;wenn Rest >= ZAHL_A/2 ist, d.h. >=0,5 ist, erhhe ZAHL_C um 1
	subwf ZAHL_B3,w				;subtrahiere ZAHL_A/2 von Rest
	btfss STATUS,C				;prfen, ob Ergebnis 0 oder positiv (STATUS,C=1)
	return						;Ergebnis negativ (STATUS,C=0), Rcksprung
	btfss STATUS,Z				;prfen, ob Ergebnis = 0  (STATUS,Z=1)
	goto runden_div32_inc		;Ergebnis positiv, erhhe ZAHL_C um 1
	movf ZAHL_A2,w				;Ergebnis = 0, nchst kleineres Byte prfen, hole ZAHL_A2 ins W-Reg.
	subwf ZAHL_B2,w				;subtrahiere W-Reg. von ZAHL_B2
	btfss STATUS,C				;prfen, ob Ergebnis negativ, d.h. <0
	return						;Ergebnis negativ, Rcksprung
	btfss STATUS,Z				;prfen, ob Ergebnis = 0
	goto runden_div32_inc		;Ergebnis positiv, erhhe ZAHL_C um 1
	movf ZAHL_A1,w				;Ergebnis = 0, nchst kleineres Byte prfen, hole ZAHL_A1 ins W-Reg.
	subwf ZAHL_B1,w				;subtrahiere W-Reg. von ZAHL_B1
	btfss STATUS,C				;prfen, ob Ergebnis negativ, d.h. <0
	return						;Ergebnis negativ, Rcksprung
	btfss STATUS,Z				;prfen, ob Ergebnis = 0
	goto runden_div32_inc		;Ergebnis positiv, erhhe ZAHL_C um 1
	movf ZAHL_A0,w				;Ergebnis = 0, nchst kleineres Byte prfen, hole ZAHL_A0 ins W-Reg.
	subwf ZAHL_B0,w				;subtrahiere W-Reg. von ZAHL_B0
	btfss STATUS,C				;prfen, ob Ergebnis 0 oder positiv (STATUS,C=1)
	return						;Rest (ZAHL_B) < ZAHL_A/2, Rcksprung aus dem Unterprogramm
runden_div32_inc	
	incfsz ZAHL_C0				;Rest (ZAHL_B) >= ZAHL_A/2, erhhe ZAHL_C0 um 1 und prfe den berlauf
	return						;kein berlauf, Rcksprung aus dem Unterprogramm
	incfsz ZAHL_C1				;berlauf vorhanden,  erhhe ZAHL_C1 um 1 und prfe den berlauf
	return						;kein berlauf, Rcksprung aus dem Unterprogramm
	incfsz ZAHL_C2				;berlauf vorhanden,  erhhe ZAHL_C2 um 1 und prfe den berlauf
	return						;kein berlauf, Rcksprung aus dem Unterprogramm
	incf ZAHL_C3				;berlauf vorhanden,  erhhe ZAHL_C3 um 1
	return						;Rcksprung aus dem Unterprogramm


test_kleiner					;Unterprogramm zum prfen, ob ZAHL_C < W-Reg., Start mit hchstem Byte
	subwf ZAHL_C3,w				;subtrahiere W-Reg. von ZAHL_C3
	btfss STATUS,C				;prfen, ob Ergebnis 0 oder positiv (STATUS,C=1)
	return						;Ergebnis negativ (STATUS,C=0), Rcksprung
	btfss STATUS,Z				;prfen, ob Ergebnis = 0  (STATUS,Z=1)
	return						;Ergebnis positiv, Rcksprung
	movf TEMP_12,w				;Ergebnis = 0, nchst kleineres Byte prfen, hole TEMP_12 ins W-Reg.
	subwf ZAHL_C2,w				;subtrahiere W-Reg. von ZAHL_C2
	btfss STATUS,C				;prfen, ob Ergebnis negativ, d.h. <0
	return						;Ergebnis negativ, Rcksprung
	btfss STATUS,Z				;prfen, ob Ergebnis = 0
	return						;Ergebnis positiv, Rcksprung
	movf TEMP_11,w				;Ergebnis = 0, nchst kleineres Byte prfen, hole TEMP_11 ins W-Reg.
	subwf ZAHL_C1,w				;subtrahiere W-Reg. von ZAHL_C1
	btfss STATUS,C				;prfen, ob Ergebnis negativ, d.h. <0
	return						;Ergebnis negativ, Rcksprung
	btfss STATUS,Z				;prfen, ob Ergebnis = 0
	return						;Ergebnis positiv, Rcksprung
	movf TEMP_10,w				;Ergebnis = 0, nchst kleineres Byte prfen, hole TEMP_10 ins W-Reg.
	subwf ZAHL_C0,w				;subtrahiere W-Reg. von ZAHL_C0
	return


div_2h12
	movlw d'4'					;Teilen von ZAHL_C durch 2^12 (=4096) in 2 Schritten: 1.: teilen durch 16, 
	movwf LOOP_1				;  2.: teilen durch 256
div_2h12_loop
	bcf STATUS,C				;Lsche Carry Bit, damit es beim Rotieren nicht strt
	rrcf ZAHL_C4					;teilen durch 16: 4 x nach rechts verschieben
	rrcf ZAHL_C3
	rrcf ZAHL_C2
	rrcf ZAHL_C1
	rrcf ZAHL_C0
	decfsz LOOP_1				;reduziere Zhler um 1 und prfe ob 0
	goto div_2h12_loop			;noch nicht 4x geschoben, zurck zur Schleife
								;das durch 2^12 geteilte Ergebnis steht in ZAHL_C1 (L) bis ZAHL_C4 (H), Rest in ZAHL_C0

	btfss ZAHL_C0,7				;wenn Rest >=128 ist, d.h. >=0,5 ist, erhhe ZAHL_C1 um 1
	goto $+0x10					;Rest (ZAHL_C0) < 128, weiter bei kopieren des Ergebnis
	incfsz ZAHL_C1				;Rest (ZAHL_C0) >=128 erhhe Ergebnis (ZAHL_C1) um 1
	goto $+0x0C
	incfsz ZAHL_C2				;es ist ein berlauf aufgetreten, erhhe um 1 und prfe ob 0
	goto $+0x08
	incfsz ZAHL_C3				;es ist ein berlauf aufgetreten, erhhe um 1 und prfe ob 0
	goto $+0x04
	incf ZAHL_C4				;es ist ein berlauf aufgetreten, erhhe um 1

	movff ZAHL_C1,ZAHL_C0		;kopieren der Register
	movff ZAHL_C2,ZAHL_C1
	movff ZAHL_C3,ZAHL_C2
	movff ZAHL_C4,ZAHL_C3

	return


div_2h16						;Teilen von ZAHL_C durch 2^16 (dez: 65536, hex: 0x010000)
								;das durch 2^16 geteilte Ergebnis steht in ZAHL_C0 (L) bis ZAHL_C3 (H)
	btfss ZAHL_C1,7				;  wenn Rest >=128 ist, d.h. >=0,5 ist, erhhe ZAHL_C0 um 1
	goto div_2h16_sub					;Rest (ZAHL_C1) < 128, weiter bei kopieren des Ergebnis
	incfsz ZAHL_C2				;Rest (ZAHL_C1) >=128 erhhe Ergebnis (ZAHL_C2) um 1
	goto div_2h16_sub
	incfsz ZAHL_C3				;es ist ein berlauf aufgetreten, erhhe um 1
	goto div_2h16_sub
	incfsz ZAHL_C4				;es ist ein berlauf aufgetreten, erhhe um 1
	goto div_2h16_sub
	incf ZAHL_C5				;es ist ein berlauf aufgetreten, erhhe um 1
div_2h16_sub
	movff ZAHL_C2,ZAHL_C0		;kopieren der Register
	movff ZAHL_C3,ZAHL_C1
	movff ZAHL_C4,ZAHL_C2
	movff ZAHL_C5,ZAHL_C3
	return

div_2h24						;Teilen von ZAHL_C durch 2^24 (hex: 0x01000000)
								;das durch 2^24 geteilte Ergebnis steht in ZAHL_C0 (L) bis ZAHL_C3 (H)
	btfss ZAHL_C2,7				;  wenn Rest >=128 ist, d.h. >=0,5 ist, erhhe ZAHL_C0 um 1
	goto div_2h24_sub					;Rest (ZAHL_C1) < 128, weiter bei kopieren des Ergebnis
	incfsz ZAHL_C3				;Rest (ZAHL_C1) >=128 erhhe Ergebnis (ZAHL_C2) um 1
	goto div_2h24_sub
	incfsz ZAHL_C4				;es ist ein berlauf aufgetreten, erhhe um 1
	goto div_2h24_sub
	incfsz ZAHL_C5				;es ist ein berlauf aufgetreten, erhhe um 1
	goto div_2h24_sub
	incf ZAHL_C6				;es ist ein berlauf aufgetreten, erhhe um 1
div_2h24_sub
	movff ZAHL_C3,ZAHL_C0		;kopieren der Register
	movff ZAHL_C4,ZAHL_C1
	movff ZAHL_C5,ZAHL_C2
	movff ZAHL_C6,ZAHL_C3
	return


convert_hex_dig				;Unterprogramm zum Zerlegen einer 32-Bit Zahl in die einzelnen Digits (max.999.999.999)zur Anzeige auf dem Display
	clrf DISP_8				;Lsche Register, von der 32-Bit Zahl in Register ZAHL_C wird ZAHL_B abgezogen bis Ergebnis negativ
	clrf ZAHL_C4			;
	movlw 0x00				;Lade ZAHL_B mit 100.000.000 (0x05F5E100)
	movwf ZAHL_B0
	movlw 0xE1				;
	movwf ZAHL_B1
	movlw 0xF5				;
	movwf ZAHL_B2
	movlw 0x05				;
	movwf ZAHL_B3
conv_hex_dig8
	incf DISP_8				;erhhe Digit um 1
	call sub32				;ZAHL_C minus ZAHL_B, das Ergebnis steht in ZAHL_C
	movf ZAHL_C4,w			;Wenn Ergebnis der Subtraktion noch positiv ist, ist das berlaufregister ZAHL_C4 = 0
	btfsc STATUS,Z			;Prfen ob ZAHL_C4 >0<
	goto conv_hex_dig8		;berlaufregister ZAHL_C4 ist noch 0, zurck zur Schleife
	decf DISP_8				;berlaufregister ZAHL_C4 ist >0<, letzte Erhhung des DISP_ Digit 
	call add32				;   und letzte Subtraktion rckgngig machen

	clrf DISP_7				;Lsche Register, von der 32-Bit Zahl in Register ZAHL_C wird ZAHL_B abgezogen bis Ergebnis negativ
	movlw 0x80				;Lade ZAHL_B mit 10.000.000 (0x00989680)
	movwf ZAHL_B0
	movlw 0x96				;
	movwf ZAHL_B1
	movlw 0x98				;
	movwf ZAHL_B2
	clrf ZAHL_B3
conv_hex_dig7
	incf DISP_7				;erhhe Digit um 1
	call sub32				;ZAHL_C minus ZAHL_B, das Ergebnis steht in ZAHL_C
	movf ZAHL_C4,w			;Wenn Ergebnis der Subtraktion noch positiv ist, ist das berlaufregister ZAHL_C4 = 0
	btfsc STATUS,Z			;Prfen ob ZAHL_C4 >0<
	goto conv_hex_dig7		;berlaufregister ZAHL_C4 ist noch 0, zurck zur Schleife
	decf DISP_7				;berlaufregister ZAHL_C4 ist >0<, letzte Erhhung des DISP_ Digit 
	call add32				;   und letzte Subtraktion rckgngig machen

	clrf DISP_6				;Lsche Register, von der 32-Bit Zahl in Register ZAHL_C wird ZAHL_B abgezogen bis Ergebnis negativ
	movlw 0x40				;Lade ZAHL_B mit 1.000.000 (0x000F4240)
	movwf ZAHL_B0
	movlw 0x42				;
	movwf ZAHL_B1
	movlw 0x0F				;
	movwf ZAHL_B2
	clrf ZAHL_B3
conv_hex_dig6
	incf DISP_6				;erhhe Digit um 1
	call sub32				;ZAHL_C minus ZAHL_B, das Ergebnis steht in ZAHL_C
	movf ZAHL_C4,w			;Wenn Ergebnis der Subtraktion noch positiv ist, ist das berlaufregister ZAHL_C4 = 0
	btfsc STATUS,Z			;Prfen ob ZAHL_C4 >0<
	goto conv_hex_dig6		;berlaufregister ZAHL_C4 ist noch 0, zurck zur Schleife
	decf DISP_6				;berlaufregister ZAHL_C4 ist >0<, letzte Erhhung des DISP_ Digit 
	call add32				;   und letzte Subtraktion rckgngig machen

	clrf DISP_5				;Lsche Register, von der 32-Bit Zahl in Register ZAHL_C wird ZAHL_B abgezogen bis Ergebnis negativ
	movlw 0xA0				;Lade ZAHL_B mit 100.000 (0x000186A0)
	movwf ZAHL_B0
	movlw 0x86				;
	movwf ZAHL_B1
	movlw 0x01				;
	movwf ZAHL_B2
	clrf ZAHL_B3
conv_hex_dig5
	incf DISP_5				;erhhe Digit um 1
	call sub32				;ZAHL_C minus ZAHL_B, das Ergebnis steht in ZAHL_C
	movf ZAHL_C4,w			;Wenn Ergebnis der Subtraktion noch positiv ist, ist das berlaufregister ZAHL_C4 = 0
	btfsc STATUS,Z			;Prfen ob ZAHL_C4 >0<
	goto conv_hex_dig5		;berlaufregister ZAHL_C4 ist noch 0, zurck zur Schleife
	decf DISP_5				;berlaufregister ZAHL_C4 ist >0<, letzte Erhhung des DISP_ Digit 
	call add32				;   und letzte Subtraktion rckgngig machen

	clrf DISP_4				;Lsche Register, von der 32-Bit Zahl in Register ZAHL_C wird ZAHL_B abgezogen bis Ergebnis negativ
	movlw 0x10				;Lade ZAHL_B mit 10.000 (0x00002710)
	movwf ZAHL_B0
	movlw 0x27				;
	movwf ZAHL_B1
	clrf ZAHL_B2
	clrf ZAHL_B3
conv_hex_dig4
	incf DISP_4				;erhhe Digit um 1
	call sub32				;ZAHL_C minus ZAHL_B, das Ergebnis steht in ZAHL_C
	movf ZAHL_C4,w			;Wenn Ergebnis der Subtraktion noch positiv ist, ist das berlaufregister ZAHL_C4 = 0
	btfsc STATUS,Z			;Prfen ob ZAHL_C4 >0<
	goto conv_hex_dig4		;berlaufregister ZAHL_C4 ist noch 0, zurck zur Schleife
	decf DISP_4				;berlaufregister ZAHL_C4 ist >0<, letzte Erhhung des DISP_ Digit 
	call add32				;   und letzte Subtraktion rckgngig machen

	clrf DISP_3				;Lsche Register, von der 32-Bit Zahl in Register ZAHL_C wird ZAHL_B abgezogen bis Ergebnis negativ
	movlw 0xE8				;Lade ZAHL_B mit 1.000 (0x000003E8)
	movwf ZAHL_B0
	movlw 0x03				;
	movwf ZAHL_B1
	clrf ZAHL_B2
	clrf ZAHL_B3
conv_hex_dig3
	incf DISP_3				;erhhe Digit um 1
	call sub32				;ZAHL_C minus ZAHL_B, das Ergebnis steht in ZAHL_C
	movf ZAHL_C4,w			;Wenn Ergebnis der Subtraktion noch positiv ist, ist das berlaufregister ZAHL_C4 = 0
	btfsc STATUS,Z			;Prfen ob ZAHL_C4 >0<
	goto conv_hex_dig3		;berlaufregister ZAHL_C4 ist noch 0, zurck zur Schleife
	decf DISP_3				;berlaufregister ZAHL_C4 ist >0<, letzte Erhhung des DISP_ Digit 
	call add32				;   und letzte Subtraktion rckgngig machen

	clrf DISP_2				;Lsche Register, von der 32-Bit Zahl in Register ZAHL_C wird ZAHL_B abgezogen bis Ergebnis negativ
	movlw 0x64				;Lade ZAHL_B mit 100 (0x00000064)
	movwf ZAHL_B0
	clrf ZAHL_B1
	clrf ZAHL_B2
	clrf ZAHL_B3
conv_hex_dig2
	incf DISP_2				;erhhe Digit um 1
	call sub32				;ZAHL_C minus ZAHL_B, das Ergebnis steht in ZAHL_C
	movf ZAHL_C4,w			;Wenn Ergebnis der Subtraktion noch positiv ist, ist das berlaufregister ZAHL_C4 = 0
	btfsc STATUS,Z			;Prfen ob ZAHL_C4 >0<
	goto conv_hex_dig2		;berlaufregister ZAHL_C4 ist noch 0, zurck zur Schleife
	decf DISP_2				;berlaufregister ZAHL_C4 ist >0<, letzte Erhhung des DISP_ Digit 
	call add32				;   und letzte Subtraktion rckgngig machen

	clrf DISP_1				;Lsche Register, von der 32-Bit Zahl in Register ZAHL_C wird ZAHL_B abgezogen bis Ergebnis negativ
	movlw 0x0A				;Lade ZAHL_B mit 10 (0x0000000A)
	movwf ZAHL_B0
	clrf ZAHL_B1
	clrf ZAHL_B2
	clrf ZAHL_B3
conv_hex_dig1
	incf DISP_1				;erhhe Digit um 1
	call sub32				;ZAHL_C minus ZAHL_B, das Ergebnis steht in ZAHL_C
	movf ZAHL_C4,w			;Wenn Ergebnis der Subtraktion noch positiv ist, ist das berlaufregister ZAHL_C4 = 0
	btfsc STATUS,Z			;Prfen ob ZAHL_C4 >0<
	goto conv_hex_dig1		;berlaufregister ZAHL_C4 ist noch 0, zurck zur Schleife
	decf DISP_1				;berlaufregister ZAHL_C4 ist >0<, letzte Erhhung des DISP_ Digit 
	call add32				;   und letzte Subtraktion rckgngig machen

	movff ZAHL_C0,DISP_0	;
	return

convert_dig_hex				;Unterprogramm zum Umwandeln der einzelnen Digits (max.999.999.999) in eine 32-Bit Zahl
	call clear_ZAHL_C		;Lsche Additionsregister ZAHL_C

	movf DISP_8,w			;Hole Digit nach W und prfe ob >0<
	btfsc STATUS,Z
	goto conv_dig_hex7		;Digit ist 0, weiter bei Auswertung des nchst kleineren Digits
	movlw 0x00				;Lade Additionsregister ZAHL_B mit 100.000.000 (0x05F5E100)
	movwf ZAHL_B0
	movlw 0xE1				;
	movwf ZAHL_B1
	movlw 0xF5				;
	movwf ZAHL_B2
	movlw 0x05				;
	movwf ZAHL_B3	
conv_dig_hex8_loop
	call add32				;Addiere ZAHL_B zu ZAHL_C
	decfsz DISP_8			;Reduziere Digit um 1 und Prfe ob 0
	goto conv_dig_hex8_loop	;Digit ist noch nicht 0, zurck zur Schleife
conv_dig_hex7
	movf DISP_7,w			;Hole Digit nach W und prfe ob >0<
	btfsc STATUS,Z
	goto conv_dig_hex6		;Digit ist 0, weiter bei Auswertung des nchst kleineren Digits
	movlw 0x80				;Lade ZAHL_B mit 10.000.000 (0x00989680)
	movwf ZAHL_B0
	movlw 0x96				;
	movwf ZAHL_B1
	movlw 0x98				;
	movwf ZAHL_B2
	clrf ZAHL_B3
conv_dig_hex7_loop
	call add32				;Addiere ZAHL_B zu ZAHL_C
	decfsz DISP_7			;Reduziere Digit um 1 und Prfe ob 0
	goto conv_dig_hex7_loop	;Digit ist noch nicht 0, zurck zur Schleife
conv_dig_hex6
	movf DISP_6,w			;Hole Digit nach W und prfe ob >0<
	btfsc STATUS,Z
	goto conv_dig_hex5		;Digit ist 0, weiter bei Auswertung des nchst kleineren Digits
	movlw 0x40				;Lade ZAHL_B mit 1.000.000 (0x000F4240)
	movwf ZAHL_B0
	movlw 0x42				;
	movwf ZAHL_B1
	movlw 0x0F				;
	movwf ZAHL_B2
	clrf ZAHL_B3
conv_dig_hex6_loop
	call add32				;Addiere ZAHL_B zu ZAHL_C
	decfsz DISP_6			;Reduziere Digit um 1 und Prfe ob 0
	goto conv_dig_hex6_loop	;Digit ist noch nicht 0, zurck zur Schleife
conv_dig_hex5
	movf DISP_5,w			;Hole Digit nach W und prfe ob >0<
	btfsc STATUS,Z
	goto conv_dig_hex4		;Digit ist 0, weiter bei Auswertung des nchst kleineren Digits
	movlw 0xA0				;Lade ZAHL_B mit 100.000 (0x000186A0)
	movwf ZAHL_B0
	movlw 0x86				;
	movwf ZAHL_B1
	movlw 0x01				;
	movwf ZAHL_B2
	clrf ZAHL_B3
conv_dig_hex5_loop
	call add32				;Addiere ZAHL_B zu ZAHL_C
	decfsz DISP_5			;Reduziere Digit um 1 und Prfe ob 0
	goto conv_dig_hex5_loop	;Digit ist noch nicht 0, zurck zur Schleife
conv_dig_hex4
	movf DISP_4,w			;Hole Digit nach W und prfe ob >0<
	btfsc STATUS,Z
	goto conv_dig_hex3		;Digit ist 0, weiter bei Auswertung des nchst kleineren Digits
	movlw 0x10				;Lade ZAHL_B mit 10.000 (0x00002710)
	movwf ZAHL_B0
	movlw 0x27				;
	movwf ZAHL_B1
	clrf ZAHL_B2
	clrf ZAHL_B3
conv_dig_hex4_loop
	call add32				;Addiere ZAHL_B zu ZAHL_C
	decfsz DISP_4			;Reduziere Digit um 1 und Prfe ob 0
	goto conv_dig_hex4_loop	;Digit ist noch nicht 0, zurck zur Schleife
conv_dig_hex3
	movf DISP_3,w			;Hole Digit nach W und prfe ob >0<
	btfsc STATUS,Z
	goto conv_dig_hex2		;Digit ist 0, weiter bei Auswertung des nchst kleineren Digits
	movlw 0xE8				;Lade ZAHL_B mit 1.000 (0x000003E8)
	movwf ZAHL_B0
	movlw 0x03				;
	movwf ZAHL_B1
	clrf ZAHL_B2
	clrf ZAHL_B3
conv_dig_hex3_loop
	call add32				;Addiere ZAHL_B zu ZAHL_C
	decfsz DISP_3			;Reduziere Digit um 1 und Prfe ob 0
	goto conv_dig_hex3_loop	;Digit ist noch nicht 0, zurck zur Schleife
conv_dig_hex2
	movf DISP_2,w			;Hole Digit nach W und prfe ob >0<
	btfsc STATUS,Z
	goto conv_dig_hex1		;Digit ist 0, weiter bei Auswertung des nchst kleineren Digits
	movlw 0x64				;Lade ZAHL_B mit 100 (0x00000064)
	movwf ZAHL_B0
	clrf ZAHL_B1
	clrf ZAHL_B2
	clrf ZAHL_B3
conv_dig_hex2_loop
	call add32				;Addiere ZAHL_B zu ZAHL_C
	decfsz DISP_2			;Reduziere Digit um 1 und Prfe ob 0
	goto conv_dig_hex2_loop	;Digit ist noch nicht 0, zurck zur Schleife
conv_dig_hex1
	movf DISP_1,w			;Hole Digit nach W und prfe ob >0<
	btfsc STATUS,Z
	goto conv_dig_hex0		;Digit ist 0, weiter bei Auswertung des nchst kleineren Digits
	movlw 0x0A				;Lade ZAHL_B mit 10 (0x0000000A)
	movwf ZAHL_B0
	clrf ZAHL_B1
	clrf ZAHL_B2
	clrf ZAHL_B3
conv_dig_hex1_loop
	call add32				;Addiere ZAHL_B zu ZAHL_C
	decfsz DISP_1			;Reduziere Digit um 1 und Prfe ob 0
	goto conv_dig_hex1_loop	;Digit ist noch nicht 0, zurck zur Schleife
conv_dig_hex0
	movf DISP_0,w			;Hole Digit nach W und prfe ob >0<
	btfsc STATUS,Z
	return					;Digit ist 0, weiter bei Auswertung des nchst kleineren Digits
	movwf ZAHL_B0			;Speichere Digit im Additionsregister
	clrf ZAHL_B1
	clrf ZAHL_B2
	clrf ZAHL_B3
	call add32				;Addiere ZAHL_B zu ZAHL_C
	return

clear_ZAHL_C
	clrf ZAHL_C0			;Register ZAHL_C0 - ZAHL_C7 lschen
	clrf ZAHL_C1
	clrf ZAHL_C2
	clrf ZAHL_C3
	clrf ZAHL_C4
	clrf ZAHL_C5
	clrf ZAHL_C6
	clrf ZAHL_C7
	return


;*******************************************************************************************
;
;	Unterprogramme fr die Men Steuerung
;
;*******************************************************************************************

menu_befehl					;Unterprogramm zum Laden der Tastaturbefehle: RC6:RC4 = TA_DB2:TA_DB0
	movlw 0xFF				;lade Register Befehl mit Wert 255
	movwf BEFEHL
	movff PORTC,TEMP_13		;kopiere PORTC in Temp Register	
	btfss TA_DB0			;prfen ob Eingang Taste high ist
	goto menu_befehl1		;Eingang low, weiter
	btfss TA_DB1			;prfen ob Eingang Taste high ist
	goto menu_befehl1		;Eingang low, weiter
	btfsc TA_DB2			;prfen ob Eingang Taste low ist
	return					;Eingang high, zurck
menu_befehl1
	movlw d'1'				;kurze Wartezeit von 1 ms, damit ggf. zweiter Eingang auch low ist
	call wait
	movff PORTC,TEMP_13		;kopiere PORTC in Temp Register	
	swapf TEMP_13			;tausche obere und untere 4 Bit, damit RC6:RC4 in Temp 13 auf Bit2:Bit0 liegen
	bsf TEMP_13,7			;Bit7:Bit3 auf 1 setzen
	bsf TEMP_13,6
	bsf TEMP_13,5
	bsf TEMP_13,4
	bsf TEMP_13,3
	movf TEMP_13,w 			;255 - Temp 13 ergibt die Befehlszahl von 0-5, wobei Wert 0 keinem Befehl entspricht
	sublw 0xFF
	movwf TEMP_13			;prfen ob 0
	btfsc STATUS,Z
	return					; =0, Rcksprung aus dem Unterprogramm
	bsf STATUS,C			; >0<, setze Carry Bit auf 1
	bcf BEFEHL,0
	rlcf BEFEHL,f			;Register mit 11111110 nach links schieben, Carry Bit ( 1 ) wird von rechts eingeschoben
	decfsz TEMP_13			;reduziere Befehlszahl um 1 und prfe ob 0
	goto $-4				;noch nicht 0, zurck zur Schleife
	return					;


menue_1up
	incf MENU_COUNT			;erhhe Menpunkt Zhler um 1
	movf MENU_COUNT_MAX,w	;hole Maximalwert der Menpunkte ins W-Register und erhhe um 1 
	incf MENU_COUNT_MAX,w
	subwf MENU_COUNT,w		;subtrahiere aktuellen Menpunkt nach Erhhung vom Maximalwert und prfe ob 0
	btfss STATUS, Z		
	goto $+0x08				;Ergebnis ist nicht 0
	movlw d'1'				;Ergebnis ist 0, Rcksetzen des Zhlers mit Wert 1
	movwf MENU_COUNT
	movlw d'250'			;Wartezeit von 250ms
	call wait 
	bcf MENU_STAT,6			;lsche das Statusbit
	bcf MENU_STAT,7			;lsche das Statusbit
	return

menue_1dn
	decfsz MENU_COUNT		;reduziere Menpunkt Zhler um 1 und prfe ob 0
	goto $+0x08				;Zhler ist nicht 0
	movf MENU_COUNT_MAX,w	;Zhler ist 0, Rcksetzen des Zhlers mit Maximalwert
	movwf MENU_COUNT
	movlw d'250'			;Wartezeit von 250ms
	call wait 
	bcf MENU_STAT,6			;lsche das Statusbit
	bcf MENU_STAT,7			;lsche das Statusbit
	return


inc_dig_temp_cnt				;Unterprogramm zur Erhhung des Menpunktes horizontal
	movf TEMP_CNT_MAX,w			;
	subwf TEMP_DIG_CNT,w
	btfss STATUS,Z				;prfen ob Zhler gleich dem Maximalwert ist
	goto $+0x0A					;wenn ja, berspringe diesen Befehl	
	movlw d'1'
	movwf TEMP_DIG_CNT
	goto inc_temp_cnt_end
	incf TEMP_DIG_CNT
inc_temp_cnt_end
	bsf TEMP_DIG_STAT,0			;setze das Status bit
	return		

dec_dig_temp_cnt				;Unterprogramm zur Reduzierung des Menpunktes horizontal
	decfsz TEMP_DIG_CNT			;reduziere Zhler um 1 und prfe ob der der Zhler 0 ist
	goto dec_temp_cnt_end		;wenn ja, berspringe diesen Befehl	
	movf TEMP_CNT_MAX,w			;
	movwf TEMP_DIG_CNT
dec_temp_cnt_end
	bsf TEMP_DIG_STAT,0			;setze das Status bit
	return



dig_mess1					;Unterprogramm zur Erhhung/Reduzierung der Werte im Menuepunkt Messen
							;1:Programm
	movlw d'1'				;prfen ob der 1. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w	;
	btfss STATUS,Z			;prfen ob Ergebniss =0		
	goto dig_mess2			;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	btfss TEMP_DIG_STAT,1	;prfen ob Bit fr Erhhung des zu ndernden Wertes gesetzt ist
	goto dec_mess1			;Bit ist nicht gesetzt, gehe zu Reduzierung des zu ndernden Wertes
	incf PROGRAM			;Erhhe Zhler um 1
	movlw d'3'				;prfen ob Max. Wert +1
	subwf PROGRAM,w
	btfsc STATUS,Z
	clrf PROGRAM			;Max. Wert +1 ist erreicht, Rcksetzen mit Min. Wert
	goto dig_mess_end
dec_mess1					;1:Programm
	movf PROGRAM,w			;hole Wert ins W-Register
	btfsc STATUS,Z			;prfe ob Wert <0>
	goto $+0x08				;=0, rcksetzen des Registers mit Max. Wert
	decf PROGRAM			;<0>, reduziere Zhler um 1
	goto dig_mess_end		;
	movlw d'2'				;=0, rcksetzen des Registers mit Max. Wert
	movwf PROGRAM
	goto dig_mess_end
dig_mess2					;2:Messeingang
	movlw d'2'				;prfen ob der 1. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w	;
	btfss STATUS, Z			;prfen ob Ergebniss =0		
	goto dig_mess_end		;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	btfss TEMP_DIG_STAT,1	;prfen ob Bit fr Erhhung des zu ndernden Wertes gesetzt ist
	goto dec_mess2			;Bit ist nicht gesetzt, gehe zu Reduzierung des zu ndernden Wertes
	incf EINGANG			;Erhhe Zhler um 1
	movlw d'2'				;prfen ob Max. Wert +1
	subwf EINGANG,w
	btfsc STATUS,Z		
	clrf EINGANG			;Max. Wert +1 ist erreicht, Rcksetzen mit Min. Wert
	goto dig_mess_end
dec_mess2					;2:Messeingang
	movf EINGANG,w			;hole Wert ins W-Register
	btfsc STATUS,Z			;prfe ob Wert <0>
	goto $+0x08				;=0, rcksetzen des Registers mit Max. Wert
	decf EINGANG			;<0>, reduziere Zhler um 1
	goto dig_mess_end		;
	movlw d'1'				;=0, rcksetzen des Registers mit Max. Wert
	movwf EINGANG
dig_mess_end
	call disp_set_mess		;Unterprogramm fr Anzeige Menpunkt 1.Messen
cursor_mess
	movlw 0x86				;lade Adresse fr Programm ins W-Register
	btfss TEMP_DIG_CNT,0	;kopiere die Cursor Stelle (1-10) ins Low Byte der Tabellenadresse
	movlw 0xC9				;lade Adresse fr Messeingang ins W-Register
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw d'250'			;Wartezeit von 250ms
	call wait 		
	clrf TEMP_DIG_STAT		;lsche Statusbit Register
	return


dig_dds1					;Unterprogramm zur Erhhung/Reduzierung der Werte im Menuepunkt dds
							;1:Wellenform
	movlw d'1'				;prfen ob der 1. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w	;
	btfss STATUS, Z			;prfen ob Ergebniss =0		
	goto dig_dds2			;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	btfss TEMP_DIG_STAT,1	;prfen ob Bit fr Erhhung des zu ndernden Wertes gesetzt ist
	goto dec_dds1			;Bit ist nicht gesetzt, gehe zu Reduzierung des zu ndernden Wertes
	incf DDS_WAVEFORM
	movlw d'4'
	subwf DDS_WAVEFORM,w
	btfsc STATUS,Z		
	clrf DDS_WAVEFORM
	goto dig_dds_end
dec_dds1					;1:Wellenform
	movf DDS_WAVEFORM,w		;hole Wert ins W-Register
	btfsc STATUS,Z			;prfe ob Wert <0>
	goto $+0x08				;wenn ja berspringe diesen Befehl, wenn nein setze Register auf 0
	decf DDS_WAVEFORM		;reduziere Zhler um 1
	goto dig_dds_end		;schiebe Wert in temporres Display Register
	movlw d'3'
	movwf DDS_WAVEFORM
	goto dig_dds_end		;springe zum Ende des Unterprogramms
dig_dds2					;7:Frequenz 10.000er
	movlw d'2'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_dds3
	movff DISP_5,TEMP_13	;Wert fr 10.000er, untere 4 Bit
	call inc_dec_dds
	movff TEMP_13,DISP_5
	goto dig_dds_end
dig_dds3					;8:Frequenz 1.000er
	movlw d'3'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_dds4
	movff DISP_4,TEMP_13	;Wert fr 1.000er, untere 4 Bit
	call inc_dec_dds
	movff TEMP_13,DISP_4
	goto dig_dds_end
dig_dds4					;9:Frequenz 100er
	movlw d'4'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_dds5
	movff DISP_3,TEMP_13	;Wert fr 100er, untere 4 Bit
	call inc_dec_dds
	movff TEMP_13,DISP_3
	goto dig_dds_end
dig_dds5					;10:Frequenz 10er
	movlw d'5'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_dds6
	movff DISP_2,TEMP_13	;Wert fr 10er, untere 4 Bit
	call inc_dec_dds
	movff TEMP_13,DISP_2
	goto dig_dds_end
dig_dds6					;11:Frequenz 1er
	movlw d'6'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_dds7
	movff DISP_1,TEMP_13	;Wert fr 1er, untere 4 Bit
	call inc_dec_dds
	movff TEMP_13,DISP_1
	goto dig_dds_end
dig_dds7					;12:Frequenz 0,1er
	movlw d'7'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_dds8
	movff DISP_0,TEMP_13	;Wert fr 0,1er, untere 4 Bit
	call inc_dec_dds
	movff TEMP_13,DISP_0
	goto dig_dds_end
dig_dds8					;8:Filter Ein/Aus
	movlw d'8'				;prfen ob der 1. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w	;
	btfss STATUS, Z			;prfen ob Ergebniss =0		
	goto dig_dds9			;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	bcf DDS_FILTER,1
	btfss DDS_FILTER,0
	bsf DDS_FILTER,1
	bcf STATUS,C
	rrcf DDS_FILTER
	goto dig_dds_end		;springe zum Ende des Unterprogramms
dig_dds9					;9:Ausgangspegel 10er
	movlw d'9'				;prfen ob der 1. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w	;
	btfss STATUS, Z			;prfen ob Ergebniss =0		
	goto dig_dds10			;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	btfsc TEMP_DIG_STAT,1	;prfen ob Bit fr Erhhung des zu ndernden Wertes NICHT gesetzt ist
	goto dec_dds9			;Bit ist gesetzt, gehe zu Reduzierung des zu ndernden Wertes
	incf DDS_VOL
	movlw d'34'
	subwf DDS_VOL,w
	btfsc STATUS,Z		
	clrf DDS_VOL
	goto dig_dds_end		;springe zum Ende des Unterprogramms
dec_dds9					;9:Ausgangspegel 1er
	movf DDS_VOL,w			;hole Wert ins W-Register
	btfsc STATUS,Z			;prfe ob Wert <0>
	goto $+0x08				;wenn ja berspringe diesen Befehl, wenn nein setze Register auf 0
	decf DDS_VOL			;reduziere Zhler um 1
	goto $+0x08				;schiebe Wert in temporres Display Register
	movlw d'33'
	movwf DDS_VOL
	goto dig_dds_end		;springe zum Ende des Unterprogramms
dig_dds10					;11:Winkel 100er
	movlw d'10'				;prfen ob der 3. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w	;
	btfss STATUS, Z			;prfen ob Ergebniss =0		
	goto dig_dds11			;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	btfss TEMP_DIG_STAT,1	;prfen ob Bit fr Erhhung des zu ndernden Wertes gesetzt ist
	goto dec_dds10			;Bit ist nicht gesetzt, gehe zu Reduzierung des zu ndernden Wertes
	incf ZAHL_A2
	movlw d'4'
	subwf ZAHL_A2,w
	btfsc STATUS,Z		
	clrf ZAHL_A2
	goto dig_dds_end		;springe zum Ende des Unterprogramms
dec_dds10					;11:Winkel 100er
	movf ZAHL_A2,w			;hole Wert ins W-Register
	btfsc STATUS,Z			;prfe ob Wert <0>
	goto $+0x08				;wenn ja berspringe diesen Befehl, wenn nein setze Register auf 0
	decf ZAHL_A2			;reduziere Zhler um 1
	goto dig_dds_end		;springe zum Ende des Unterprogramms
	movlw d'3'
	movwf ZAHL_A2
	goto dig_dds_end
dig_dds11					;12:Winkel 10er
	movlw d'11'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_dds12
	movff ZAHL_A1,TEMP_13	;Wert fr Winkel 10er, untere 4 Bit
	call inc_dec_dds
	movff TEMP_13,ZAHL_A1
	goto dig_dds_end
dig_dds12					;13:Winkel 1er
	movlw d'12'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_dds_end
	movff ZAHL_A0,TEMP_13	;Wert fr Winkel 1er, untere 4 Bit
	call inc_dec_dds
	movff TEMP_13,ZAHL_A0
	goto dig_dds_end
dig_dds_end
	call disp_set_dds		;Unterprogramm fr Anzeige Menpunkt 3.VCO/Wobbel
cursor_dds
	movf TEMP_DIG_CNT,w		;kopiere die Cursor Stelle (1-10) ins Low Byte der Tabellenadresse
	movwf TBLPTRL
	decf TBLPTRL			;-1 (Versatz)
	movlw 0x7B				;High Byte der Tabellenadresse 
	movwf TBLPTRH
	TBLRD*+ 				;lese den Tabellenwert ins TABLAT Register und Erhhe die Adresse um 1
	movf TABLAT,W 			;Hole den Tabellenwert nach W und send ihn an das Display
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw d'250'			;Wartezeit von 250ms
	call wait 		
	clrf TEMP_DIG_STAT		;lsche Statusbit Register
	return

dig_vco1					;Unterprogramm zur Erhhung/Reduzierung der Werte im Menuepunkt vco
							;1	2	3	4	5	6	7	8	9	10	11	12	13	14	15	16
							;V	C	O		L	O	G		F	u	0	0	0	0	H	z
							;D	3		S	1	0	2	4		M	B		1	,	0	s
							;1:Programm: VCO=4, Wobbel=5
	movlw d'1'				;prfen ob der 1. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w	;
	btfss STATUS, Z			;prfen ob Ergebniss =0		
	goto dig_vco2			;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	movlw d'4'
	subwf PROGRAM,w
	btfss STATUS,Z
	goto $+0x0E
	movlw d'5'				;Wobbel ist gewhlt
	movwf PROGRAM
	movlw d'10'				;Wobbel hat max 10 Stellen, die gesetzt werden
	movwf TEMP_CNT_MAX
	goto dig_vco_end
	movlw d'4'				;VCO ist gewhlt
	movwf PROGRAM
	movlw d'7'				;VCO hat max 7 Stellen, die gesetzt werden
	movwf TEMP_CNT_MAX
	goto dig_vco_end		;springe zum Ende des Unterprogramms
dig_vco2					;2:Kennlinie: 0 = Linear, 1 = Logarithmisch
	movlw d'2'				;prfen ob der 2. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w	;
	btfss STATUS, Z			;prfen ob Ergebniss =0		
	goto dig_vco3			;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	movlw d'0'				;Hole Wert 0 nach W
	btfss KENNLIN,0			;prfen ob Kennlinie = 1
	movlw d'1'				;Kennlinie = 0, Hole Wert 1 nach W und schreibe ihn ins Register
	movwf KENNLIN
	goto dig_vco_end		;springe zum Ende des Unterprogramms
dig_vco3					;3:Frequenz Fu 1.000er
	movlw d'3'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_vco4
	movff DISP_4,TEMP_13	;Wert fr 1.000er, untere 4 Bit
	call inc_dec_dds
	movff TEMP_13,DISP_4
	goto dig_vco_end
dig_vco4					;4:Frequenz Fu 100er
	movlw d'4'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_vco5
	movff DISP_3,TEMP_13	;Wert fr 100er, untere 4 Bit
	call inc_dec_dds
	movff TEMP_13,DISP_3
	goto dig_vco_end
dig_vco5					;5:Frequenz Fu 10er
	movlw d'5'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_vco6
	movff DISP_2,TEMP_13	;Wert fr 10er, untere 4 Bit
	call inc_dec_dds
	movff TEMP_13,DISP_2
	goto dig_vco_end
dig_vco6					;10:Frequenz 1er
	movlw d'6'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_vco7
	movff DISP_1,TEMP_13	;Wert fr 1er, untere 4 Bit
	call inc_dec_dds
	movff TEMP_13,DISP_1
	goto dig_vco_end
dig_vco7					;7:Anzahl Dekaden : 1-3
	movlw d'7'				;prfen ob der 3. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w	;
	btfss STATUS, Z			;prfen ob Ergebniss =0		
	goto dig_vco8			;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	btfss TEMP_DIG_STAT,1	;prfen ob Bit fr Erhhung des zu ndernden Wertes gesetzt ist
	goto dec_vco7			;Bit ist gesetzt, gehe zu Reduzierung des zu ndernden Wertes
	incf DEKADE				;Erhhe Wert
	movlw d'4'				;
	subwf DEKADE,w
	btfss STATUS,Z		
	goto dig_vco_end		;springe zum Ende des Unterprogramms
	movlw d'1'				;Rcksetzen mit Wert 1
	movwf DEKADE
	goto dig_vco_end		;springe zum Ende des Unterprogramms
dec_vco7					;3:Anzahl Dekaden
	decfsz DEKADE			;reduziere Zhler um 1 und prfe ob 0
	goto dig_vco_end		;>0<, springe zum Ende des Unterprogramms
	movlw d'3'				;0, Rcksetzen mit Wert 3
	movwf DEKADE
	goto dig_vco_end		;springe zum Ende des Unterprogramms
dig_vco8					;8:Anzahl Schritte/Durchlauf : 32 ... 1024
	movlw d'8'				;prfen ob der 1. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w	;
	btfss STATUS, Z			;prfen ob Ergebniss =0		
	goto dig_vco9			;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	btfsc TEMP_DIG_STAT,1	;prfen ob Bit fr Erhhung des zu ndernden Wertes NICHT (btfsc) gesetzt ist
	goto dec_vco8			;Bit ist gesetzt, gehe zu Reduzierung des zu ndernden Wertes
	incf WOB_STEP			;Erhhe Register um 1 (Register hat Werte von 1 bis 6)
	movlw d'7'				;wenn nach Erhhung Registerwert = 7: berlauf, rcksetzen des Registers mit Wert 1 
	subwf WOB_STEP,w
	btfss STATUS,Z		
	goto dig_vco_end		;Wert <7
	movlw d'1'				;Wert =7, Rcksetzen des Registers mit Wert 1 
	movwf WOB_STEP
	goto dig_vco_end
dec_vco8					;8:Schritte/Durchlauf
	decfsz WOB_STEP			;reduziere Register um 1 (Register hat Werte von 1 bis 6) und prfe ob 0
	goto dig_vco_end		;Wert >0
	movlw d'6'				;Wert =0, Rcksetzen des Registers mit Wert 6 
	movwf WOB_STEP
	goto dig_vco_end
dig_vco9					;9: Manuell/Auto + Kontinuierlich/Burst
	movlw d'9'				;prfen ob der 9. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w	;
	btfss STATUS, Z			;prfen ob Ergebniss =0		
	goto dig_vco10			;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	btfss TEMP_DIG_STAT,1	;prfen ob Bit fr Erhhung des zu ndernden Wertes gesetzt ist
	goto dec_vco9			;Bit ist nicht gesetzt, gehe zu Reduzierung des zu ndernden Wertes
	incf WOB_M_A			;erhhe Wert (Register hat Werte von 0 bis 3)
	movlw d'4'				;wenn nach Erhhung Registerwert = 4: berlauf, rcksetzen des Registers mit Wert 0 
	subwf WOB_M_A,w
	btfss STATUS,Z		
	goto dig_vco9_end		;Wert <4
	clrf WOB_M_A			;Wert =4, Rcksetzen des Registers mit Wert 0 
	goto dig_vco9_end
dec_vco9					;9:Manuell/Auto + Kontinuierlich/Burst
	movf WOB_M_A,w			;hole Wert ins W-Register
	btfsc STATUS,Z			;prfe ob Wert <0>
	goto $+0x08				;Wert ist 0, Rcksetzen des Registers mit Wert 3
	decf WOB_M_A			;Wert <0>, reduziere Zhler um 1
	goto dig_vco9_end		;
	movlw d'3'				;Wert ist 0, Rcksetzen des Registers mit Wert 3
	movwf WOB_M_A
dig_vco9_end
	movlw d'9'				;Bei Manuell betrgt die Anzahl Stellen 9, Hole Wert 9 nach W
	btfss WOB_M_A,0			;Prfen, ob Bit0 fr Manuell gesetzt ist
	movlw d'10'				;Bit ist nicht gesetzt, d.h. Automatisch, Hole Wert 10 nach W
	movwf TEMP_CNT_MAX		;Kopiere W nach TEMP_CNT_MAX
dig_vco10					;10: Zeit/Schritt
	movlw d'10'				;prfen ob der 1. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w	;
	btfss STATUS, Z			;prfen ob Ergebniss =0		
	goto dig_vco_end		;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	btfss TEMP_DIG_STAT,1	;prfen ob Bit fr Erhhung des zu ndernden Wertes gesetzt ist
	goto dec_vco10			;Bit ist nicht gesetzt, gehe zu Reduzierung des zu ndernden Wertes
	incf WOB_ZEIT			;erhhe Wert (Register hat Werte von 0 bis 9)
	movlw d'11'				;wenn nach Erhhung Registerwert = 11: berlauf, rcksetzen des Registers mit Wert 0 
	subwf WOB_ZEIT,w
	btfss STATUS,Z		
	goto dig_vco_end		;Wert <11
	clrf WOB_ZEIT			;Wert =11, Rcksetzen des Registers mit Wert 0 
	goto dig_vco_end
dec_vco10					;10:Zeit/Schritt
	movf WOB_ZEIT,w			;hole Wert ins W-Register
	btfsc STATUS,Z			;prfe ob Wert <0>
	goto $+0x08				;Wert ist 0, Rcksetzen des Registers mit Wert 10
	decf WOB_ZEIT			;Wert <0>, reduziere Zhler um 1
	goto dig_vco_end		;
	movlw d'10'				;Wert ist 0, Rcksetzen des Registers mit Wert 10
	movwf WOB_ZEIT
dig_vco_end
	call clear_display		;Aufruf des Unterprogramms zum Lschen des Displays
	call disp_set_vco		;Unterprogramm fr Anzeige Menpunkt 3.VCO/Wobbel
cursor_vco
	movf TEMP_DIG_CNT,w		;kopiere die Cursor Stelle (1-10) ins Low Byte der Tabellenadresse
	addlw d'11'				;Versatz 
	movwf TBLPTRL
	movlw 0x7B				;High Byte der Tabellenadresse 
	movwf TBLPTRH
	TBLRD*+ 				;lese den Tabellenwert ins TABLAT Register und Erhhe die Adresse um 1
	movf TABLAT,W 			;Hole den Tabellenwert nach W und send ihn an das Display
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw d'250'			;Wartezeit von 250ms
	call wait 		
	clrf TEMP_DIG_STAT		;lsche Statusbit Register
	return

dig_dacgain
	movlw d'0'				;hole Wert 0 nach W
	btfss DAC_GAIN,0		;prfe ob DAC_GAIN = 1
	movlw d'1'				;=0, setze DAC_GAIN auf 1
	movwf DAC_GAIN			;=1, setze DAC_GAIN auf 0
	call disp_set_dacgain	;Aufruf des Unterprogramms
	movlw d'250'			;Wartezeit von 250ms
	call wait 		
	return	

inc_contrast
	incf CONTRAST
	movlw d'64'
	subwf CONTRAST,w
	btfss STATUS,C
	goto incdec_contr_end
	movlw d'63'
	movwf CONTRAST
	goto incdec_contr_end
dec_contrast
	decf CONTRAST
	movlw d'30'
	subwf CONTRAST,w
	btfsc STATUS,C
	goto incdec_contr_end
	movlw d'30'
	movwf CONTRAST
incdec_contr_end
	call init_display
	bsf TEMP_DIG_STAT,0
	return


inc_dec_dds
	btfss TEMP_DIG_STAT,1	;prfen ob Bit fr Erhhung des zu ndernden Wertes gesetzt ist
	goto dec_dds			;Bit ist nicht gesetzt, gehe zu Reduzierung des zu ndernden Wertes
	incf TEMP_13			;Erhhe Wert um 1
	movlw d'10'				;prfen ob 10, d.h. bergelaufen
	subwf TEMP_13,w
	btfsc STATUS,Z		
	clrf TEMP_13			;ja, bergelaufen, rcksetzen auf 0
	return
dec_dds
	movf TEMP_13,w			;hole Wert ins W-Register
	btfsc STATUS,Z			;prfe ob Wert <0>
	goto $+0x08				;=0, rcksetzen des Registers mit Wert 9
	decf TEMP_13			;<0>, reduziere Zhler um 1
	return					;
	movlw d'9'				;=0, rcksetzen des Registers mit Wert 9
	movwf TEMP_13
	return

;*******************************************************************************************
;
;	Unterprogramme zum Senden von Daten an das Digitalpoti und den DAC
;
;*******************************************************************************************


dds_pegel					;Unterprogramms zum senden des DDS Pegel an das Digitalpoti MCP4151
	movff DDS_VOL,TBLPTRL	;kopiere DDS Pegel in das Low Byte der Tabellenadresse 
	movlw 0x7E				;High Byte der Tabellenadresse 
	movwf TBLPTRH
	TBLRD*	 				;lese den Tabellenwert ins TABLAT Register
	movf TABLAT,W 			;Hole den Tabellenwert nach W und kopiere ihn ins Potentimeter Register, L-Byte
	movwf POTI_L
	clrf POTI_H				;lsche H-Byte
	incfsz POTI_L			;L-Byte muss um 1 erhht werden, erhhe und prfe auf berlauf
	goto $+6				;kein berlauf, H-Byte =0
	incf POTI_H				;berlauf vorhanden, H-Byte =1
	bsf SI_CLK				;Takt auf High setzen
	nop
	bcf CS_VOL				;Chip Select des Digitalpoti auf Low setzen
	movf POTI_H,w
	call write_data			;Senden des H-Byte 	
	movf POTI_L,w
	call write_data			;Senden des L-Byte 	
	bsf CS_VOL				;Chip Select des Digitalpoti wieder auf High setzen
	return


send_dac_data				; 
	rlcf DAC_L				;Low und High Byte 2x nach links schieben (siehe MCP4812 Datenblatt)
	rlcf DAC_H
	rlcf DAC_L
	rlcf DAC_H
	bsf DAC_H,4				;Shutdown Bit auf High setzen
	bcf CS_DAC				;Chip select auf Low setzen
	movf DAC_H,w			;senden des High Bytes
	call write_data
	movf DAC_L,w			;senden des low Bytes
	call write_data
	nop
	bsf CS_DAC				;Chip select auf High setzen, Daten werden bernommen
	bcf SI_DAT				;Pin auf Low zurcksetzen
	return

write_data
	movwf SHIFT_REG			;Das W-Register wird in das Reg. SHIFT_REG kopiert
	bcf SI_CLK				;Takt auf low setzen, damit die Daten bei der steigenden Flanke bernommen werden
	movlw d'8'				;es sollen 8 Bits bertragen werden
	movwf SHIFT_CNT			;Zhler fr die bertragenen bits
write_data_loop
	rlcf SHIFT_REG,F			;Das Zeichen wird nach links verschoben. Das Carryflag wird an den Ausgang weitergegeben
	btfsc STATUS,C			;Prfen, welchen Wert das Carryflag hat
	bsf SI_DAT				;Carryflag = 1 -> Pin SI auf Highpegel setzen
	btfss STATUS,C			;Nochmal prfen, da immer nur ein Befehl bersprungen wird
	bcf SI_DAT				;Carryflag = 0 -> Pin SI auf Lowpegel setzen
	nop						;kurze Wartezeit, um die bertragungsgeschwindigkeit zu reduzieren
	nop
	bsf SI_CLK				;mit der steigenden Flanke werden die Daten bernommen
	nop						;kurze Wartezeit
	nop
	bcf SI_CLK				;Takt wieder auf low setzen
	decfsz SHIFT_CNT,F		;ein bit wurde bertragen -> Zhler um 1 verringern
	goto write_data_loop	;nchstes bit bertragen
	return					;Rcksprung aus dem Unterprogramm



;*******************************************************************************************
;
;	Unterprogramme zum Lesen / Schreiben von Einstellungswerten in das interne EEPROM
;
;*******************************************************************************************

readeepr_data				;lesen/kopieren der Starteinstellungen aus dem internen EEPROM
	movlw 0x00				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf PROGRAM			;Kopieren der Daten von W nach F, 0=Frequenz,1=Periode,2=Zhler,3=DDS,4=Wobbel

	movlw 0x01				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf EINGANG			;Kopieren der Daten von W nach F, 0 = TTL, 1 = Analog, 2 = DDS, 3 = Vorteiler

	movlw 0x02				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf DDS_WAVEFORM		;Kopieren der Daten von W nach F, DDS Wellenform: 0=Sinus, 1=Dreieck, 2=Rechteck

	movlw 0x03				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf DDS_VOL			;Kopieren der Daten von W nach F, DDS Ausgangspegel: 0-63 entspricht 0dB bis -63dB

	movlw 0x04				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf DDS_FREQ_0		;Kopieren der Daten von W nach F, DDS Frequenz (x10 fr eine Nachkommastelle) 

	movlw 0x05				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf DDS_FREQ_1		;Kopieren der Daten von W nach F, DDS Frequenz (x10 fr eine Nachkommastelle) 

	movlw 0x06				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf DDS_FREQ_2		;Kopieren der Daten von W nach F, DDS Frequenz (x10 fr eine Nachkommastelle) 

	movlw 0x07				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf DDS_PHASE_L		;Kopieren der Daten von W nach F, DDS Phasenwinkel L-Byte 

	movlw 0x08				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf DDS_PHASE_H		;Kopieren der Daten von W nach F, DDS Phasenwinkel H-Byte 

	movlw 0x09				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf DDS_FILTER		;Kopieren der Daten von W nach F, DDS Filter On/Off 

	movlw 0x0A				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf WOB_FU_L			;Kopieren der Daten von W nach F, Wobbel Fu L-Byte

	movlw 0x0B				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf WOB_FU_M			;Kopieren der Daten von W nach F, Wobbel Fu M-Byte

	movlw 0x0C				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf WOB_FU_H			;Kopieren der Daten von W nach F, Wobbel Fu H-Byte

	movlw 0x0D				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf KENNLIN			;Kopieren der Daten von W nach F, Wobbel Kennlinie: 0 = Linear, 1 = Logarithmisch

	movlw 0x0E				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf DEKADE			;Kopieren der Daten von W nach F, Wobbel Dekaden: 1-3

	movlw 0x0F				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf WOB_STEP			;Kopieren der Daten von W nach F, Wobbel Steps: 1=1024, 2=512, 3=256, 4=128, 5=64, 6=32

	movlw 0x10				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf WOB_M_A			;Kopieren der Daten von W nach F, Wobbel Man/Aut: 0 = Automatisch, 1 = manuell

	movlw 0x11				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf WOB_ZEIT			;Kopieren der Daten von W nach F, Wobbel delta t: 	0=10ms, 1=20ms, 2=50ms, 3=100ms, 4=200ms,
							;		5=500ms, 6=1000ms, 7=2000ms

	movlw 0x12				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf DAC_GAIN			;Kopieren der Daten von W nach F, DAC Verstrkung: 1 = 0-2048mV, 0 = 0-4096mV

	movlw 0x13				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf CONTRAST			;Kopieren der Daten von W nach F, Kontrast Display: 24-63


writeepr_mess				;Unterprogramm zum Schreiben von Einstellungswerten in das interne EEPROM
	
	movff PROGRAM,TEMP_13	;kopiere Wert fr Programm in das Schreibregister fr das EEPROM
	movlw 0x00				;Lade die Adresse des EEPROM in die der Wert geschrieben werden soll
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM

	movff EINGANG,TEMP_13	;kopiere Wert fr Eingang in das Schreibregister fr das EEPROM
	movlw 0x01				;Lade die Adresse des EEPROM in die der Wert geschrieben werden soll
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM
	return

writeepr_dds
	movff PROGRAM,TEMP_13		;kopiere Wert fr DDS Frequenz (x10) in das Schreibregister fr das EEPROM
	movlw 0x00				;Lade die Adresse des EEPROM in die der Wert geschrieben werden soll
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM

	movff DDS_WAVEFORM,TEMP_13	;kopiere Wert fr DDS Wellenform in das Schreibregister fr das EEPROM
	movlw 0x02				;Lade die Adresse des EEPROM in die der Wert geschrieben werden soll
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM

	movff DDS_VOL,TEMP_13	;kopiere Wert fr DDS Signalpegel in das Schreibregister fr das EEPROM
	movlw 0x03				;Lade die Adresse des EEPROM in die der Wert geschrieben werden soll
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM

	movff DDS_FREQ_0,TEMP_13		;kopiere Wert fr DDS Frequenz (x10) in das Schreibregister fr das EEPROM
	movlw 0x04				;Lade die Adresse des EEPROM in die der Wert geschrieben werden soll
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM

	movff DDS_FREQ_1,TEMP_13		;kopiere Wert fr DDS Frequenz (x10) in das Schreibregister fr das EEPROM
	movlw 0x05				;Lade die Adresse des EEPROM in die der Wert geschrieben werden soll
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM

	movff DDS_FREQ_2,TEMP_13		;kopiere Wert fr DDS Frequenz (x10) in das Schreibregister fr das EEPROM
	movlw 0x06				;Lade die Adresse des EEPROM in die der Wert geschrieben werden soll
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM

	movff DDS_PHASE_L,TEMP_13		;kopiere Wert fr DDS Phasenwinkel (x10) in das Schreibregister fr das EEPROM
	movlw 0x07				;Lade die Adresse des EEPROM in die der Wert geschrieben werden soll
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM

	movff DDS_PHASE_H,TEMP_13		;kopiere Wert fr DDS Phasenwinkel (x10) in das Schreibregister fr das EEPROM
	movlw 0x08				;Lade die Adresse des EEPROM in die der Wert geschrieben werden soll
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM

	movff DDS_FILTER,TEMP_13		;kopiere Wert fr DDS Frequenz (x10) in das Schreibregister fr das EEPROM
	movlw 0x09				;Lade die Adresse des EEPROM in die der Wert geschrieben werden soll
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM
	return

writeepr_vco_wob			;Unterprogramms zum Schreiben von Einstellungswerten in das interne EEPROM
	movff PROGRAM,TEMP_13	;Kopiere Programm in das Schreibregister fr das EEPROM
	movlw 0x00				;Lade die Adresse des EEPROM in die der Wert geschrieben werden soll
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM

	movf ZAHL_C0,w			;Kopiere Fu (L-Byte) von den Rechenregistern in die Frequenz- und Schreibregister fr das EEPROM
	movwf WOB_FU_L			;
	movwf TEMP_13			;
	movlw 0x0A				;Lade die Adresse des EEPROM in die der Wert geschrieben werden soll
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM

	movf ZAHL_C1,w			;Kopiere Fu (M-Byte) von den Rechenregistern in die Frequenz- und Schreibregister fr das EEPROM
	movwf WOB_FU_M
	movwf TEMP_13			;
	movlw 0x0B				;Lade die Adresse des EEPROM in die der Wert geschrieben werden soll
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM

	movf ZAHL_C2,w			;Kopiere Fu (H-Byte) von den Rechenregistern in die Frequenz- und Schreibregister fr das EEPROM
	movwf WOB_FU_H
	movwf TEMP_13			;
	movlw 0x0C				;Lade die Adresse des EEPROM in die der Wert geschrieben werden soll
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM

	movff KENNLIN,TEMP_13	;Kopiere Kennlinie (Lin/Log) in das Schreibregister fr das EEPROM
	movlw 0x0D				;Lade die Adresse des EEPROM in die der Wert geschrieben werden soll
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM

	movff DEKADE,TEMP_13	;Kopiere Anzahl Dekaden in das Schreibregister fr das EEPROM
	movlw 0x0E				;Lade die Adresse des EEPROM in die der Wert geschrieben werden soll
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM

	movff WOB_STEP,TEMP_13	;Kopiere Programm in das Schreibregister fr das EEPROM
	movlw 0x0F				;Lade die Adresse des EEPROM in die der Wert geschrieben werden soll
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM

	movff WOB_M_A,TEMP_13	;Kopiere Kennlinie (Lin/Log) in das Schreibregister fr das EEPROM
	movlw 0x10				;Lade die Adresse des EEPROM in die der Wert geschrieben werden soll
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM

	movff WOB_ZEIT,TEMP_13	;Kopiere Anzahl Dekaden in das Schreibregister fr das EEPROM
	movlw 0x11				;Lade die Adresse des EEPROM in die der Wert geschrieben werden soll
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM
	return


readEE							;Unterprogramm zum lesen der Zelle deren Adresse in W steht (internes EEPROM)
	movwf EEADR 				; Data Memory Address to read
	bcf EECON1,EEPGD 			; Point to DATA memory
	bcf EECON1,CFGS 			; Access EEPROM
	bsf EECON1,RD 				; EEPROM Read
	movf EEDATA,W 				; W = EEDATA
	return

writeEE							;Unterprogramm zum Schreiben des Wertes in TEMP_13 an die Adresse, die in W steht (internes EEPROM)
	movwf EEADR 				; Data Memory Address to write
	movf TEMP_13,W 				;
	movwf EEDATA 				; Data Memory Value to write
	bcf EECON1,EEPGD 			; Point to DATA memory
	bcf EECON1,CFGS 			; Access EEPROM
	bsf EECON1,WREN 			; Enable writes
	bcf INTCON,GIE 				; Disable Interrupts
	movlw 55h 					;Required Sequence
	movwf EECON2 				; Write 55h
	movlw 0AAh 					;
	movwf EECON2 				; Write 0AAh
	bsf EECON1,WR 				; Set WR bit to begin write
	;bsf INTCON,GIE 				; Enable Interrupts
	btfsc EECON1,WR				;prfen, ob das EEPROM beschrieben werden kann
	goto $-0x04					;warten, bis das bit WR von der Hardware zurckgesetzt wurde
	bcf EECON1,WREN 			; Disable writes on write complete (EEIF set)
	return



;*******************************************************************************************
; Unterprogramme zur Anzeige auf dem Display
;*******************************************************************************************

disp_menue_1
	call clear_display		;Aufruf des Unterprogramms zum Lschen des Displays/Cursors
	call cursor_off
	movlw d'1'				;
	subwf MENU_COUNT,w	;
	btfss STATUS,Z		
	goto disp_menue_2
	call dispm1_mess
	goto disp_menue_end
disp_menue_2
	movlw d'2'				;
	subwf MENU_COUNT, w	;
	btfss STATUS,Z		
	goto disp_menue_3
	call dispm1_dds
	goto disp_menue_end
disp_menue_3
	movlw d'3'				;
	subwf MENU_COUNT, w	;
	btfss STATUS,Z		
	goto disp_menue_4
	call dispm1_vco
	goto disp_menue_end
disp_menue_4
	movlw d'4'				;
	subwf MENU_COUNT, w	;
	btfss STATUS,Z		
	goto disp_menue_end
	call dispm1_einstell
	goto disp_menue_end
disp_menue_end
	movlw d'250'				;Wartezeit von 250ms
	call wait 	
	bsf MENU_STAT,7				;setze das Status Bit
	return

dispm1_mess
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'1'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'M'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A's'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A's'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'n'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return

dispm1_dds
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'2'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'D'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'D'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'S'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return

dispm1_vco
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'3'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'V'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'C'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'O'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'/'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'W'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'o'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'b'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'b'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'l'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return


dispm1_einstell
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'4'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'E'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'i'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'n'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A's'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A't'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'l'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'l'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'u'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'n'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'g'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'n'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return


disp_main
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw d'1'
	subwf PROGRAM,w
	btfss STATUS,Z
	goto disp_prog_freq
	movlw A'P'					;ASCII:P fr Periode
	call disp_daten				;bertrage das Zeichen
	goto disp_wert
disp_prog_freq
	movlw A'F'					;ASCII:F fr Frequenz
	call disp_daten				;bertrage das Zeichen
disp_wert
	movlw A':'					;ASCII: ":"
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;Daten fr "blank"
	call disp_daten				;bertrage das Zeichen
	movf DISP_6,w				;Daten fr Millionen 1er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	btfss KOMMA,6				;prfen ob ein Komma gesetzt werden soll
	goto $+0x08					;nein, weiter
	movlw A','					;ja, Daten fr ","
	call disp_daten				;bertrage das Zeichen
	movf DISP_5,w				;Daten fr 100.000er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	btfss KOMMA,5				;prfen ob ein Komma gesetzt werden soll
	goto $+0x08					;nein, weiter
	movlw A','					;ja, Daten fr ","
	call disp_daten				;bertrage das Zeichen
	movf DISP_4,w				;Daten fr 10.000er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	btfss KOMMA,4				;prfen ob ein Komma gesetzt werden soll
	goto $+0x08					;nein, weiter
	movlw A','					;ja, Daten fr ","
	call disp_daten				;bertrage das Zeichen
	movf DISP_3,w				;Daten fr 1.000er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movf DISP_2,w				;Daten fr 100er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movf DISP_1,w				;Daten fr 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movf DISP_0,w				;Daten fr 1er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;Daten fr "blank"
	call disp_daten				;bertrage das Zeichen
	movf UNIT,w					;Zeichen fr die Einheit, z.B. k fr kilo, m fr milli, u fr mikro, n fr nano
	call disp_daten				;bertrage das Zeichen

	movlw d'1'
	subwf PROGRAM,w
	btfss STATUS,Z
	goto $+0x0C
	movlw A's'					;ASCII:s fr Sekunde
	call disp_daten				;bertrage das Zeichen
	goto disp_main_z2
	movlw A'H'					;ASCII:H fr Herz
	call disp_daten				;bertrage das Zeichen
	movlw A'z'					;ASCII:z
	call disp_daten				;bertrage das Zeichen
disp_main_z2
	movlw 0xC0					;Zeichen 1 in der 2. Zeile (=0x80 + 0x40)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movf DISP_PRESC,w			;Daten fr Prescaler
	call convdisp_hex_2dig		;Unterprogramms zum Auftrennen in 10er und 1er plus Anzeige
	movlw 0x20					;Daten fr "blank"
	call disp_daten				;bertrage das Zeichen
	call disp_freq_eingang		;Bit ist nicht gesetzt, Aufruf des Unterprogramms
	return						;Bit ist gesetzt, kein Update erforderlich

disp_zaehl
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'Z'					;ASCII:Z
	call disp_daten				;bertrage das Zeichen
	movlw A':'					;ASCII: ":"
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;Daten fr "blank"
	call disp_daten				;bertrage das Zeichen
	movf DISP_8,w				;Daten fr Millionen  100er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movf DISP_7,w				;Daten fr Millionen  10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movf DISP_6,w				;Daten fr Millionen 1er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII: "."
	call disp_daten				;bertrage das Zeichen
	movf DISP_5,w				;Daten fr 100.000er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movf DISP_4,w				;Daten fr 10.000er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movf DISP_3,w				;Daten fr 1.000er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII: "."
	call disp_daten				;bertrage das Zeichen
	movf DISP_2,w				;Daten fr 100er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movf DISP_1,w				;Daten fr 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movf DISP_0,w				;Daten fr 1er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;Daten fr "blank"
	call disp_daten				;bertrage das Zeichen
	movlw A'n'					;ASCII:n fr Anzahl
	call disp_daten				;bertrage das Zeichen
	movlw 0xC0					;Zeichen 1 in der 2. Zeile (=0x80 + 0x40)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	call disp_freq_eingang		;Bit ist nicht gesetzt, Aufruf des Unterprogramms
	return

disp_dds
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'D'					;ASCII: "D"
	call disp_daten				;bertrage das Zeichen
	movlw A'D'					;ASCII: "D"
	call disp_daten				;bertrage das Zeichen
	movlw A'S'					;ASCII: "S"
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;Daten fr "blank"
	call disp_daten				;bertrage das Zeichen
	call disp_freq				;Aufruf des Unterprogramms zum Schreiben der Frequenz
	movlw A'H'					;ASCII: "H"
	call disp_daten				;bertrage das Zeichen
	movlw A'z'					;ASCII: "z"
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;Daten fr "blank"
	call disp_daten				;bertrage das Zeichen
	call disp_dds_filter
	;2.Zeile
	movlw 0xC0					;Zeichen 1 in der 2. Zeile 
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	call disp_dds_waveform		;Aufruf des Unterprogramms zum Schreiben der Wellenform (SIN, TRI, SAW, SQR)
	movlw 0x20					;Daten fr "blank"
	call disp_daten				;bertrage das Zeichen
	call disp_dds_vol			;Aufruf des Unterprogramms zum Schreiben des Ausgangspegels (-00dB bis -42dB)
	movlw 0x20					;Daten fr "blank"
	call disp_daten				;bertrage das Zeichen
	call disp_dds_phase			;Unterprogramm zum Schreiben der Phase
	return

disp_vco
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	call disp_vco_wob
	movlw 0x8C					;Zeichen 13 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'H'					;ASCII: "H"
	call disp_daten				;bertrage das Zeichen
	movlw A'z'					;ASCII: "z"
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;ASCII: "blank"
	call disp_daten				;bertrage das Zeichen	
	call disp_dds_filter
	;2.Reihe
	movlw 0xC0					;Zeichen 1 in der 2. Zeile 
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	call disp_vco_kl			;Aufruf des Unterprogramms zum Schreiben der Kennlinie (LIN/LOG)
	movlw 0x20					;ASCII: "blank"
	call disp_daten				;bertrage das Zeichen	
	movf DEKADE,w				;Daten fr Anzahl Dekaden
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw A'D'					;ASCII: "D"
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII: "e"
	call disp_daten				;bertrage das Zeichen
	movlw A'k'					;ASCII: "k"
	call disp_daten				;bertrage das Zeichen
	btfss PROGRAM,0				;Prfen, ob Wobbel gewhlt ist (Wobbel =5, Bit 0 prfen)
	goto disp_vco_wf			;Wobbel ist nicht gewhlt, weiter bei Anzeige Wellenform
	movlw 0x20					;ASCII: "blank"
	call disp_daten				;bertrage das Zeichen	
	movlw A'M'					;ASCII: hole "M" ins W-Register	
	btfss WOB_M_A,0				;prfen ob "Manuell" (1) gewhlt ist
	movlw A'A'				;ASCII: "Auto" (0) ist gewhlt, hole "A" ins W-Register
	call disp_daten				;bertrage das Zeichen
	btfss WOB_M_A,1				;prfen ob "Burst" (1) gewhlt ist
	goto disp_vco_wf			;"Burst" (1) ist nicht gewhlt, weiter bei Anzeige Wellenform
	movlw A'B'				; "Burst" (1) ist gewhlt, ASCII: hole "B" ins W-Register	
	call disp_daten				;bertrage das Zeichen
disp_vco_wf
	movlw 0xCD				;Zeichen 14 in der 2. Zeile 
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	call disp_dds_waveform			;Aufruf des Unterprogramms zum Schreiben der Wellenform (SIN, TRI, SAW, SQR)
	return

disp_vco_freq
	movlw 0x84					;Zeichen 5 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	call disp_freq				;Aufruf des Unterprogramms zum Schreiben der Frequenz
	return

disp_wob_graph
	movff WOB_COUNT_L,AD_DA_L	;kopiere WOB_COUNT Reg. nach AD_DA Reg., der AD_DA Wert (0-1023) wird durch 64 geteilt (=> 0-15)
	movff WOB_COUNT_H,AD_DA_H
	movlw d'6'					;Die Temp Register werden 6 mal nach rechts verschoben (2^6 = 64), d.h. durch 64 geteilt 
	movwf LOOP_1
	bcf STATUS,C
	rrcf AD_DA_H
	rrcf AD_DA_L
	decfsz LOOP_1				;
	goto $-0x08					;noch nicht 6 mal geschoben, zurck zur Schleife
	movf AD_DA_L,w				;6 mal geschoben, hole Low Byte nach W und addiere die 1.Adresse in der 1.Reihe
	btfsc STATUS,Z
	goto $+0x12					;Wert ist 0, lsche letztes Zeichen in Reihe 1 und schreibe Zeichen an erster Position
	addlw 0x7F
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw 0x20					;Zeichen fr "blank"
	call disp_daten				;bertrage das Zeichen	
	movlw 0x7C					;Zeichen fr "I"
	call disp_daten				;bertrage das Zeichen	
	return
	movlw 0x8F					;Adresse fr letztes Zeichen in Reihe 1
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw 0x20					;Zeichen fr "blank"
	call disp_daten				;bertrage das Zeichen	
	movlw 0x80					;Adresse fr erstes Zeichen in Reihe 1
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw 0x7C					;Zeichen fr "I"
	call disp_daten				;bertrage das Zeichen	
	return


disp_freq_eingang
	movlw 0x01
	subwf EINGANG,w
	btfsc STATUS,Z	
	goto disp_freq_analog
	movlw A'T'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'T'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'L'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	return	
disp_freq_analog
	movlw A'A'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'N'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A' '					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	return	


disp_uhr
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'U'					;ASCII:U
	call disp_daten				;bertrage das Zeichen
	movlw A'h'					;ASCII:h
	call disp_daten				;bertrage das Zeichen
	movlw A'r'					;ASCII:r
	call disp_daten				;bertrage das Zeichen
	movlw b'00111010'			;Daten fr ":"
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;Daten fr "blank"
	call disp_daten				;bertrage das Zeichen
	movf ZAHL_A2,w				;Daten fr Stunde
	call convdisp_hex_2dig		;Unterprogramms zum Auftrennen in 10er und 1er plus Anzeige
	movlw b'00111010'			;Daten fr ":"
	call disp_daten				;bertrage das Zeichen
	movf ZAHL_A1,w				;Daten fr Minute
	call convdisp_hex_2dig		;Unterprogramms zum Auftrennen in 10er und 1er plus Anzeige
	movlw b'00111010'			;Daten fr ":"
	call disp_daten				;bertrage das Zeichen
	movf ZAHL_A0,w				;Daten fr Sekunde
	call convdisp_hex_2dig		;Unterprogramms zum Auftrennen in 10er und 1er plus Anzeige
	return

disp_set_mess
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'P'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'r'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'o'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'g'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A':'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	call disp_mess_prog
;2.Reihe
	movlw 0xC0					;Zeichen 1 in der 2. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'E'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'i'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'n'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'g'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'a'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'n'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'g'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A':'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	call disp_mess_eingang
	return

disp_mess_prog
	movlw 0x86					;Zeichen 7 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw 0x02
	subwf PROGRAM,w
	btfsc STATUS,Z	
	goto disp_mess_z
	movlw 0x01
	subwf PROGRAM,w
	btfsc STATUS,Z	
	goto disp_mess_p
	movlw A'F'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'r'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	movlw A'q'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'u'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	movlw A'n'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'z'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return	
disp_mess_z
	movlw A'Z'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'a'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	movlw A'h'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'l'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	movlw A'r'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	return	
disp_mess_p
	movlw A'P'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'r'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	movlw A'i'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'o'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'd'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	return	

disp_mess_eingang
	movlw 0xC9					;Zeichen 10 in der 2. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	btfsc EINGANG,0
	goto disp_mess_analog
	movlw A'T'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'T'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'L'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	return	
disp_mess_analog
	movlw A'A'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'n'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'a'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	movlw A'l'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'o'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'g'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	return	

disp_set_dds
	movlw 0x80					;Zeichen 1 in der 1. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	call disp_dds_waveform		;Aufruf des Unterprogramms zum Schreiben der Wellenform (SIN, TRI, SAW, SQR)
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	call disp_freq				;Unterprogramm zum Schreiben der Frequenz
	movlw A'H'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'z'					;ASCII:
	call disp_daten				;bertrage das Zeichen
;2.Reihe
	movlw 0xC0					;Zeichen 1 in der 2. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	call disp_dds_vol			;Unterprogramm zum Schreiben des Volumens
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	call disp_dds_phase			;Unterprogramm zum Schreiben der Phase
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	call disp_dds_filter		;Aufruf des Unterprogramms zum Schreiben Fi/-
	return

disp_dds_waveform
	movlw 0x03
	subwf DDS_WAVEFORM,w
	btfsc STATUS,Z	
	goto disp_dds_sqr
	movlw 0x02
	subwf DDS_WAVEFORM,w
	btfsc STATUS,Z	
	goto disp_dds_saw
	movlw 0x01
	subwf DDS_WAVEFORM,w
	btfsc STATUS,Z	
	goto disp_dds_tri
	movlw A'S'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'I'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'N'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	return
disp_dds_tri
	movlw A'T'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'R'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'I'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	return
disp_dds_saw
	movlw A'S'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'A'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'W'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	return
disp_dds_sqr
	movlw A'S'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'Q'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'R'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	return

disp_dds_vol
	movlw A'-'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movf DDS_VOL,w				;kopiere DDS Volumen ins Low Byte der Tabellenadresse
	addlw 0x22
	movwf TBLPTRL
	movlw 0x7E					;High Byte der Tabellenadresse 
	movwf TBLPTRH
	TBLRD*	 					;lese den Tabellenwert ins TABLAT Register
	movf TABLAT,W 				;Hole den Tabellenwert nach W
	call convdisp_hex_2dig		;Unterprogramms zum Auftrennen in 10er und 1er plus Anzeige
	movlw A'd'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'B'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	return

disp_dds_filter
	btfss DDS_FILTER,0
	goto disp_dds_fi_off
	movlw A'F'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'i'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return
disp_dds_fi_off
	movlw A'-'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;ASCII: Blank
	call disp_daten				;bertrage das Zeichen
	return

disp_dds_phase
	movf ZAHL_A2,w				;Daten fr Winkel 100er
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movf ZAHL_A1,w				;Daten fr Winkel 10er
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movf ZAHL_A0,w				;Daten fr Winkel 1er
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw d'223'				;Zeichen fr 
	call disp_daten				;bertrage das Zeichen
	return

disp_set_vco
								;1	2	3	4	5	6	7	8	9	10	11	12	13	14	15	16
								;V	C	O		L	O	G		F	u	0	0	0	0	H	z
								;D	3		S	1	0	2	4		M	B		1	,	0	s
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	call disp_vco_wob			;Aufruf des Unterprogramms zum schreiben von "VCO" oder "WOB"
	movlw 0x20					;ASCII: blank
	call disp_daten				;bertrage das Zeichen
	call disp_vco_kl			;Aufruf des Unterprogramms zum Schreiben der Kennlinie (LIN/LOG)
	movlw 0x20					;ASCII: blank
	call disp_daten				;bertrage das Zeichen
	movlw A'F'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'u'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movf DISP_4,w				;Daten fr 1.000er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movf DISP_3,w				;Daten fr 100er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movf DISP_2,w				;Daten fr 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movf DISP_1,w				;Daten fr 1er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw A'H'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'z'					;ASCII:
	call disp_daten				;bertrage das Zeichen
;2.Reihe
	movlw 0xC0					;Zeichen 1 in der 2. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'D'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movf DEKADE,w
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen

	btfss PROGRAM,0				;Prfen ob Programm VCO (=4 => 00000100) oder Wobbel (=5 => 00000101) gewhlt ist (Bit 0 prfen) 
	return						;VCO ist gewhlt, die folgenden Zeilen werden nicht bentigt
	movlw 0x20					;ASCII: blank
	call disp_daten				;bertrage das Zeichen
	movlw A'S'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	call disp_steps				;Aufruf des Unterprogramms zum Schreiben der Steps (0032 ... 1024)
	movlw 0x20					;ASCII: blank
	call disp_daten				;bertrage das Zeichen
	call disp_m_a				;Aufruf des Unterprogramms zum Schreiben von M/A (manuell/auto) und B (Burst)

	btfsc WOB_M_A,0				;prfen ob Bit fr "Manuell" nicht gesetzt ist
	return						;Bit fr "Manuell" ist gesetzt, Rcksprung
	movlw 0x20					;ASCII: blank
	call disp_daten				;bertrage das Zeichen
	call disp_dt
	return

disp_freq						;Unterprogramm zum Schreiben der Frequenz in den Punkten DDS, Wobbel
	movf DISP_5,w				;und Menpunkten DDS, Wobbel
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movf DISP_4,w
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movf DISP_3,w
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movf DISP_2,w
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movf DISP_1,w
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw A','					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movf DISP_0,w
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	return

disp_freq_fu
	movf DISP_4,w
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movf DISP_3,w
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movf DISP_2,w
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movf DISP_1,w
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	return

disp_vco_wob
	btfsc PROGRAM,0				;Prfen ob Programm VCO (=4 => 00000100) oder Wobbel (=5 => 00000101) gewhlt ist (Bit 0 prfen) 
	goto disp_vco_wob1			;Bit 0 ist gesetzt, d.h. Wobbel ist gewhlt	
	movlw A'V'					;Bit 0 ist nicht gesetzt, d.h. VCO ist gewhlt,	ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'C'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'O'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return
disp_vco_wob1
	movlw A'W'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'O'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'B'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return

disp_vco_kl						;Unterprogramms zum Schreiben der Kennlinie (LIN/LOG)
	movf KENNLIN,w				;hole Kennline nach W und prfe ob 0
	btfss STATUS,Z
	goto disp_vco_kl_log		;Kennlinie ist nicht 0, weiter beim Unterprogramms zum Schreiben der Kennlinie (LOG)
	goto disp_vco_kl_lin		;Kennlinie ist 0, weiter beim Unterprogramms zum Schreiben der Kennlinie (LIN)
disp_vco_kl_log
	movlw A'L'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'o'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'g'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return
disp_vco_kl_lin
	movlw A'L'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'i'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'n'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return


disp_steps
	movlw d'1'					;prfen, ob 1024 Schritte (WOB_STEP =1 ) gewhlt sind
	subwf WOB_STEP,w
	btfsc STATUS,Z
	goto write_step1			;1024 Schritte (WOB_STEP =1 ) sind gewhlt
	movlw d'2'					;prfen, ob 512 Schritte (WOB_STEP =2 ) gewhlt sind
	subwf WOB_STEP,w
	btfsc STATUS,Z
	goto write_step2			;512 Schritte (WOB_STEP =2 ) sind gewhlt
	movlw d'3'					;prfen, ob 256 Schritte (WOB_STEP =3 ) gewhlt sind
	subwf WOB_STEP,w
	btfsc STATUS,Z
	goto write_step3			;256 Schritte (WOB_STEP =3 ) sind gewhlt
	movlw d'4'					;prfen, ob 128 Schritte (WOB_STEP =4 ) gewhlt sind
	subwf WOB_STEP,w
	btfsc STATUS,Z
	goto write_step4			;128 Schritte (WOB_STEP =4 ) sind gewhlt
	movlw d'5'					;prfen, ob 64 Schritte (WOB_STEP =5 ) gewhlt sind
	subwf WOB_STEP,w
	btfsc STATUS,Z
	goto write_step5			;64 Schritte (WOB_STEP =5 ) sind gewhlt
	goto write_step6			;32 Schritte (WOB_STEP =6 ) sind gewhlt
write_step1						;1024 Schritte (WOB_STEP =1 ) sind gewhlt
	movlw A'1'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'0'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'2'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'4'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	return
write_step2						;512 Schritte (WOB_STEP =2 ) sind gewhlt
	movlw A'0'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'5'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'1'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'2'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	return
write_step3						;256 Schritte (WOB_STEP =3 ) sind gewhlt
	movlw A'0'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'2'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'5'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'6'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	return
write_step4						;128 Schritte (WOB_STEP =4 ) sind gewhlt
	movlw A'0'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'1'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'2'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'8'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	return
write_step5						;64 Schritte (WOB_STEP =5 ) sind gewhlt
	movlw A'0'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'0'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'6'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'4'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	return
write_step6						;32 Schritte (WOB_STEP =6 ) sind gewhlt
	movlw A'0'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'0'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'3'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'2'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	return

disp_m_a
	movlw A'M'					;Hole "M" fr "Manuell" nach W
	btfss WOB_M_A,0				;prfen ob Bit fr "Manuell" gesetzt ist
	movlw A'A'					;Bit ist nicht gesetzt, d.h. "Auto" ist gewhlt
	call disp_daten				;bertrage das Zeichen
	movlw A'B'					;Hole "B" fr "Burst" nach W
	btfss WOB_M_A,1				;prfen ob Bit fr "Burst" gesetzt ist
	movlw 0x20					;Bit ist nicht gesetzt, hole Zeichen fr Blank nach W
	call disp_daten				;bertrage das Zeichen
	return						;


disp_dt
	movlw d'0'					;prfen, ob 1ms (WOB_Zeit =0 ) gewhlt sind
	subwf WOB_ZEIT,w
	btfsc STATUS,Z
	goto write_zeit0			;2ms (WOB_Zeit =0 ) sind gewhlt
	movlw d'1'					;prfen, ob 2ms (WOB_Zeit =1 ) gewhlt sind
	subwf WOB_ZEIT,w
	btfsc STATUS,Z
	goto write_zeit1			;2ms (WOB_Zeit =1 ) sind gewhlt
	movlw d'2'					;prfen, ob 5ms (WOB_Zeit =2 ) gewhlt sind
	subwf WOB_ZEIT,w
	btfsc STATUS,Z
	goto write_zeit2			;5ms (WOB_Zeit =2 ) sind gewhlt
	movlw d'3'					;prfen, ob 10ms (WOB_Zeit =3 ) gewhlt sind
	subwf WOB_ZEIT,w
	btfsc STATUS,Z
	goto write_zeit3			;10ms (WOB_Zeit =3 ) sind gewhlt
	movlw d'4'					;prfen, ob 20ms (WOB_Zeit =4 ) gewhlt sind
	subwf WOB_ZEIT,w
	btfsc STATUS,Z
	goto write_zeit4			;20ms (WOB_Zeit =4 ) sind gewhlt
	movlw d'5'					;prfen, ob 50ms (WOB_Zeit =5 ) gewhlt sind
	subwf WOB_ZEIT,w
	btfsc STATUS,Z
	goto write_zeit5			;50ms (WOB_Zeit =5 ) sind gewhlt
	movlw d'6'					;prfen, ob 100ms (WOB_Zeit =6 ) gewhlt sind
	subwf WOB_ZEIT,w
	btfsc STATUS,Z
	goto write_zeit6			;100ms (WOB_Zeit =6 ) sind gewhlt
	movlw d'7'					;prfen, ob 200ms (WOB_Zeit =7 ) gewhlt sind
	subwf WOB_ZEIT,w
	btfsc STATUS,Z
	goto write_zeit7			;200ms (WOB_Zeit =7 ) sind gewhlt
	movlw d'8'					;prfen, ob 500ms (WOB_Zeit =8 ) gewhlt sind
	subwf WOB_ZEIT,w
	btfsc STATUS,Z
	goto write_zeit8			;500ms (WOB_Zeit =8 ) sind gewhlt
	movlw d'9'					;prfen, ob 1000ms (WOB_Zeit =9 ) gewhlt sind
	subwf WOB_ZEIT,w
	btfsc STATUS,Z
	goto write_zeit9			;1000ms (WOB_Zeit =9 ) sind gewhlt
	goto write_zeit10			;2000ms (WOB_Zeit =10 ) sind gewhlt
write_zeit0						;1ms (WOB_Zeit =0 ) sind gewhlt
	movlw A'1'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'm'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A's'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	return
write_zeit1						;2ms (WOB_Zeit =1 ) sind gewhlt
	movlw A'2'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'm'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A's'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	return
write_zeit2						;5ms (WOB_Zeit =2 ) sind gewhlt
	movlw A'5'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'm'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A's'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	return
write_zeit3						;10ms (WOB_Zeit =3 ) sind gewhlt
	movlw A'1'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'0'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'm'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A's'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	return
write_zeit4						;20ms (WOB_Zeit =4 ) sind gewhlt
	movlw A'2'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'0'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'm'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A's'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	return
write_zeit5						;50ms (WOB_Zeit =5 ) sind gewhlt
	movlw A'5'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'0'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'm'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A's'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	return
write_zeit6						;100ms (WOB_Zeit =6 ) sind gewhlt
	movlw A'0'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A','					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'1'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A's'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	return
write_zeit7						;200ms (WOB_Zeit =7 ) sind gewhlt
	movlw A'0'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A','					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'2'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A's'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	return
write_zeit8						;500ms (WOB_Zeit =8 ) sind gewhlt
	movlw A'0'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A','					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'5'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A's'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	return
write_zeit9						;1000ms (WOB_Zeit =9 ) sind gewhlt
	movlw A'1'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A','					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'0'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A's'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	return
write_zeit10					;2000ms (WOB_Zeit =10 ) sind gewhlt
	movlw A'2'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A','					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'0'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A's'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	return

disp_fehler1
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'F'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'u'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'/'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'D'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'k'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'z'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'u'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'h'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'o'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'c'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'h'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return

disp_fehler2
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'F'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'u'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'/'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'd'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A't'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'z'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'u'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'k'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'l'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'i'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'n'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return


disp_menu_einstell
	call clear_display		;Aufruf des Unterprogramms zum Lschen des Displays
	movlw d'1'				;
	subwf MENU_COUNT,w	;
	btfss STATUS,Z		
	goto disp_menu_einstell2
	call dispm_quhr
	goto disp_menu_einst_end
disp_menu_einstell2
	movlw d'2'				;
	subwf MENU_COUNT, w	;
	btfss STATUS,Z		
	goto disp_menu_einstell3
	call dispm_q1hz
	goto disp_menu_einst_end
disp_menu_einstell3
	movlw d'3'				;
	subwf MENU_COUNT, w	;
	btfss STATUS,Z		
	goto disp_menu_einstell4
	call dispm_dacgain
	goto disp_menu_einst_end
disp_menu_einstell4
	movlw d'4'				;
	subwf MENU_COUNT, w	;
	btfss STATUS,Z		
	goto disp_menu_einst_end
	call dispm_contrast
	goto disp_menu_einst_end
disp_menu_einst_end
	movlw d'250'				;Wartezeit von 250ms
	call wait 	
	bsf MENU_STAT,6				;setze das Status Bit
	return

dispm_quhr
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'4'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'1'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'Q'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'u'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'a'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'r'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'z'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	movlw A'('					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'U'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'h'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'r'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A')'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return

dispm_q1hz
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'4'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'2'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'Q'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'u'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'a'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'r'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'z'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	movlw A'('					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'1'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'H'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'z'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A')'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return


dispm_dacgain
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'4'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'3'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'D'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'A'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'C'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;ASCII: blank
	call disp_daten				;bertrage das Zeichen
	movlw A'G'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'a'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'i'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'n'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return

dispm_contrast
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'4'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'4'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	call disp_contrast			;Unterprogramm zum Schreiben von "Kontrast"
	return


disp_q1hz
	movlw 0x85					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'p'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'p'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'm'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x8D					;Zeichen 14 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A's'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return

disp_q1hz_vz
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movf TEMP_18,w				;Vorzeichen der Abweichung
	call disp_daten				;bertrage das Zeichen
	return

disp_q1hz_dat
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movf DISP_3,w				;Digit 4
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movf DISP_2,w				;Digit 3
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movf DISP_1,w				;Digit 2
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movf DISP_0,w				;Digit 1
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	return


disp_set_dacgain
	movlw 0x80					;Zeichen 1 in der 1. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'V'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'o'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'u'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A't'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A':'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'0'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'-'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movf DAC_GAIN,w				;prfe ob DAC_GAIN = 0, wenn ja schreibe 4 Volt, wenn nein schreibe 2 Volt
	btfss STATUS,Z
	goto $+0x0C					;DAC_GAIN >0< => schreibe 2 Volt
	movlw A'4'					;DAC_GAIN = 0 => schreibe 4 Volt
	call disp_daten				;bertrage das Zeichen
	goto $+0x08
	movlw A'2'					;DAC_GAIN >0< => schreibe 2 Volt
	call disp_daten				;bertrage das Zeichen
	movlw A'V'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x88					;Zeichen 9 in der 1. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	return

disp_set_contrast
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	call disp_contrast			;Unterprogramm zum Schreiben von "Kontrast"
	movlw A':'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	movlw 0x8A
	call disp_befehl
	movf CONTRAST,w
	call convdisp_hex_2dig		;Unterprogramms zum Auftrennen in 10er und 1er plus Anzeige
	movlw 0x8B
	call disp_befehl
	call cursor_on
	bcf TEMP_DIG_STAT,0
	movlw d'250'
	call wait
	return

disp_contrast					;Unterprogramm zum Schreiben von "Kontrast"
	movlw A'K'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'o'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'n'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A't'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'r'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'a'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A's'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A't'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return



convdisp_hex_2dig				;Unterprogramms zum Auftrennen in 10er und 1er plus Anzeige
	call convert_dig			;Aufruf des Unterprogramms zum Auftrennen in 100er, 10er und 1er
	movf TEMP_14,w				;Daten 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movf TEMP_15,w				;Daten 1ser, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	return


convert_dig					;Unterprogramm zur Umwandlung von zB Zahl 123 in 1,2,3
	movwf TEMP_15			;Temp Register fr Zwischenspeicherung des zu zerlegenden Wertes
	clrf TEMP_13			;lsche Temp Register
	clrf TEMP_14			;lsche Temp Register
dig_100er
	movlw d'100'			;schreibe Wert 100 ins W-Register
	subwf TEMP_15,w			;reduziere den zu zerlegenden Werte um 100 im W-Register (zum Testen)
	btfss STATUS,C			;prfen,ob Ergebnis negativ
	goto dig_10er			;Ergebnis negativ, weiter bei 10er
	incf TEMP_13			;Ergebnis noch positiv, Erhhe 100er um 1 
	movlw d'100'			; und reduziere den zu zerlegenden Werte um 100 im F-Register
	subwf TEMP_15,f			;
	goto dig_100er			;Zurck zur Schleife
dig_10er
	movlw d'10'				;schreibe Wert 10 ins W-Register
	subwf TEMP_15,w			;reduziere den zu zerlegenden Werte um 10 im W-Register (zum Testen)
	btfss STATUS,C			;prfen,ob Ergebnis negativ
	goto dig_1er			;Ergebnis negativ, weiter bei 1er
	incf TEMP_14			;Ergebnis noch positiv, Erhhe 10er um 1 
	movlw d'10'				; und reduziere den zu zerlegenden Werte um 10 im F-Register
	subwf TEMP_15,f			;
	goto dig_10er			;Zurck zur Schleife
dig_1er
	movf TEMP_15,w
	return


init_display					;Unterprogramm zur Initialisierung des Displays
	movlw d'250'				;geforderte Wartezeit des Displays von 200ms
	call wait 		
	movlw 0x38					;Befehl: Function Set
	call disp_befehl			;
	movlw d'1'					;geforderte Wartezeit des Displays von 26,3us
	call wait 		
	movlw 0x39					;Befehl: Function Set, (muss zweimal bertragen werden)
	call disp_befehl			;
	movlw d'1'					;geforderte Wartezeit des Displays von 26,3us
	call wait 		
	movlw 0x14					;Befehl: Bias Set
	call disp_befehl			;
	movlw d'1'					;geforderte Wartezeit des Displays von 26,3us
	call wait 	

	movff CONTRAST,TEMP_2		;kopiere Kontrastwert in Temp Register
	bcf TEMP_2,4				;lsche Bits 4+5, da hier nur Bits 0-3 bentigt werden
	bcf TEMP_2,5	
	movf TEMP_2,w				;addiere Bits 0-3 und 0x70
	addlw 0x70					;Befehl: Contrast Set
	call disp_befehl			;
	movlw d'1'					;geforderte Wartezeit des Displays von 26,3us
	call wait 		

	clrf TEMP_2				;kopiere Bits 4+5 von Contrast Register nach Bits 0+1 des Temp Register
	btfsc CONTRAST,4
	bsf TEMP_2,0
	btfsc CONTRAST,5
	bsf TEMP_2,1
	movf TEMP_2,w				;addiere Bits 0-1 und 0x50
	addlw 0x50					;Befehl: Power/Icon/Contrast Contr.
	call disp_befehl			;
	movlw d'1'					;geforderte Wartezeit des Displays von 26,3us
	call wait 		

	movlw 0x69					;Befehl: Follower Control
	call disp_befehl			;
	movlw d'1'					;geforderte Wartezeit des Displays von 26,3us
	call wait 		
	movlw 0x0C					;Befehl: Display On/Off Control (00001100)
	call disp_befehl			;
	movlw d'1'					;geforderte Wartezeit des Displays von 26,3us
	call wait 		
	call clear_display			;Aufruf des Unterprogramms zum Lschen des Displays
	movlw 0x06					;Befehl: Entry Mode Set (00000110)
	call disp_befehl			;
	return

clear_display					;Unterprogramm zum lschen des Displays					
	movlw 0x01					;Befehl: Clear Display
	call disp_befehl	;
	movlw d'2'					;geforderte Wartezeit des Displays von 1,08ms
	call wait 		
	return

cursor_off
	movlw 0x0C					;Befehl: Display On/Off Control, Cursor OFF (00001100)
	call disp_befehl			;
	movlw d'20'					;Wartezeit min 27us
	movwf LOOP_1				;
	decfsz LOOP_1				;
	goto $-0x02
	return

cursor_on
	movlw 0x0E					;Befehl: Display On/Off Control, Cursor ON (00001110)
	call disp_befehl			;
	movlw d'20'					;geforderte Wartezeit min 27us
	movwf LOOP_1				;
	decfsz LOOP_1				;
	goto $-0x02
	return



disp_befehl
	movwf SHIFT_REG				;Register mit den Daten, die ausgegeben werden sollen
	bcf DISP_EN					;Enable auf Low setzen
	nop
	nop
	bcf DISP_RS					;RS auf Low setzen, da es sich um ein Kommando handelt
	call disp_routine			;Aufruf des Unterprogramms zum Schreiben der Display Adresse / Daten
	return

disp_daten
	movwf SHIFT_REG				;Das W-Register wird in das Reg. SHIFT_REG kopiert
	bcf DISP_EN					;Enable auf Low setzen
	nop
	nop
	bsf DISP_RS					;RS auf High setzen, da es sich um ein Zeichen handelt
	call disp_routine			;Aufruf des Unterprogramms zum Schreiben der Display Adresse / Daten
	return

disp_routine
	bcf SI_CLK					;Takt auf Low setzen, damit die Daten bei der steigenden
									;Flanke bernommen werden knnen
	movlw d'8'					;es sollen 8 Bits bertragen werden
	movwf SHIFT_CNT				;Zhler fr die bertragenen bits
shiftDaten
	rlcf SHIFT_REG,F				;Das Zeichen wird nach links verschoben. Das Carryflag wird
									;an den Ausgang weitergegeben
	btfsc STATUS,C				;Prfen, welchen Wert das Carryflag hat
	bsf SI_DAT					;Carryflag = 1 -> Pin SI_DAT auf Highpegel setzen
	btfss STATUS,C				;Nochmal prfen, da immer nur ein Befehl bersprungen wird
	bcf SI_DAT					;Carryflag = 0 -> Pin SI_DAT auf Lowpegel setzen
	nop							;kurze Wartezeit, um die bertragungsgeschwindigkeit zu reduzieren
	nop
	nop
	nop
	bsf SI_CLK					;mit dieser steigenden Flanke werden die Daten bernommen
	nop							;kurze Wartezeit
	nop
	nop
	nop
	bcf SI_CLK					;negative Flanke des Taktes
	decfsz SHIFT_CNT,F			;ein bit wurde bertragen -> Zhler um 1 verringern
	goto shiftDaten				;nchstes bit bertragen
	bsf DISP_EN					;Enable auf High setzen, mit der positiven Flanke werden Daten bertragen 
	nop
	bcf DISP_RS					;bertragung beendet Pin RS auf Low zurcksetzen
	bcf SI_DAT					;Pin SI_DAT auf Low zurcksetzen
	movlw d'150'				;geforderte Wartezeit (min. 26,3 us)
	movwf LOOP_1
	decfsz LOOP_1
	goto $-0x02
	return						;Rcksprung aus dem Unterprogramm

wait
	movwf LOOP_1			;
	movlw d'14'
	movwf LOOP_3
wait_loop
	movlw d'250'
	movwf LOOP_2
loop_48						;bei 48MHz muss loop_48 48 Takte lang sein
	nop
	decfsz LOOP_3
	goto $-0x02
	movlw d'14'
	movwf LOOP_3
	decfsz LOOP_2
	goto loop_48
	decfsz LOOP_1
	goto wait_loop
	return



;*******************************************************************************************
;Tabellen:
;	ORG 0x78FE		;Logarithmentabelle (0=6.554, 255=64.949) Low Byte
;	ORG 0x7B00		;Cursor Tabelle DDS, VCO,Wobbel,
;	ORG 0x7BFE		;Logarithmentabelle (0=6.554, 255=64.949) High Byte
;	ORG 0x7E00		;Volumen Tabelle fr Code Digitalpotentiometer
;	ORG 0x7E22		;Volumen Tabelle fr Anzeige in dB
;	ORG 0x7E44		;Tabellen fr deltaT Timer1 und deltaT/2 Timer0
;	ORG 0x7F00		;Sinustabelle
;*******************************************************************************************


	ORG 0x78FE

log_tab_L				;Logarithmentabelle (1.000-10.000) Low Byte
	addwf PCL,f
	retlw 0x9A		;Datenpunkt 0
	retlw 0xD5		;Datenpunkt 1
	retlw 0x11		;Datenpunkt 2
	retlw 0x4D		;Datenpunkt 3
	retlw 0x8A		;Datenpunkt 4
	retlw 0xC7		;Datenpunkt 5
	retlw 0x05		;Datenpunkt 6
	retlw 0x43		;Datenpunkt 7
	retlw 0x83		;Datenpunkt 8
	retlw 0xC2		;Datenpunkt 9
	retlw 0x02		;Datenpunkt 10
	retlw 0x43		;Datenpunkt 11
	retlw 0x85		;Datenpunkt 12
	retlw 0xC6		;Datenpunkt 13
	retlw 0x09		;Datenpunkt 14
	retlw 0x4C		;Datenpunkt 15
	retlw 0x90		;Datenpunkt 16
	retlw 0xD4		;Datenpunkt 17
	retlw 0x19		;Datenpunkt 18
	retlw 0x5F		;Datenpunkt 19
	retlw 0xA5		;Datenpunkt 20
	retlw 0xEC		;Datenpunkt 21
	retlw 0x34		;Datenpunkt 22
	retlw 0x7C		;Datenpunkt 23
	retlw 0xC5		;Datenpunkt 24
	retlw 0x0E		;Datenpunkt 25
	retlw 0x58		;Datenpunkt 26
	retlw 0xA3		;Datenpunkt 27
	retlw 0xEF		;Datenpunkt 28
	retlw 0x3B		;Datenpunkt 29
	retlw 0x88		;Datenpunkt 30
	retlw 0xD5		;Datenpunkt 31
	retlw 0x23		;Datenpunkt 32
	retlw 0x72		;Datenpunkt 33
	retlw 0xC2		;Datenpunkt 34
	retlw 0x12		;Datenpunkt 35
	retlw 0x64		;Datenpunkt 36
	retlw 0xB5		;Datenpunkt 37
	retlw 0x08		;Datenpunkt 38
	retlw 0x5B		;Datenpunkt 39
	retlw 0xAF		;Datenpunkt 40
	retlw 0x04		;Datenpunkt 41
	retlw 0x5A		;Datenpunkt 42
	retlw 0xB0		;Datenpunkt 43
	retlw 0x07		;Datenpunkt 44
	retlw 0x5F		;Datenpunkt 45
	retlw 0xB8		;Datenpunkt 46
	retlw 0x12		;Datenpunkt 47
	retlw 0x6C		;Datenpunkt 48
	retlw 0xC7		;Datenpunkt 49
	retlw 0x23		;Datenpunkt 50
	retlw 0x80		;Datenpunkt 51
	retlw 0xDE		;Datenpunkt 52
	retlw 0x3C		;Datenpunkt 53
	retlw 0x9C		;Datenpunkt 54
	retlw 0xFC		;Datenpunkt 55
	retlw 0x5D		;Datenpunkt 56
	retlw 0xBF		;Datenpunkt 57
	retlw 0x22		;Datenpunkt 58
	retlw 0x86		;Datenpunkt 59
	retlw 0xEA		;Datenpunkt 60
	retlw 0x50		;Datenpunkt 61
	retlw 0xB6		;Datenpunkt 62
	retlw 0x1E		;Datenpunkt 63
	retlw 0x86		;Datenpunkt 64
	retlw 0xEF		;Datenpunkt 65
	retlw 0x5A		;Datenpunkt 66
	retlw 0xC5		;Datenpunkt 67
	retlw 0x31		;Datenpunkt 68
	retlw 0x9E		;Datenpunkt 69
	retlw 0x0C		;Datenpunkt 70
	retlw 0x7B		;Datenpunkt 71
	retlw 0xEC		;Datenpunkt 72
	retlw 0x5D		;Datenpunkt 73
	retlw 0xCF		;Datenpunkt 74
	retlw 0x42		;Datenpunkt 75
	retlw 0xB6		;Datenpunkt 76
	retlw 0x2C		;Datenpunkt 77
	retlw 0xA2		;Datenpunkt 78
	retlw 0x19		;Datenpunkt 79
	retlw 0x92		;Datenpunkt 80
	retlw 0x0C		;Datenpunkt 81
	retlw 0x86		;Datenpunkt 82
	retlw 0x02		;Datenpunkt 83
	retlw 0x7F		;Datenpunkt 84
	retlw 0xFD		;Datenpunkt 85
	retlw 0x7C		;Datenpunkt 86
	retlw 0xFD		;Datenpunkt 87
	retlw 0x7E		;Datenpunkt 88
	retlw 0x01		;Datenpunkt 89
	retlw 0x85		;Datenpunkt 90
	retlw 0x0A		;Datenpunkt 91
	retlw 0x90		;Datenpunkt 92
	retlw 0x17		;Datenpunkt 93
	retlw 0xA0		;Datenpunkt 94
	retlw 0x2A		;Datenpunkt 95
	retlw 0xB5		;Datenpunkt 96
	retlw 0x41		;Datenpunkt 97
	retlw 0xCF		;Datenpunkt 98
	retlw 0x5E		;Datenpunkt 99
	retlw 0xEE		;Datenpunkt 100
	retlw 0x80		;Datenpunkt 101
	retlw 0x13		;Datenpunkt 102
	retlw 0xA7		;Datenpunkt 103
	retlw 0x3D		;Datenpunkt 104
	retlw 0xD3		;Datenpunkt 105
	retlw 0x6C		;Datenpunkt 106
	retlw 0x05		;Datenpunkt 107
	retlw 0xA0		;Datenpunkt 108
	retlw 0x3D		;Datenpunkt 109
	retlw 0xDB		;Datenpunkt 110
	retlw 0x7A		;Datenpunkt 111
	retlw 0x1B		;Datenpunkt 112
	retlw 0xBD		;Datenpunkt 113
	retlw 0x60		;Datenpunkt 114
	retlw 0x05		;Datenpunkt 115
	retlw 0xAC		;Datenpunkt 116
	retlw 0x54		;Datenpunkt 117
	retlw 0xFE		;Datenpunkt 118
	retlw 0xA9		;Datenpunkt 119
	retlw 0x55		;Datenpunkt 120
	retlw 0x04		;Datenpunkt 121
	retlw 0xB4		;Datenpunkt 122
	retlw 0x65		;Datenpunkt 123
	retlw 0x18		;Datenpunkt 124
	retlw 0xCD		;Datenpunkt 125
	retlw 0x83		;Datenpunkt 126
	retlw 0x3B		;Datenpunkt 127
	retlw 0xF4		;Datenpunkt 128
	retlw 0xB0		;Datenpunkt 129
	retlw 0x6C		;Datenpunkt 130
	retlw 0x2B		;Datenpunkt 131
	retlw 0xEB		;Datenpunkt 132
	retlw 0xAE		;Datenpunkt 133
	retlw 0x71		;Datenpunkt 134
	retlw 0x37		;Datenpunkt 135
	retlw 0xFE		;Datenpunkt 136
	retlw 0xC8		;Datenpunkt 137
	retlw 0x93		;Datenpunkt 138
	retlw 0x60		;Datenpunkt 139
	retlw 0x2E		;Datenpunkt 140
	retlw 0xFF		;Datenpunkt 141
	retlw 0xD1		;Datenpunkt 142
	retlw 0xA6		;Datenpunkt 143
	retlw 0x7C		;Datenpunkt 144
	retlw 0x54		;Datenpunkt 145
	retlw 0x2E		;Datenpunkt 146
	retlw 0x0B		;Datenpunkt 147
	retlw 0xE9		;Datenpunkt 148
	retlw 0xC9		;Datenpunkt 149
	retlw 0xAB		;Datenpunkt 150
	retlw 0x8F		;Datenpunkt 151
	retlw 0x76		;Datenpunkt 152
	retlw 0x5E		;Datenpunkt 153
	retlw 0x48		;Datenpunkt 154
	retlw 0x35		;Datenpunkt 155
	retlw 0x24		;Datenpunkt 156
	retlw 0x15		;Datenpunkt 157
	retlw 0x08		;Datenpunkt 158
	retlw 0xFD		;Datenpunkt 159
	retlw 0xF4		;Datenpunkt 160
	retlw 0xEE		;Datenpunkt 161
	retlw 0xEA		;Datenpunkt 162
	retlw 0xE8		;Datenpunkt 163
	retlw 0xE9		;Datenpunkt 164
	retlw 0xEC		;Datenpunkt 165
	retlw 0xF1		;Datenpunkt 166
	retlw 0xF8		;Datenpunkt 167
	retlw 0x02		;Datenpunkt 168
	retlw 0x0F		;Datenpunkt 169
	retlw 0x1D		;Datenpunkt 170
	retlw 0x2E		;Datenpunkt 171
	retlw 0x42		;Datenpunkt 172
	retlw 0x58		;Datenpunkt 173
	retlw 0x71		;Datenpunkt 174
	retlw 0x8C		;Datenpunkt 175
	retlw 0xAA		;Datenpunkt 176
	retlw 0xCA		;Datenpunkt 177
	retlw 0xED		;Datenpunkt 178
	retlw 0x13		;Datenpunkt 179
	retlw 0x3B		;Datenpunkt 180
	retlw 0x66		;Datenpunkt 181
	retlw 0x94		;Datenpunkt 182
	retlw 0xC4		;Datenpunkt 183
	retlw 0xF7		;Datenpunkt 184
	retlw 0x2D		;Datenpunkt 185
	retlw 0x65		;Datenpunkt 186
	retlw 0xA1		;Datenpunkt 187
	retlw 0xDF		;Datenpunkt 188
	retlw 0x20		;Datenpunkt 189
	retlw 0x65		;Datenpunkt 190
	retlw 0xAC		;Datenpunkt 191
	retlw 0xF6		;Datenpunkt 192
	retlw 0x43		;Datenpunkt 193
	retlw 0x93		;Datenpunkt 194
	retlw 0xE6		;Datenpunkt 195
	retlw 0x3C		;Datenpunkt 196
	retlw 0x95		;Datenpunkt 197
	retlw 0xF1		;Datenpunkt 198
	retlw 0x51		;Datenpunkt 199
	retlw 0xB3		;Datenpunkt 200
	retlw 0x19		;Datenpunkt 201
	retlw 0x82		;Datenpunkt 202
	retlw 0xEE		;Datenpunkt 203
	retlw 0x5E		;Datenpunkt 204
	retlw 0xD1		;Datenpunkt 205
	retlw 0x47		;Datenpunkt 206
	retlw 0xC1		;Datenpunkt 207
	retlw 0x3E		;Datenpunkt 208
	retlw 0xBE		;Datenpunkt 209
	retlw 0x42		;Datenpunkt 210
	retlw 0xCA		;Datenpunkt 211
	retlw 0x55		;Datenpunkt 212
	retlw 0xE3		;Datenpunkt 213
	retlw 0x76		;Datenpunkt 214
	retlw 0x0C		;Datenpunkt 215
	retlw 0xA5		;Datenpunkt 216
	retlw 0x42		;Datenpunkt 217
	retlw 0xE3		;Datenpunkt 218
	retlw 0x88		;Datenpunkt 219
	retlw 0x30		;Datenpunkt 220
	retlw 0xDD		;Datenpunkt 221
	retlw 0x8D		;Datenpunkt 222
	retlw 0x41		;Datenpunkt 223
	retlw 0xF9		;Datenpunkt 224
	retlw 0xB5		;Datenpunkt 225
	retlw 0x75		;Datenpunkt 226
	retlw 0x39		;Datenpunkt 227
	retlw 0x01		;Datenpunkt 228
	retlw 0xCE		;Datenpunkt 229
	retlw 0x9E		;Datenpunkt 230
	retlw 0x73		;Datenpunkt 231
	retlw 0x4C		;Datenpunkt 232
	retlw 0x29		;Datenpunkt 233
	retlw 0x0A		;Datenpunkt 234
	retlw 0xF0		;Datenpunkt 235
	retlw 0xDA		;Datenpunkt 236
	retlw 0xC9		;Datenpunkt 237
	retlw 0xBC		;Datenpunkt 238
	retlw 0xB4		;Datenpunkt 239
	retlw 0xB0		;Datenpunkt 240
	retlw 0xB1		;Datenpunkt 241
	retlw 0xB6		;Datenpunkt 242
	retlw 0xC0		;Datenpunkt 243
	retlw 0xCF		;Datenpunkt 244
	retlw 0xE2		;Datenpunkt 245
	retlw 0xFB		;Datenpunkt 246
	retlw 0x18		;Datenpunkt 247
	retlw 0x3A		;Datenpunkt 248
	retlw 0x61		;Datenpunkt 249
	retlw 0x8D		;Datenpunkt 250
	retlw 0xBE		;Datenpunkt 251
	retlw 0xF4		;Datenpunkt 252
	retlw 0x2F		;Datenpunkt 253
	retlw 0x70		;Datenpunkt 254
	retlw 0xB5		;Datenpunkt 255


	ORG 0x7B00		;Cursor Tabelle DDS, VCO,Wobbel,

	de 0x80,0x84	;DDS: Cursor Position 1,2
	de 0x85,0x87	;DDS: Cursor Position 3,4
	de 0x88,0x89	;DDS: Cursor Position 5,6
	de 0x8B,0xC2	;DDS: Cursor Position 7,8
	de 0xC6,0xC7	;DDS: Cursor Position 9,10
	de 0xC8,0xCB	;DDS: Cursor Position 11,12
	de 0x80,0x84	;VCO,Wobbel: Cursor Position 1,2
	de 0x8A,0x8B	;VCO,Wobbel: Cursor Position 3,4
	de 0x8C,0x8D	;VCO,Wobbel: Cursor Position 5,6
	de 0xC1,0xC7	;VCO,Wobbel: Cursor Position 7,8
	de 0xC9,0xCC	;VCO,Wobbel: Cursor Position 9,10


	ORG 0x7BFE

log_tab_H				;Logarithmentabelle (1.000-10.000) High Byte
	addwf PCL,f
	retlw 0x19		;Datenpunkt 0	
	retlw 0x19		;Datenpunkt 1	
	retlw 0x1A		;Datenpunkt 2	
	retlw 0x1A		;Datenpunkt 3	
	retlw 0x1A		;Datenpunkt 4	
	retlw 0x1A		;Datenpunkt 5	
	retlw 0x1B		;Datenpunkt 6	
	retlw 0x1B		;Datenpunkt 7	
	retlw 0x1B		;Datenpunkt 8	
	retlw 0x1B		;Datenpunkt 9	
	retlw 0x1C		;Datenpunkt 10	
	retlw 0x1C		;Datenpunkt 11	
	retlw 0x1C		;Datenpunkt 12	
	retlw 0x1C		;Datenpunkt 13	
	retlw 0x1D		;Datenpunkt 14	
	retlw 0x1D		;Datenpunkt 15	
	retlw 0x1D		;Datenpunkt 16	
	retlw 0x1D		;Datenpunkt 17	
	retlw 0x1E		;Datenpunkt 18	
	retlw 0x1E		;Datenpunkt 19	
	retlw 0x1E		;Datenpunkt 20	
	retlw 0x1E		;Datenpunkt 21	
	retlw 0x1F		;Datenpunkt 22	
	retlw 0x1F		;Datenpunkt 23	
	retlw 0x1F		;Datenpunkt 24	
	retlw 0x20		;Datenpunkt 25	
	retlw 0x20		;Datenpunkt 26	
	retlw 0x20		;Datenpunkt 27	
	retlw 0x20		;Datenpunkt 28	
	retlw 0x21		;Datenpunkt 29	
	retlw 0x21		;Datenpunkt 30	
	retlw 0x21		;Datenpunkt 31	
	retlw 0x22		;Datenpunkt 32	
	retlw 0x22		;Datenpunkt 33	
	retlw 0x22		;Datenpunkt 34	
	retlw 0x23		;Datenpunkt 35	
	retlw 0x23		;Datenpunkt 36	
	retlw 0x23		;Datenpunkt 37	
	retlw 0x24		;Datenpunkt 38	
	retlw 0x24		;Datenpunkt 39	
	retlw 0x24		;Datenpunkt 40	
	retlw 0x25		;Datenpunkt 41	
	retlw 0x25		;Datenpunkt 42	
	retlw 0x25		;Datenpunkt 43	
	retlw 0x26		;Datenpunkt 44	
	retlw 0x26		;Datenpunkt 45	
	retlw 0x26		;Datenpunkt 46	
	retlw 0x27		;Datenpunkt 47	
	retlw 0x27		;Datenpunkt 48	
	retlw 0x27		;Datenpunkt 49	
	retlw 0x28		;Datenpunkt 50	
	retlw 0x28		;Datenpunkt 51	
	retlw 0x28		;Datenpunkt 52	
	retlw 0x29		;Datenpunkt 53	
	retlw 0x29		;Datenpunkt 54	
	retlw 0x29		;Datenpunkt 55	
	retlw 0x2A		;Datenpunkt 56	
	retlw 0x2A		;Datenpunkt 57	
	retlw 0x2B		;Datenpunkt 58	
	retlw 0x2B		;Datenpunkt 59	
	retlw 0x2B		;Datenpunkt 60	
	retlw 0x2C		;Datenpunkt 61	
	retlw 0x2C		;Datenpunkt 62	
	retlw 0x2D		;Datenpunkt 63	
	retlw 0x2D		;Datenpunkt 64	
	retlw 0x2D		;Datenpunkt 65	
	retlw 0x2E		;Datenpunkt 66	
	retlw 0x2E		;Datenpunkt 67	
	retlw 0x2F		;Datenpunkt 68	
	retlw 0x2F		;Datenpunkt 69	
	retlw 0x30		;Datenpunkt 70	
	retlw 0x30		;Datenpunkt 71	
	retlw 0x30		;Datenpunkt 72	
	retlw 0x31		;Datenpunkt 73	
	retlw 0x31		;Datenpunkt 74	
	retlw 0x32		;Datenpunkt 75	
	retlw 0x32		;Datenpunkt 76	
	retlw 0x33		;Datenpunkt 77	
	retlw 0x33		;Datenpunkt 78	
	retlw 0x34		;Datenpunkt 79	
	retlw 0x34		;Datenpunkt 80	
	retlw 0x35		;Datenpunkt 81	
	retlw 0x35		;Datenpunkt 82	
	retlw 0x36		;Datenpunkt 83	
	retlw 0x36		;Datenpunkt 84	
	retlw 0x36		;Datenpunkt 85	
	retlw 0x37		;Datenpunkt 86	
	retlw 0x37		;Datenpunkt 87	
	retlw 0x38		;Datenpunkt 88	
	retlw 0x39		;Datenpunkt 89	
	retlw 0x39		;Datenpunkt 90	
	retlw 0x3A		;Datenpunkt 91	
	retlw 0x3A		;Datenpunkt 92	
	retlw 0x3B		;Datenpunkt 93	
	retlw 0x3B		;Datenpunkt 94	
	retlw 0x3C		;Datenpunkt 95	
	retlw 0x3C		;Datenpunkt 96	
	retlw 0x3D		;Datenpunkt 97	
	retlw 0x3D		;Datenpunkt 98	
	retlw 0x3E		;Datenpunkt 99	
	retlw 0x3E		;Datenpunkt 100	
	retlw 0x3F		;Datenpunkt 101	
	retlw 0x40		;Datenpunkt 102	
	retlw 0x40		;Datenpunkt 103	
	retlw 0x41		;Datenpunkt 104	
	retlw 0x41		;Datenpunkt 105	
	retlw 0x42		;Datenpunkt 106	
	retlw 0x43		;Datenpunkt 107	
	retlw 0x43		;Datenpunkt 108	
	retlw 0x44		;Datenpunkt 109	
	retlw 0x44		;Datenpunkt 110	
	retlw 0x45		;Datenpunkt 111	
	retlw 0x46		;Datenpunkt 112	
	retlw 0x46		;Datenpunkt 113	
	retlw 0x47		;Datenpunkt 114	
	retlw 0x48		;Datenpunkt 115	
	retlw 0x48		;Datenpunkt 116	
	retlw 0x49		;Datenpunkt 117	
	retlw 0x49		;Datenpunkt 118	
	retlw 0x4A		;Datenpunkt 119	
	retlw 0x4B		;Datenpunkt 120	
	retlw 0x4C		;Datenpunkt 121	
	retlw 0x4C		;Datenpunkt 122	
	retlw 0x4D		;Datenpunkt 123	
	retlw 0x4E		;Datenpunkt 124	
	retlw 0x4E		;Datenpunkt 125	
	retlw 0x4F		;Datenpunkt 126	
	retlw 0x50		;Datenpunkt 127	
	retlw 0x50		;Datenpunkt 128	
	retlw 0x51		;Datenpunkt 129	
	retlw 0x52		;Datenpunkt 130	
	retlw 0x53		;Datenpunkt 131	
	retlw 0x53		;Datenpunkt 132	
	retlw 0x54		;Datenpunkt 133	
	retlw 0x55		;Datenpunkt 134	
	retlw 0x56		;Datenpunkt 135	
	retlw 0x56		;Datenpunkt 136	
	retlw 0x57		;Datenpunkt 137	
	retlw 0x58		;Datenpunkt 138	
	retlw 0x59		;Datenpunkt 139	
	retlw 0x5A		;Datenpunkt 140	
	retlw 0x5A		;Datenpunkt 141	
	retlw 0x5B		;Datenpunkt 142	
	retlw 0x5C		;Datenpunkt 143	
	retlw 0x5D		;Datenpunkt 144	
	retlw 0x5E		;Datenpunkt 145	
	retlw 0x5F		;Datenpunkt 146	
	retlw 0x60		;Datenpunkt 147	
	retlw 0x60		;Datenpunkt 148	
	retlw 0x61		;Datenpunkt 149	
	retlw 0x62		;Datenpunkt 150	
	retlw 0x63		;Datenpunkt 151	
	retlw 0x64		;Datenpunkt 152	
	retlw 0x65		;Datenpunkt 153	
	retlw 0x66		;Datenpunkt 154	
	retlw 0x67		;Datenpunkt 155	
	retlw 0x68		;Datenpunkt 156	
	retlw 0x69		;Datenpunkt 157	
	retlw 0x6A		;Datenpunkt 158	
	retlw 0x6A		;Datenpunkt 159	
	retlw 0x6B		;Datenpunkt 160	
	retlw 0x6C		;Datenpunkt 161	
	retlw 0x6D		;Datenpunkt 162	
	retlw 0x6E		;Datenpunkt 163	
	retlw 0x6F		;Datenpunkt 164	
	retlw 0x70		;Datenpunkt 165	
	retlw 0x71		;Datenpunkt 166	
	retlw 0x72		;Datenpunkt 167	
	retlw 0x74		;Datenpunkt 168	
	retlw 0x75		;Datenpunkt 169	
	retlw 0x76		;Datenpunkt 170	
	retlw 0x77		;Datenpunkt 171	
	retlw 0x78		;Datenpunkt 172	
	retlw 0x79		;Datenpunkt 173	
	retlw 0x7A		;Datenpunkt 174	
	retlw 0x7B		;Datenpunkt 175	
	retlw 0x7C		;Datenpunkt 176	
	retlw 0x7D		;Datenpunkt 177	
	retlw 0x7E		;Datenpunkt 178	
	retlw 0x80		;Datenpunkt 179	
	retlw 0x81		;Datenpunkt 180	
	retlw 0x82		;Datenpunkt 181	
	retlw 0x83		;Datenpunkt 182	
	retlw 0x84		;Datenpunkt 183	
	retlw 0x85		;Datenpunkt 184	
	retlw 0x87		;Datenpunkt 185	
	retlw 0x88		;Datenpunkt 186	
	retlw 0x89		;Datenpunkt 187	
	retlw 0x8A		;Datenpunkt 188	
	retlw 0x8C		;Datenpunkt 189	
	retlw 0x8D		;Datenpunkt 190	
	retlw 0x8E		;Datenpunkt 191	
	retlw 0x8F		;Datenpunkt 192	
	retlw 0x91		;Datenpunkt 193	
	retlw 0x92		;Datenpunkt 194	
	retlw 0x93		;Datenpunkt 195	
	retlw 0x95		;Datenpunkt 196	
	retlw 0x96		;Datenpunkt 197	
	retlw 0x97		;Datenpunkt 198	
	retlw 0x99		;Datenpunkt 199	
	retlw 0x9A		;Datenpunkt 200	
	retlw 0x9C		;Datenpunkt 201	
	retlw 0x9D		;Datenpunkt 202	
	retlw 0x9E		;Datenpunkt 203	
	retlw 0xA0		;Datenpunkt 204	
	retlw 0xA1		;Datenpunkt 205	
	retlw 0xA3		;Datenpunkt 206	
	retlw 0xA4		;Datenpunkt 207	
	retlw 0xA6		;Datenpunkt 208	
	retlw 0xA7		;Datenpunkt 209	
	retlw 0xA9		;Datenpunkt 210	
	retlw 0xAA		;Datenpunkt 211	
	retlw 0xAC		;Datenpunkt 212	
	retlw 0xAD		;Datenpunkt 213	
	retlw 0xAF		;Datenpunkt 214	
	retlw 0xB1		;Datenpunkt 215	
	retlw 0xB2		;Datenpunkt 216	
	retlw 0xB4		;Datenpunkt 217	
	retlw 0xB5		;Datenpunkt 218	
	retlw 0xB7		;Datenpunkt 219	
	retlw 0xB9		;Datenpunkt 220	
	retlw 0xBA		;Datenpunkt 221	
	retlw 0xBC		;Datenpunkt 222	
	retlw 0xBE		;Datenpunkt 223	
	retlw 0xBF		;Datenpunkt 224	
	retlw 0xC1		;Datenpunkt 225	
	retlw 0xC3		;Datenpunkt 226	
	retlw 0xC5		;Datenpunkt 227	
	retlw 0xC7		;Datenpunkt 228	
	retlw 0xC8		;Datenpunkt 229	
	retlw 0xCA		;Datenpunkt 230	
	retlw 0xCC		;Datenpunkt 231	
	retlw 0xCE		;Datenpunkt 232	
	retlw 0xD0		;Datenpunkt 233	
	retlw 0xD2		;Datenpunkt 234	
	retlw 0xD3		;Datenpunkt 235	
	retlw 0xD5		;Datenpunkt 236	
	retlw 0xD7		;Datenpunkt 237	
	retlw 0xD9		;Datenpunkt 238	
	retlw 0xDB		;Datenpunkt 239	
	retlw 0xDD		;Datenpunkt 240	
	retlw 0xDF		;Datenpunkt 241	
	retlw 0xE1		;Datenpunkt 242	
	retlw 0xE3		;Datenpunkt 243	
	retlw 0xE5		;Datenpunkt 244	
	retlw 0xE7		;Datenpunkt 245	
	retlw 0xE9		;Datenpunkt 246	
	retlw 0xEC		;Datenpunkt 247	
	retlw 0xEE		;Datenpunkt 248	
	retlw 0xF0		;Datenpunkt 249	
	retlw 0xF2		;Datenpunkt 250	
	retlw 0xF4		;Datenpunkt 251	
	retlw 0xF6		;Datenpunkt 252	
	retlw 0xF9		;Datenpunkt 253	
	retlw 0xFB		;Datenpunkt 254	
	retlw 0xFD		;Datenpunkt 255	

	ORG 0x7E00			;Volumen Tabelle fr Code Digitalpotentiometer
						
	de 0xFF,0xE3		;DP0: Code 255, 0dB		DP1: Code 227, -1dB
	de 0xCA,0xB4		;DP2: Code 202, -2dB	DP3: Code 180, -3dB
	de 0xA0,0x8F		;DP4: Code 160, -4dB	DP5: Code 143, -5dB
	de 0x7F,0x71		;DP6: Code 127, -6dB	DP7: Code 113, -7dB
	de 0x65,0x5A		;DP8: Code 101, -8dB	DP9: Code 90, -9dB
	de 0x50,0x47		;DP10: Code 80, -10dB	DP11: Code 71, -11dB
	de 0x3F,0x38		;DP12: Code 63, -12dB	DP13: Code 56, -13dB
	de 0x32,0x2D		;DP14: Code 50, -14dB	DP15: Code 45, -15dB
	de 0x28,0x23		;DP16: Code 40, -16dB	DP17: Code 35, -17dB
	de 0x1F,0x1C		;DP18: Code 31, -18dB	DP19: Code 28, -19dB
	de 0x19,0x16		;DP20: Code 25, -20dB	DP21: Code 22, -21dB
	de 0x13,0x11		;DP22: Code 19, -22dB	DP23: Code 17, -23dB
	de 0x0F,0x0D		;DP24: Code 15, -24dB	DP25: Code 13, -25dB
	de 0x0C,0x0A		;DP26: Code 12, -26dB	DP27: Code 10, -27dB
	de 0x07,0x05		;DP28: Code 7, -30dB	DP29: Code 5, -33dB
	de 0x03,0x02		;DP30: Code 3, -36dB	DP31: Code 2, -39dB
	de 0x01,0x00		;DP32: Code 1, -42dB	DP33: Code 0, -48dB

	ORG 0x7E22			;Volumen Tabelle fr Anzeige in dB

	de 0x00,0x01		;DP0: Code 255, -0dB	DP1: Code 227, -1dB
	de 0x02,0x03		;DP2: Code 202, -2dB	DP3: Code 180, -3dB
	de 0x04,0x05		;DP4: Code 160, -4dB	DP5: Code 143, -5dB
	de 0x06,0x07		;DP6: Code 127, -6dB	DP7: Code 113, -7dB
	de 0x08,0x09		;DP8: Code 101, -8dB	DP9: Code 90, -9dB
	de 0x0A,0x0B		;DP10: Code 80, -10dB	DP11: Code 71, -11dB
	de 0x0C,0x0D		;DP12: Code 63, -12dB	DP13: Code 56, -13dB
	de 0x0E,0x0F		;DP14: Code 50, -14dB	DP15: Code 45, -15dB
	de 0x10,0x11		;DP16: Code 40, -16dB	DP17: Code 35, -17dB
	de 0x12,0x13		;DP18: Code 31, -18dB	DP19: Code 28, -19dB
	de 0x14,0x15		;DP20: Code 25, -20dB	DP21: Code 22, -21dB
	de 0x16,0x17		;DP22: Code 19, -22dB	DP23: Code 17, -23dB
	de 0x18,0x19		;DP24: Code 15, -24dB	DP25: Code 13, -25dB
	de 0x1A,0x1B		;DP26: Code 12, -26dB	DP27: Code 10, -27dB
	de 0x1E,0x21		;DP28: Code 7, -30dB	DP29: Code 5, -33dB
	de 0x24,0x27		;DP30: Code 3, -36dB	DP31: Code 2, -39dB
	de 0x2A,0x30		;DP32: Code 1, -42dB	DP33: Code 0, -48dB

	ORG 0x7E44			;Tabellen fr deltaT Timer1 und deltaT/2 Timer0

	de 0x0C,0x12		;DP0:  dt = 1ms,, 					DP1:  dt = 2ms,
	de 0x18,0x1E		;DP2:  dt = 5ms,, 					DP3:  dt = 10ms,
	de 0x24,0x2A		;DP4:  dt = 20ms,, 					DP5:  dt = 50ms,
	de 0x30,0x36		;DP6:  dt = 100ms,, 				DP7:  dt = 200ms,
	de 0x3C,0x42		;DP8:  dt = 500ms,, 				DP9:  dt = 1000ms,
	de 0x48,0x48		;DP10:  dt = 2000ms,, 				DP11: 
	de 0x01,0xE8		;DP12:  dt = 1ms,WOB_ZEIT_H, 		DP13:  dt = 1ms,WOB_ZEIT_LN
	de 0x02,0x00		;DP14:  dt = 1ms,WOB_LOOP, 			DP15:  dt = 1ms,TMR0, H-Byte
	de 0x0B,0xB8		;DP16:  dt = 1ms,TMR0, M-Byte, 		DP17:  dt = 1ms,TMR0, L-Byte
	de 0x01,0xD1		;DP18:  dt = 2ms,WOB_ZEIT_H, 		DP19:  dt = 2ms,WOB_ZEIT_LN
	de 0x01,0x00		;DP20:  dt = 2ms,WOB_LOOP, 			DP21:  dt = 2ms,TMR0, H-Byte
	de 0x17,0x70		;DP22:  dt = 2ms,TMR0, M-Byte, 		DP23:  dt = 2ms,TMR0, L-Byte
	de 0x01,0x8A		;DP24:  dt = 5ms,WOB_ZEIT_H, 		DP25:  dt = 5ms,WOB_ZEIT_LN
	de 0x05,0x00		;DP26:  dt = 5ms,WOB_LOOP, 			DP27:  dt = 5ms,TMR0, H-Byte
	de 0x3A,0x98		;DP28:  dt = 5ms,TMR0, M-Byte, 		DP29:  dt = 5ms,TMR0, L-Byte
	de 0x01,0x15		;DP30:  dt = 10ms,WOB_ZEIT_H, 		DP31:  dt = 10ms,WOB_ZEIT_LN
	de 0x02,0x00		;DP32:  dt = 10ms,WOB_LOOP, 		DP33:  dt = 10ms,TMR0, H-Byte
	de 0x75,0x30		;DP34:  dt = 10ms,TMR0, M-Byte, 	DP35:  dt = 10ms,TMR0, L-Byte
	de 0x02,0x2B		;DP36:  dt = 20ms,WOB_ZEIT_H, 		DP37:  dt = 20ms,WOB_ZEIT_LN
	de 0x01,0x00		;DP38:  dt = 20ms,WOB_LOOP, 		DP39:  dt = 20ms,TMR0, H-Byte
	de 0xEA,0x60		;DP40:  dt = 20ms,TMR0, M-Byte, 	DP41:  dt = 20ms,TMR0, L-Byte
	de 0x05,0x6C		;DP42:  dt = 50ms,WOB_ZEIT_H, 		DP43:  dt = 50ms,WOB_ZEIT_LN
	de 0x01,0x02		;DP44:  dt = 50ms,WOB_LOOP, 		DP45:  dt = 50ms,TMR0, H-Byte
	de 0x49,0xF0		;DP46:  dt = 50ms,TMR0, M-Byte, 	DP47:  dt = 50ms,TMR0, L-Byte
	de 0x0A,0xD8		;DP48:  dt = 100ms,WOB_ZEIT_H, 		DP49:  dt = 100ms,WOB_ZEIT_LN
	de 0x01,0x04		;DP50:  dt = 100ms,WOB_LOOP, 		DP51:  dt = 100ms,TMR0, H-Byte
	de 0x93,0xE0		;DP52:  dt = 100ms,TMR0, M-Byte, 	DP53:  dt = 100ms,TMR0, L-Byte
	de 0x13,0xB0		;DP54:  dt = 200ms,WOB_ZEIT_H, 		DP55:  dt = 200ms,WOB_ZEIT_LN
	de 0x02,0x09		;DP56:  dt = 200ms,WOB_LOOP, 		DP57:  dt = 200ms,TMR0, H-Byte
	de 0x27,0xC0		;DP58:  dt = 200ms,TMR0, M-Byte, 	DP59:  dt = 200ms,TMR0, L-Byte
	de 0x2E,0x39		;DP60:  dt = 500ms,WOB_ZEIT_H, 		DP61:  dt = 500ms,WOB_ZEIT_LN
	de 0x01,0x16		;DP62:  dt = 500ms,WOB_LOOP, 		DP63:  dt = 500ms,TMR0, H-Byte
	de 0xE3,0x60		;DP64:  dt = 500ms,TMR0, M-Byte, 	DP65:  dt = 500ms,TMR0, L-Byte
	de 0x5C,0x72		;DP66:  dt = 1000ms,WOB_ZEIT_H, 	DP67:  dt = 1000ms,WOB_ZEIT_LN
	de 0x02,0x2D		;DP68:  dt = 1000ms,WOB_LOOP, 		DP69:  dt = 1000ms,TMR0, H-Byte
	de 0xC6,0xC0		;DP70:  dt = 1000ms,TMR0, M-Byte, 	DP71:  dt = 1000ms,TMR0, L-Byte
	de 0xB8,0xE5		;DP72:  dt = 2000ms,WOB_ZEIT_H, 	DP73:  dt = 2000ms,WOB_ZEIT_LN
	de 0x01,0x5B		;DP74:  dt = 2000ms,WOB_LOOP, 		DP75:  dt = 2000ms,TMR0, H-Byte
	de 0x8D,0x80		;DP76:  dt = 2000ms,TMR0, M-Byte, 	DP77:  dt = 2000ms,TMR0, L-Byte



	ORG 0x7F00			;Sinustabelle

	de 0x80,0x81		;Datenpunkt 0, 1
	de 0x82,0x82		;Datenpunkt 2, 3
	de 0x83,0x84		;Datenpunkt 4, 5
	de 0x85,0x85		;Datenpunkt 6, 7
	de 0x86,0x87		;Datenpunkt 8, 9
	de 0x88,0x89		;Datenpunkt 10, 11
	de 0x89,0x8A		;Datenpunkt 12, 13
	de 0x8B,0x8C		;Datenpunkt 14, 15
	de 0x8C,0x8D		;Datenpunkt 16, 17
	de 0x8E,0x8F		;Datenpunkt 18, 19
	de 0x90,0x90		;Datenpunkt 20, 21
	de 0x91,0x92		;Datenpunkt 22, 23
	de 0x93,0x93		;Datenpunkt 24, 25
	de 0x94,0x95		;Datenpunkt 26, 27
	de 0x96,0x97		;Datenpunkt 28, 29
	de 0x97,0x98		;Datenpunkt 30, 31
	de 0x99,0x9A		;Datenpunkt 32, 33
	de 0x9A,0x9B		;Datenpunkt 34, 35
	de 0x9C,0x9D		;Datenpunkt 36, 37
	de 0x9D,0x9E		;Datenpunkt 38, 39
	de 0x9F,0xA0		;Datenpunkt 40, 41
	de 0xA0,0xA1		;Datenpunkt 42, 43
	de 0xA2,0xA3		;Datenpunkt 44, 45
	de 0xA4,0xA4		;Datenpunkt 46, 47
	de 0xA5,0xA6		;Datenpunkt 48, 49
	de 0xA7,0xA7		;Datenpunkt 50, 51
	de 0xA8,0xA9		;Datenpunkt 52, 53
	de 0xA9,0xAA		;Datenpunkt 54, 55
	de 0xAB,0xAC		;Datenpunkt 56, 57
	de 0xAC,0xAD		;Datenpunkt 58, 59
	de 0xAE,0xAF		;Datenpunkt 60, 61
	de 0xAF,0xB0		;Datenpunkt 62, 63
	de 0xB1,0xB2		;Datenpunkt 64, 65
	de 0xB2,0xB3		;Datenpunkt 66, 67
	de 0xB4,0xB4		;Datenpunkt 68, 69
	de 0xB5,0xB6		;Datenpunkt 70, 71
	de 0xB6,0xB7		;Datenpunkt 72, 73
	de 0xB8,0xB9		;Datenpunkt 74, 75
	de 0xB9,0xBA		;Datenpunkt 76, 77
	de 0xBB,0xBB		;Datenpunkt 78, 79
	de 0xBC,0xBD		;Datenpunkt 80, 81
	de 0xBD,0xBE		;Datenpunkt 82, 83
	de 0xBF,0xC0		;Datenpunkt 84, 85
	de 0xC0,0xC1		;Datenpunkt 86, 87
	de 0xC2,0xC2		;Datenpunkt 88, 89
	de 0xC3,0xC4		;Datenpunkt 90, 91
	de 0xC4,0xC5		;Datenpunkt 92, 93
	de 0xC5,0xC6		;Datenpunkt 94, 95
	de 0xC7,0xC7		;Datenpunkt 96, 97
	de 0xC8,0xC9		;Datenpunkt 98, 99
	de 0xC9,0xCA		;Datenpunkt 100, 101
	de 0xCB,0xCB		;Datenpunkt 102, 103
	de 0xCC,0xCD		;Datenpunkt 104, 105
	de 0xCD,0xCE		;Datenpunkt 106, 107
	de 0xCE,0xCF		;Datenpunkt 108, 109
	de 0xD0,0xD0		;Datenpunkt 110, 111
	de 0xD1,0xD1		;Datenpunkt 112, 113
	de 0xD2,0xD3		;Datenpunkt 114, 115
	de 0xD3,0xD4		;Datenpunkt 116, 117
	de 0xD4,0xD5		;Datenpunkt 118, 119
	de 0xD6,0xD6		;Datenpunkt 120, 121
	de 0xD7,0xD7		;Datenpunkt 122, 123
	de 0xD8,0xD8		;Datenpunkt 124, 125
	de 0xD9,0xDA		;Datenpunkt 126, 127
	de 0xDA,0xDB		;Datenpunkt 128, 129
	de 0xDB,0xDC		;Datenpunkt 130, 131
	de 0xDC,0xDD		;Datenpunkt 132, 133
	de 0xDD,0xDE		;Datenpunkt 134, 135
	de 0xDE,0xDF		;Datenpunkt 136, 137
	de 0xDF,0xE0		;Datenpunkt 138, 139
	de 0xE0,0xE1		;Datenpunkt 140, 141
	de 0xE1,0xE2		;Datenpunkt 142, 143
	de 0xE2,0xE3		;Datenpunkt 144, 145
	de 0xE3,0xE4		;Datenpunkt 146, 147
	de 0xE4,0xE5		;Datenpunkt 148, 149
	de 0xE5,0xE6		;Datenpunkt 150, 151
	de 0xE6,0xE7		;Datenpunkt 152, 153
	de 0xE7,0xE8		;Datenpunkt 154, 155
	de 0xE8,0xE9		;Datenpunkt 156, 157
	de 0xE9,0xE9		;Datenpunkt 158, 159
	de 0xEA,0xEA		;Datenpunkt 160, 161
	de 0xEB,0xEB		;Datenpunkt 162, 163
	de 0xEC,0xEC		;Datenpunkt 164, 165
	de 0xEC,0xED		;Datenpunkt 166, 167
	de 0xED,0xEE		;Datenpunkt 168, 169
	de 0xEE,0xEE		;Datenpunkt 170, 171
	de 0xEF,0xEF		;Datenpunkt 172, 173
	de 0xF0,0xF0		;Datenpunkt 174, 175
	de 0xF0,0xF1		;Datenpunkt 176, 177
	de 0xF1,0xF1		;Datenpunkt 178, 179
	de 0xF2,0xF2		;Datenpunkt 180, 181
	de 0xF2,0xF3		;Datenpunkt 182, 183
	de 0xF3,0xF3		;Datenpunkt 184, 185
	de 0xF4,0xF4		;Datenpunkt 186, 187
	de 0xF4,0xF5		;Datenpunkt 188, 189
	de 0xF5,0xF5		;Datenpunkt 190, 191
	de 0xF6,0xF6		;Datenpunkt 192, 193
	de 0xF6,0xF6		;Datenpunkt 194, 195
	de 0xF7,0xF7		;Datenpunkt 196, 197
	de 0xF7,0xF8		;Datenpunkt 198, 199
	de 0xF8,0xF8		;Datenpunkt 200, 201
	de 0xF8,0xF9		;Datenpunkt 202, 203
	de 0xF9,0xF9		;Datenpunkt 204, 205
	de 0xF9,0xF9		;Datenpunkt 206, 207
	de 0xFA,0xFA		;Datenpunkt 208, 209
	de 0xFA,0xFA		;Datenpunkt 210, 211
	de 0xFB,0xFB		;Datenpunkt 212, 213
	de 0xFB,0xFB		;Datenpunkt 214, 215
	de 0xFB,0xFC		;Datenpunkt 216, 217
	de 0xFC,0xFC		;Datenpunkt 218, 219
	de 0xFC,0xFC		;Datenpunkt 220, 221
	de 0xFC,0xFD		;Datenpunkt 222, 223
	de 0xFD,0xFD		;Datenpunkt 224, 225
	de 0xFD,0xFD		;Datenpunkt 226, 227
	de 0xFD,0xFD		;Datenpunkt 228, 229
	de 0xFD,0xFE		;Datenpunkt 230, 231
	de 0xFE,0xFE		;Datenpunkt 232, 233
	de 0xFE,0xFE		;Datenpunkt 234, 235
	de 0xFE,0xFE		;Datenpunkt 236, 237
	de 0xFE,0xFE		;Datenpunkt 238, 239
	de 0xFE,0xFF		;Datenpunkt 240, 241
	de 0xFF,0xFF		;Datenpunkt 242, 243
	de 0xFF,0xFF		;Datenpunkt 244, 245
	de 0xFF,0xFF		;Datenpunkt 246, 247
	de 0xFF,0xFF		;Datenpunkt 248, 249
	de 0xFF,0xFF		;Datenpunkt 250, 251
	de 0xFF,0xFF		;Datenpunkt 252, 253
	de 0xFF,0xFF		;Datenpunkt 254, 255

          END